@@ -109,6 +109,8 @@ void usage()
 
 int main( int argc, char *argv[] )
 {
+    cout << "Firewall Builder GUI " VERSION << endl << flush;
+
     filename = "";
     print_output_file_name = "";
     fwbdebug = 0;
@@ -180,6 +182,7 @@ int main( int argc, char *argv[] )
     if (fwbdebug) qDebug("Reading settings ...");
     st = new FWBSettings();
     st->init(force_first_time_run_flag);
+    FWObject::setDataDir(st->getDataDir().toUtf8().constData());
     if (fwbdebug) qDebug("done");
 
     wfl = new UserWorkflow();
