@@ -726,7 +726,12 @@ void IPTImporter::pushPolicyRule()
 
     if (target=="QUEUE")    action = PolicyRule::Pipe;
 
-    if (target=="CLASSIFY") action = PolicyRule::Classify;
+    if (target=="CLASSIFY")   // #2367
+    {
+        action = PolicyRule::Continue;
+        rule->setClassification(true);
+        ropt->setStr("classify_str", action_params["set_class"]);
+    }
 
     if (target=="LOG")
     {
@@ -782,6 +787,7 @@ void IPTImporter::pushPolicyRule()
     {
         action = PolicyRule::Continue;
         rule->setLogging(true);
+
         fwopt->setBool("use_ULOG", true);
         QString log_prefix = action_params["log_prefix"].c_str();
         log_prefix.replace("\"", "");
@@ -790,7 +796,9 @@ void IPTImporter::pushPolicyRule()
 
     if (target=="MARK")
     {
-        action = PolicyRule::Tag;
+        action = PolicyRule::Continue;
+        rule->setTagging(true);
+
         last_mark_rule = rule;
         
         ObjectSignature sig(error_tracker);
@@ -803,7 +811,8 @@ void IPTImporter::pushPolicyRule()
 
     if (target=="ROUTE")
     {
-        action = PolicyRule::Route;
+        action = PolicyRule::Continue;
+        rule->setRouting(true);
 
         if (!action_params["route_iif"].empty())
             newInterface(action_params["route_iif"]);
@@ -1057,7 +1066,8 @@ void IPTImporter::pushPolicyRule()
 
         if (current_table == "mangle")
         {
-            if (current_chain == "POSTROUTING" || current_chain == "FORWARD")
+            if ( ! rule->getClassification() &&
+                 (current_chain == "POSTROUTING" || current_chain == "FORWARD"))
             {
                 QString err = QObject::tr(
                     "Fwbuilder can not reproduce iptables rule in "
@@ -1177,10 +1187,26 @@ void IPTImporter::pushPolicyRule()
             current_rule, QString::fromUtf8(rule_comment.c_str()));
     }
 
+    if (error_tracker->hasWarnings())
+    {
+        QStringList warn = error_tracker->getWarnings();
+        // parser errors and warnings are added to the log by
+        // PFCfgParser::reportError() and PFCfgParser::reportWarning()
+        // so we dont need to add them again here
+        foreach(QString w, warn)
+        {
+            if (!w.startsWith("Parser warning:")) addMessageToLog("Warning: " + w);
+        }
+        markCurrentRuleBad();
+    }
+
     if (error_tracker->hasErrors())
     {
         QStringList err = error_tracker->getErrors();
-        addMessageToLog("Error: " + err.join("\n"));
+        foreach(QString e, err)
+        {
+            if (!e.startsWith("Parser error:")) addMessageToLog("Error: " + e);
+        }
         markCurrentRuleBad();
     }
 
