@@ -38,6 +38,7 @@
 #include "fwbuilder/Network.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/AddressRange.h"
+#include "fwbuilder/AttachedNetworks.h"
 
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/RuleElement.h"
@@ -57,6 +58,7 @@
 
 #include <QStringList>
 #include <QRegExp>
+#include <QtDebug>
 
 #include <iostream>
 #include <iomanip>
@@ -371,45 +373,24 @@ string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
 
     FWOptions *ruleopt =rule->getOptionsObject();
 
-    if (target==".CUSTOM")
+    if (rule->getTagging())
     {
-        ostr << " " << ruleopt->getStr("custom_str");
-        return ostr.str();
-    }
-
-    // there is no ULOG for ip6tables yet
-    if (!ipt_comp->ipv6 && compiler->getCachedFwOpt()->getBool("use_ULOG") &&
-         target=="LOG") target="ULOG";
-
-    if (target==".CONTINUE") // not a real target !
-        return ostr.str();
-
-    ostr << " -j " << target << " ";
-
-    if (target=="REJECT")
-      ostr << _printActionOnReject(rule);
-
-    if (target=="LOG" || target=="ULOG")    
-        ostr << _printLogParameters(rule);
-    
-    if (target=="MARK")
-    {
-//        ostr << " --set-mark " << ruleopt->getStr("tagvalue");
+        ostr << " -j MARK";
         ostr << " --set-mark " << rule->getTagValue();
+        return ostr.str();
     }
 
-    if (target=="CONNMARK")
-    {
-        ostr << ruleopt->getStr("CONNMARK_arg");
-    }
-
-    if (target=="CLASSIFY")
+    if (rule->getClassification())
     {
+        ostr << " -j CLASSIFY";
         ostr << " --set-class " << ruleopt->getStr("classify_str");
+        return ostr.str();
     }
 
-    if (target=="ROUTE")
+    if (rule->getRouting())
     {
+        ostr << " -j ROUTE";
+
         string a;
         a = ruleopt->getStr("ipt_iif");
         if (!a.empty()) ostr << " --iif " << a;
@@ -425,6 +406,35 @@ string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
 
         c = ruleopt->getBool("ipt_tee");
         if (c) ostr << " --tee";
+
+        return ostr.str();
+    }
+
+    if (target==".CUSTOM")
+    {
+        ostr << " " << ruleopt->getStr("custom_str");
+        return ostr.str();
+    }
+
+    if (target==".CONTINUE") // not a real target !
+        return ostr.str();
+
+
+    // there is no ULOG for ip6tables yet
+    if (!ipt_comp->ipv6 && compiler->getCachedFwOpt()->getBool("use_ULOG") &&
+         target=="LOG") target="ULOG";
+
+    ostr << " -j " << target << " ";
+
+    if (target=="REJECT")
+      ostr << _printActionOnReject(rule);
+
+    if (target=="LOG" || target=="ULOG")    
+        ostr << _printLogParameters(rule);
+    
+    if (target=="CONNMARK")
+    {
+        ostr << ruleopt->getStr("CONNMARK_arg");
     }
 
     return ostr.str();
@@ -608,9 +618,7 @@ string PolicyCompiler_ipt::PrintRule::_printLogPrefix(PolicyRule *rule,
 {
     FWObject *ruleset = rule->getParent();
 
-    char action[64];
-    strncpy(action,rule->getStr("stored_action").c_str(),sizeof(action));
-    for (char *cptr=action; *cptr; cptr++)  *cptr=toupper(*cptr);
+    QString action = QString(rule->getStr("stored_action").c_str()).toUpper();
 
     RuleElementItf *itf_re = rule->getItf(); assert(itf_re!=NULL);
     FWObject *rule_iface = FWObjectReference::getObject(itf_re->front());
@@ -629,7 +637,7 @@ string PolicyCompiler_ipt::PrintRule::_printLogPrefix(PolicyRule *rule,
     s1 << pos;
 
     return _printLogPrefix(s1.str(),
-                           action,
+                           action.toStdString(),
                            rule_iface_name,
                            rule->getStr("ipt_chain"),
                            ruleset->getName(),
@@ -1239,6 +1247,14 @@ string PolicyCompiler_ipt::PrintRule::_printAddr(Address  *o)
         {
             return atrt->getSourceName();
         }
+
+        if (atrt->getSubstitutionTypeName()==AttachedNetworks::TYPENAME)
+        {
+            ostr << "$i_" << atrt->getSourceName() << "_network";
+            return ostr.str();
+        }
+
+
         // at this time we only support two types of MultiAddress
         // objects: AddressTable and DNSName. Both should be converted
         // to MultiAddressRunTime at this point. If we get some other
