@@ -75,6 +75,7 @@ protected:
         std::string                    my_table;
 
         std::map<std::string, int>     tmp_chain_no;
+        std::map<std::string, int>     rule_chain_no;
         std::map<std::string, int>     chain_usage_counter;
         std::map<std::string, std::string> ipset_tables;
 
@@ -161,6 +162,29 @@ protected:
          */
         DECLARE_POLICY_RULE_PROCESSOR(checkActionInMangleTable);
 
+        /**
+         * Rule with options "Tag", "Classify" and "Route" may need to
+         * placed in the same chain (PREROUTING or POSTROUTING since
+         * ROUTE can go into both), but we can't process it if it also
+         * has action Accept (or any other action that is not
+         * "Continue"). This is because we have to split the rule
+         * before the chain is determined and so we end up with two
+         * user defined chains, one for CLASSIFY or MARK and another
+         * for ROUTE, each of them ending with ACCEPT. This means one
+         * of the chains accepts the packet and the other never sees
+         * it.  We may eventually optimize this and find a way to
+         * generate code for rules like this in one chain, but since
+         * this is rarely used case, we'll abort for now.
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(checkForUnsupportedCombinationsInMangle);
+
+        /**
+         * See #2401 Option "Route" (used to be action "Route") is
+         * deprecated.  Iptables target ROUTE is not included in major
+         * distributions (Debian, Ubuntu, Fedora, CentOS).
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(deprecateOptionRoute);
+
         /**
          * adds few predefined (or "builtin") rules on top of the policy
          */
@@ -177,7 +201,11 @@ protected:
 	/**
 	 * need to duplicate original action of this rule. We use this
 	 * information later to decide whether we need to use "-m
-	 * state --state new"
+	 * state --state new". Also this rule processor stores a copy
+	 * of flags getRouting(), getTagging() and getClassification()
+	 * which we use in setChainPreroutingForTag() and
+         * setChainPostroutingForTag() to place rules that do tagging in
+         * chain PREROUTING and similar purposes.
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(storeAction);
 
@@ -196,6 +224,7 @@ protected:
 
 	/**
 	 * set target and chain in case of route rules
+         * Deprecated beginning with 4.3.0. To be removed in future versions.
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(Route);
 
@@ -210,6 +239,29 @@ protected:
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(Logging2);
 
+	/**
+	 * splits rule if it has more than option Tag, Classify or Route
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(splitIfTagClassifyOrRoute);
+
+	/**
+	 * clears options Tag and Classify in filter table
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(clearTagClassifyInFilter);
+
+	/**
+	 * turns off logging in rules with options Tag, Classify or
+         * Route in table mangle
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(clearLogInMangle);
+
+	/**
+	 * switches action to Continue in rules with options Tag,
+         * Classify in mangle table. We deal with other actions in
+         * table filter.
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(clearActionInTagClassifyIfMangle);
+
 
 	/**
 	 * this processor checks if the rule is associated with an
@@ -231,26 +283,45 @@ protected:
         DECLARE_POLICY_RULE_PROCESSOR(convertAnyToNotFWForShadowing);
 
 	/**
-	 * processes rules with negation in Itf if it holds only one object
+	 * processes rules with negation in Src if it holds only one
+         * object.  Similar to PolicyCompiler::singleObjectNegationSrc
+         * but takes into account AddressTable objects if we compile
+         * with support for ipset module
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(singleItfNegation);
+        class SingleRENegation : public PolicyRuleProcessor
+        {
+            std::string type_name;
+            void processSingleObjectNegationInRE(libfwbuilder::FWObject *obj,
+                                                 libfwbuilder::RuleElement *re);
+            public:
+                SingleRENegation(const std::string &name,
+                     const std::string &tn) : PolicyRuleProcessor(name)
+            {
+                type_name = tn;
+            }
+            virtual bool processNext();
+        };
 
-	/**
-	 * processes rules with negation in Src if it holds only one object
-	 */
-        DECLARE_POLICY_RULE_PROCESSOR(singleSrcNegation);
-
-	/**
-	 * processes rules with negation in Dst if it holds only one object
-	 */
-        DECLARE_POLICY_RULE_PROCESSOR(singleDstNegation);
-
-	/**
-	 * processes rules with negation in Srv if it holds only one object
-	 */
-        DECLARE_POLICY_RULE_PROCESSOR(singleSrvNegation);
+        class SingleSrcNegation : public SingleRENegation
+        {
+            public:
+            SingleSrcNegation(const std::string &name) :
+                SingleRENegation(name, libfwbuilder::RuleElementSrc::TYPENAME) {}
+        };
 
+        class SingleDstNegation : public SingleRENegation
+        {
+            public:
+            SingleDstNegation(const std::string &name) :
+                SingleRENegation(name, libfwbuilder::RuleElementDst::TYPENAME) {}
+        };
 
+        class SingleSrvNegation : public SingleRENegation
+        {
+            public:
+            SingleSrvNegation(const std::string &name) :
+                SingleRENegation(name, libfwbuilder::RuleElementSrv::TYPENAME) {}
+        };
 
 	/**
 	 * processes rules with negation in Src
@@ -656,13 +727,6 @@ protected:
          */
         DECLARE_POLICY_RULE_PROCESSOR(decideOnChainForClassify);
 
-        /**
-         * find non-terminating targets (such as MARK and
-         * CLASSIFY). Put such rule in a separate chain and pass
-         * control to it using "-g"
-         */
-        DECLARE_POLICY_RULE_PROCESSOR(splitNonTerminatingTargets);
-
         /**
          * drop rules with terminating targets. Used as part of the
          * shadowing detection for non-terminating rules in the mangle
