@@ -58,11 +58,13 @@ using namespace libfwbuilder;
 
 const char *FWObject::TYPENAME={"UNDEF"};
 string FWObject::NOT_FOUND="";
+string FWObject::dataDir;
 
 
 //#define FWB_DEBUG
 //#define TI_DEBUG
 
+
 void FWObject::fromXML(xmlNodePtr root) throw(FWException)
 {
     assert(root!=NULL);
@@ -89,6 +91,25 @@ void FWObject::fromXML(xmlNodePtr root) throw(FWException)
         FREEXMLBUFF(n);
     }
 
+    n = FROMXMLCAST(xmlGetProp(root, TOXMLCAST("keywords")));
+    if (n != 0) {
+        keywords = stringToSet(n);
+        dbroot->keywords.insert(keywords.begin(), keywords.end());
+        FREEXMLBUFF(n);
+    }
+
+    n = FROMXMLCAST(xmlGetProp(root, TOXMLCAST("subfolders")));
+    if (n != 0) {
+        setStr("subfolders", n);
+        FREEXMLBUFF(n);
+    }
+
+    n = FROMXMLCAST(xmlGetProp(root, TOXMLCAST("folder")));
+    if (n != 0) {
+        setStr("folder", n);
+        FREEXMLBUFF(n);
+    }
+
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("ro")));
     if(n)
     {
@@ -96,7 +117,7 @@ void FWObject::fromXML(xmlNodePtr root) throw(FWException)
         FREEXMLBUFF(n);
     }
 
-    ref_counter = 0;
+    // ref_counter = 0;
     FWObjectDatabase *dbr = getRoot();
 
     for (xmlNodePtr cur=root->xmlChildrenNode; cur; cur=cur->next)
@@ -108,6 +129,7 @@ void FWObject::fromXML(xmlNodePtr root) throw(FWException)
             {
                 /* Add w/o validation. Trust XML to do that */
 		add(o, false);
+
                 try
                 {
                     o->fromXML(cur);
@@ -148,12 +170,18 @@ xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children)
             STRTOXMLCAST(s_id));
     }
 
+    if (!keywords.empty()) {
+        xmlNewProp(me, TOXMLCAST("keywords"),
+                   STRTOXMLCAST(setToString(keywords)));
+    }
+
     for(map<string, string>::const_iterator i=data.begin(); i!=data.end(); ++i) 
     {
         const string &name  = (*i).first;
         const string &value = (*i).second;
 
         if (name[0]=='.') continue;
+        if (name == "folder" && value.empty()) continue;
 
         xmlNewProp(me, STRTOXMLCAST(name), STRTOXMLCAST(value));
     }
@@ -336,6 +364,9 @@ bool FWObject::cmp(const FWObject *obj, bool recursive) throw(FWException)
         if (j->second!=value) return false;
     }
 
+    if (keywords.empty() != obj->keywords.empty() || keywords != obj->keywords)
+        return false;
+
     if (recursive)
     {
         if (size()!=obj->size())  return false;
@@ -408,6 +439,7 @@ FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id)
     if(!o1)
         throw FWException(string("Error creating object with type: ")+
                           x->getTypeName());
+
     // This adds with validation
     add(o1);
 
@@ -438,6 +470,12 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id)
     data = x->data;
     private_data = x->private_data;
 
+    keywords = x->keywords;
+    set<string>::const_iterator iter;
+    for (iter = keywords.begin(); iter != keywords.end(); ++iter) {
+        dbroot->keywords.insert(*iter);
+    }
+
     setReadOnly(false);
 
     if (!preserve_id)
@@ -516,6 +554,15 @@ FWObjectDatabase* FWObject::getRoot() const
     return dbroot;
 }
 
+int FWObject::getDistanceFromRoot() const
+{
+    int count = 0;
+    for (FWObject *obj = getParent(); obj != 0; obj = obj->getParent()) {
+        count++;
+    }
+    return count;
+}
+
 class pathAccumulator : public string
 {
     public:
@@ -609,7 +656,7 @@ void FWObject::remStr(const string &name)
 
 void FWObject::setStr(const string &name, const string &val)
 {
-    if (name[0]!='.') checkReadOnly();
+    if (name[0]!='.' && name != "folder") checkReadOnly();
     string old_val = data[name];
     if (old_val != val)
     {
@@ -1344,8 +1391,16 @@ bool FWObject::isReadOnly()
 
 void FWObject::checkReadOnly() throw(FWException)
 {
-    if (isReadOnly()) throw FWException(
-        string("Attempt to modify read-only object ")+getName());
+    if (isReadOnly() && ! getRoot()->getIgnoreReadOnlyFlag())
+        throw FWException(string("Attempt to modify read-only object ")+getName());
+}
+
+
+
+
+FWObjectTypedChildIterator::FWObjectTypedChildIterator() :
+    type_name(), real_iterator()
+{
 }
 
 FWObjectTypedChildIterator::FWObjectTypedChildIterator(
@@ -1357,11 +1412,18 @@ FWObjectTypedChildIterator::FWObjectTypedChildIterator(
 
 FWObjectTypedChildIterator::FWObjectTypedChildIterator(
     const FWObject *o, const std::string &_type_name)
+{
+    init(o, _type_name);
+}
+
+void FWObjectTypedChildIterator::init(
+    const FWObject *o, const std::string &_type_name)
 {
     type_name     = _type_name ;
     _end          = o->end()   ;
-    real_iterator = o->begin() ;
+
     // position to first element
+    real_iterator = o->begin() ;
     while(real_iterator!=_end && (*real_iterator)->getTypeName()!=type_name)
         real_iterator++;
     _begin = real_iterator;
@@ -1395,15 +1457,18 @@ FWObjectTypedChildIterator& FWObjectTypedChildIterator::operator++()
         return *this;
 }
 
+/*
+ * if iterator points to the first element in the list, then operator--() 
+ * should move it and make it point to end()
+ */
 FWObjectTypedChildIterator& FWObjectTypedChildIterator::operator--() 
 { 
-    if(real_iterator==_begin)
+    if(real_iterator==_end)
         return *this;
     do
     {
         real_iterator--;
-    } while(real_iterator!=_begin &&
-           (real_iterator==_end || (*real_iterator)->getTypeName()!=type_name));
+    } while(real_iterator!=_end && (*real_iterator)->getTypeName()!=type_name);
     return *this;
 }
 
@@ -1580,6 +1645,27 @@ void FWObject::_findDependencies_internal(FWObject *obj,
 
 bool FWObject::isPrimaryObject() const { return false; }
 
+const set<string> &FWObject::getAllKeywords()
+{
+    return dbroot->keywords;
+}
+
+void FWObject::addKeyword(const string &keyword)
+{
+    keywords.insert(keyword);
+    dbroot->keywords.insert(keyword);
+}
+
+void FWObject::removeKeyword(const string &keyword)
+{
+    keywords.erase(keyword);
+}
+
+void FWObject::clearKeywords()
+{
+    keywords.clear();
+}
+
 FWObjectNameCmpPredicate::FWObjectNameCmpPredicate(bool follow_refs)
 {
     follow_references = follow_refs;
