@@ -119,6 +119,9 @@ private:
     static std::string NOT_FOUND;
 
     time_t creation_time;
+    std::set<std::string> keywords;
+
+    static std::string dataDir;
 
 protected:
 
@@ -340,6 +343,7 @@ public:
      *   Returns root of the objects tree this object is part of
      */
     FWObjectDatabase* getRoot() const;
+    int getDistanceFromRoot() const;    /* How many parents does this have? */
 
     /**
      * Sets pointer to the database root
@@ -565,14 +569,22 @@ public:
     std::map<std::string, std::string>::const_iterator dataEnd()
     { return data.end();   }
 
+    const std::set<std::string> &getKeywords() { return keywords; }
+    const std::set<std::string> &getAllKeywords();
+    void addKeyword(const std::string &keyword);
+    void removeKeyword(const std::string &keyword);
+    void clearKeywords();
+
+    static std::string getDataDir() { return dataDir; }
+    static void setDataDir(const std::string &dir) { dataDir = dir; }
 };
 
 class FWObjectTypedChildIterator
 {
     public:
 
-    FWObjectTypedChildIterator(const FWObjectTypedChildIterator &o);
-    
+    FWObjectTypedChildIterator();
+    FWObjectTypedChildIterator(const FWObjectTypedChildIterator &o);   
     FWObjectTypedChildIterator(const FWObject *o, const std::string &_type_name);
     
     bool operator==(const FWObject::const_iterator& __x) const;
@@ -590,6 +602,8 @@ class FWObjectTypedChildIterator
     FWObject::const_iterator real_iterator;
     FWObject::const_iterator _begin;
     FWObject::const_iterator _end;
+
+    void init(const FWObject *o, const std::string &_type_name);
 };
  
 /**
@@ -664,10 +678,3 @@ class FWObjectFindPredicate
 }
 
 #endif // _FWOBJECT_HH
-
-
-
-
-
-
-
