@@ -56,12 +56,14 @@ namespace libfwbuilder
     // forward declarations for specialized create() methods
     class AddressRange;
     class AddressTable;
+    class AttachedNetworks;
     class Cluster;
     class StateSyncClusterGroup;
     class FailoverClusterGroup;
     class ClusterGroupOptions;
     class CustomService;
     class DNSName;
+    class DynamicGroup;
     class FWBDManagement;
     class FWIntervalReference;
     class FWObjectReference;
@@ -123,12 +125,14 @@ namespace libfwbuilder
     
     DECLARE_CREATE_OBJ_METHOD(AddressRange);
     DECLARE_CREATE_OBJ_METHOD(AddressTable);
+    DECLARE_CREATE_OBJ_METHOD(AttachedNetworks);
     DECLARE_CREATE_OBJ_METHOD(Cluster);
     DECLARE_CREATE_OBJ_METHOD(StateSyncClusterGroup);
     DECLARE_CREATE_OBJ_METHOD(FailoverClusterGroup);
     DECLARE_CREATE_OBJ_METHOD(ClusterGroupOptions);
     DECLARE_CREATE_OBJ_METHOD(CustomService);
     DECLARE_CREATE_OBJ_METHOD(DNSName);
+    DECLARE_CREATE_OBJ_METHOD(DynamicGroup);
     DECLARE_CREATE_OBJ_METHOD(FWBDManagement);
     DECLARE_CREATE_OBJ_METHOD(FWIntervalReference);
     DECLARE_CREATE_OBJ_METHOD(FWObjectReference);
@@ -249,7 +253,8 @@ protected:
         std::map<int, FWObject*> obj_index;
         int searchId;
         int predictable_id_tracker;
-
+        bool ignore_read_only;
+        
         void init_create_methods_table();
         void init_id_dict();
 
@@ -330,6 +335,13 @@ public:
          */
         void getIndexStats(int &index_size, int &hit_counter, int &miss_counter);
 
+        /**
+         * Some operations, such as object tree merging, should ignore
+         * read-only flag on individual objects.
+         */
+        bool getIgnoreReadOnlyFlag() { return ignore_read_only; }
+        void setIgnoreReadOnlyFlag(bool f) { ignore_read_only = f; }
+
         // --- XML import/export ---
     
         virtual void fromXML(xmlNodePtr xml_parent_node) throw(FWException);
@@ -378,7 +390,15 @@ public:
         void findObjectsInGroup(
             libfwbuilder::Group *g,
             std::set<libfwbuilder::FWObject *> &resset);
-    
+
+        /**
+         * We ignore read-only flag on individual objects when whole object
+         * tree is duplicated
+         */
+        virtual FWObject& duplicate(const FWObject *obj,
+                                    bool preserve_id = true) throw(FWException);
+
+        
         void recursivelyRemoveObjFromTree(FWObject* obj, bool remove_ref=false);
 
         /**
@@ -398,7 +418,6 @@ public:
          * This means returned object can be a parent for the copy of <source>.
          */
         FWObject* reproduceRelativePath(FWObject *lib, const FWObject *source);
-
     
         /**
          * fix references in children of obj according to the map_ids which
@@ -474,12 +493,14 @@ public:
 
     DECLARE_CREATE_OBJ_CLASS_METHOD(AddressRange);
     DECLARE_CREATE_OBJ_CLASS_METHOD(AddressTable);
+    DECLARE_CREATE_OBJ_CLASS_METHOD(AttachedNetworks);
     DECLARE_CREATE_OBJ_CLASS_METHOD(Cluster);
     DECLARE_CREATE_OBJ_CLASS_METHOD(StateSyncClusterGroup);
     DECLARE_CREATE_OBJ_CLASS_METHOD(FailoverClusterGroup);
     DECLARE_CREATE_OBJ_CLASS_METHOD(ClusterGroupOptions);
     DECLARE_CREATE_OBJ_CLASS_METHOD(CustomService);
     DECLARE_CREATE_OBJ_CLASS_METHOD(DNSName);
+    DECLARE_CREATE_OBJ_CLASS_METHOD(DynamicGroup);
     DECLARE_CREATE_OBJ_CLASS_METHOD(FWBDManagement);
     DECLARE_CREATE_OBJ_CLASS_METHOD(FWIntervalReference);
     DECLARE_CREATE_OBJ_CLASS_METHOD(FWObjectReference);
