@@ -6,8 +6,6 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id$
-
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
   of that license as published by the Free Software Foundation; either
@@ -74,6 +72,7 @@ using namespace std;
 int Compiler::prolog() 
 {
     temp = new Group();
+    temp->setName("Temp Group");
     fw->add(temp, false);
     return 0;
 }
@@ -138,6 +137,7 @@ void Compiler::_init(FWObjectDatabase *_db, Firewall *_fw)
 { 
     initialized = false;
     _cntr_ = 1; 
+    group_registry = NULL;
 
     temp_ruleset = NULL; 
 
@@ -328,6 +328,21 @@ void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
     }
 }
 
+/*
+ * Common interface to the operation of expanding of a group
+ * recursively. This just calls internal function
+ * _expand_group_recursive()
+ */
+void Compiler::expandGroup(FWObject *grp, list<FWObject*> &ol)
+{
+    for (FWObject::iterator i1=grp->begin(); i1!=grp->end(); ++i1)
+    {
+        FWObject *o = FWReference::getObject(*i1);
+        assert(o);
+        _expand_group_recursive(o, ol);
+    }
+}
+
 /**
  *   object 's' here is really src or dst or srv. Its children objects
  *   should all be references
@@ -335,12 +350,7 @@ void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
 void Compiler::expandGroupsInRuleElement(RuleElement *s)
 {
     list<FWObject*> cl;
-    for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1)
-    {
-        FWObject *o = FWReference::getObject(*i1);
-        assert(o);
-        _expand_group_recursive(o, cl);
-    }
+    expandGroup(s, cl);
 
     s->clearChildren();
     //s->setAnyElement();
@@ -349,6 +359,11 @@ void Compiler::expandGroupsInRuleElement(RuleElement *s)
 
     for(FWObject::iterator i2=cl.begin(); i2!=cl.end(); ++i2)
     {
+        if (!s->validateChild(*i2))
+            abort(s->getParent(),
+                  "Object '" + (*i2)->getName() +
+                  "' can not be used in rule element " + s->getTypeName());
+
         s->addRef( *i2 );
     }
 }
@@ -463,7 +478,8 @@ void Compiler::_expand_interface(Rule *rule,
  * we use physAddress only if Host option "use_mac_addr_filter" of the
  * parent Host object is true
  */
-    FWObject  *p = iface->getParentHost();
+    FWObject *p = Host::getParentHost(iface);
+    //FWObject *p = iface->getParentHost();
     Host *hp = Host::cast(p);
     if (hp==NULL) return;  // something is very broken
     FWOptions *hopt = hp->getOptionsObject();
@@ -521,8 +537,16 @@ void Compiler::_expand_interface(Rule *rule,
     }
 }
 
-bool compare_addresses(Address *a1, Address *a2)
+bool compare_addresses(FWObject *o1, FWObject *o2)
 {
+    Address *a1 = Address::cast(o1);
+    Address *a2 = Address::cast(o2);
+    if (a1 == NULL || a2 == NULL)
+    {
+        // one or both could be MultiAddress objects (e.g. DNSName)
+        return o1->getName() < o2->getName();
+    }
+
     const InetAddr *addr1 = a1->getAddressPtr();
     const InetAddr *addr2 = a2->getAddressPtr();
     if (addr1 == NULL) return true;
@@ -543,18 +567,18 @@ void Compiler::_expand_addr(Rule *rule, FWObject *s,
     list<FWObject*> cl;
     _expand_addr_recursive(rule, s, cl, expand_cluster_interfaces_fully);
     
-    list<Address*> expanded_addresses;
+    list<FWObject*> expanded_addresses;
     for (FWObject::iterator i=cl.begin(); i!=cl.end(); ++i) 
     {
-        expanded_addresses.push_back(Address::cast(*i));
+        expanded_addresses.push_back(*i);
     }
 
     expanded_addresses.sort(compare_addresses);
 
     s->clearChildren();
 
-    for (list<Address*>::iterator i1=expanded_addresses.begin();
-         i1!=expanded_addresses.end(); ++i1) 
+    for (list<FWObject*>::iterator i1=expanded_addresses.begin();
+         i1!=expanded_addresses.end(); ++i1)
     {
         s->addRef( *i1 );
     }
@@ -603,7 +627,14 @@ void Compiler::_expandAddressRanges(Rule *rule, FWObject *re)
                     h->setAddress(*(i->getAddressPtr()));
                     persistent_objects->add(h, false);
                     cl.push_back(h);
-                }
+
+                    // see GroupRegistry::registerGroupObject()
+                    if (group_registry != NULL)
+                    {
+                        group_registry->setGroupRegistryKey(
+                            h, group_registry->getGroupRegistryKey(aro));
+                    }
+               }
             }
 	} else
         {
@@ -862,12 +893,119 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
     return true;
 }
 
+/*
+ * This rule processor replaces firewall object in given rule element
+ * with run-time DNSName object with name "self" and source name (A
+ * record) set to "self". This is a trick in that when compliers see
+ * objects like that in a rule, they just put source name in the
+ * generated code verbatim. This is useful for firewall platforms that
+ * support keyword "self" (e.g. PF).
+ *
+ * Always call this RE after splitIfFirewallInSrc or splitIfFirewallInDst
+ */
+bool Compiler::ReplaceFirewallObjectWithSelfInRE::processNext()
+{
+    Rule *rule = prev_processor->getNextRule();
+    if (rule==NULL) return false;
+    RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
+
+    for (list<FWObject*>::iterator i1=re->begin(); i1!=re->end(); ++i1)
+    {
+        FWObject *obj = FWReference::getObject(*i1);
+        if (obj == compiler->fw)
+        {
+            DNSName *self = DNSName::cast(
+                compiler->persistent_objects->findObjectByName(
+                    DNSName::TYPENAME, "self"));
+            if (self == NULL)
+            {
+                self = compiler->dbcopy->createDNSName();
+                self->setName("self");
+                self->setRunTime(true);
+                self->setSourceName("self");
+                compiler->persistent_objects->add(self, false);
+            }
+
+            re->addRef(self);
+            re->removeRef(compiler->fw);
+            break;
+        }
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+bool Compiler::RegisterGroupsAndTablesInRE::processNext()
+{
+    Rule *rule = prev_processor->getNextRule();
+    if (rule==NULL) return false;
+
+    if (compiler->group_registry != NULL)
+    {
+        RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
+
+        for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+        {
+            FWObject *obj = FWReference::getObject(*i);
+            if (ObjectGroup::cast(obj)!=NULL && obj->size() > 0)
+            {
+                compiler->registerGroupObject(ObjectGroup::cast(obj));
+            }
+        }
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+void Compiler::registerGroupObject(ObjectGroup *grp)
+{
+    assert(group_registry!=NULL);
+    list<FWObject*> objects;
+    expandGroup(grp, objects);
+    group_registry->registerGroup(grp, objects);
+}
 
 bool Compiler::equalObj::operator()(FWObject *o)
 {
     return o->getId()==obj->getId();
 }
 
+bool Compiler::singleObjectNegation::processNext()
+{
+    Rule *rule = prev_processor->getNextRule(); if (rule==NULL) return false;
+
+    RuleElement *rel = RuleElement::cast(rule->getFirstByType(re_type));
+    assert(rel);
+
+    if (rel->getNeg() && rel->size()==1)
+    {
+        if (rel->getTypeName() == RuleElementItfInb::TYPENAME ||
+            rel->getTypeName() == RuleElementItfOutb::TYPENAME ||
+            rel->getTypeName() == RuleElementItf::TYPENAME )
+        {
+            rel->setNeg(false);
+            rel->setBool("single_object_negation", true);
+        } else
+        {
+            FWObject *o = rel->front();
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+            Address *reladdr = Address::cast(o);
+            if ( reladdr && reladdr->countInetAddresses(true)==1 &&
+                 !compiler->complexMatch(reladdr, compiler->fw)) 
+            {
+                rel->setNeg(false);
+                rel->setBool("single_object_negation", true);
+            }
+        }
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
 /*
  * Process negation in the "Interface" rule element. Scan objects in
  * this RE, replace cluster interfaces with interfaces of the member,
@@ -879,8 +1017,13 @@ bool Compiler::equalObj::operator()(FWObject *o)
  * the RE. However I keep the code that deals with them in place to be
  * able to use this processor without prior call to
  * replaceClusterInterfaceInItf if necessary.
+ *
+ * TODO: make this code assert() if cluster interface appears in RE/
+ *
+ * Note that rule processor singleObjectNegationItf deals with single
+ * object negation in Interface rule elements.
  */
-bool Compiler::interfaceNegationInRE::processNext()
+bool Compiler::fullInterfaceNegationInRE::processNext()
 {
     Rule *rule = prev_processor->getNextRule(); if (rule==NULL) return false;
     RuleElement *itfre = RuleElement::cast(rule->getFirstByType(re_type));
@@ -1117,6 +1260,7 @@ bool Compiler::emptyGroupsInRE::processNext()
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
         FWObject *o = FWReference::getObject(*i);
+ 
         if ( MultiAddress::cast(o)!=NULL && MultiAddress::cast(o)->isRunTime())
             continue;
 
@@ -1722,3 +1866,5 @@ bool Compiler::DropRulesByAddressFamilyAndServiceType::processNext()
 
     return true;
 }
+
+
