@@ -33,6 +33,7 @@
 #include "fwcompiler/BaseCompiler.h"
 #include "fwcompiler/RuleProcessor.h"
 #include "fwcompiler/exceptions.h"
+#include "fwcompiler/GroupRegistry.h"
 
 #include <list>
 #include <vector>
@@ -229,6 +230,10 @@ public:
 	libfwbuilder::Library *persistent_objects;
 	libfwbuilder::Firewall *fw;
 
+        // group registry is optional, the object shuld be created outside
+        // of the compiler and set using function setGroupRegistry().
+        GroupRegistry *group_registry;
+        
         std::string ruleSetName;;
         
 	libfwbuilder::RuleSet *source_ruleset;
@@ -238,6 +243,8 @@ public:
 
 	std::stringstream output;
 
+        void registerGroupObject(libfwbuilder::ObjectGroup *grp);
+        
         void registerIPv6Rule() { countIPv6Rules++; }
         bool haveIPv6Rules() { return countIPv6Rules > 0; }
 
@@ -363,16 +370,31 @@ public:
             virtual bool processNext();
         };
 
-        class interfaceNegationInRE : public BasicRuleProcessor
+        /**
+         * prepare for negation of single objects in rule elements
+         */
+        class singleObjectNegation : public BasicRuleProcessor
         {
             std::string re_type;
             public:
-                interfaceNegationInRE(const std::string &n,
-                                      std::string _type) :
+            singleObjectNegation(const std::string &n,std::string _type):
                 BasicRuleProcessor(n) { re_type=_type; }
             virtual bool processNext();
         };
 
+        /*
+         * replace interfaces in the give RE with a set of all other
+         * interfaces of the firewall.
+         */
+        class fullInterfaceNegationInRE : public BasicRuleProcessor
+        {
+            std::string re_type;
+            public:
+                fullInterfaceNegationInRE(const std::string &n, std::string _type) :
+                    BasicRuleProcessor(n) { re_type=_type; }
+            virtual bool processNext();
+        };
+
 	/**
 	 * replace cluster interface objects with inetrfaces of the member
          * firewall in the Interface rule element
@@ -828,10 +850,32 @@ public:
             virtual bool processNext();
         };
 
-
-
-
+        /**
+         * This rule processor replaces firewall object with
+         * DNSName object "self" configured as run-time with source
+         * name "self".
+         */
+        class ReplaceFirewallObjectWithSelfInRE : public BasicRuleProcessor
+        {
+            std::string re_type;
+            public:
+                ReplaceFirewallObjectWithSelfInRE(const std::string &n,
+                                      std::string _type) :
+                BasicRuleProcessor(n) { re_type=_type; }
+            virtual bool processNext();
+        };
         
+        class RegisterGroupsAndTablesInRE : public BasicRuleProcessor
+        {
+            std::string re_type;
+            public:
+                RegisterGroupsAndTablesInRE(const std::string &n,
+                                            const std::string _type) :
+                BasicRuleProcessor(n) { re_type = _type; }
+            virtual bool processNext();
+        };
+        friend class Compiler::RegisterGroupsAndTablesInRE;
+
         /**
          * prints rule in some universal format (close to that visible
          * to user in the GUI). Used for debugging purposes
@@ -988,6 +1032,11 @@ public:
 	std::string getCompiledScript();
         int getCompiledScriptLength();
 
+        void setGroupRegistry(GroupRegistry *gr) { group_registry = gr; }
+        
+        void expandGroup(libfwbuilder::FWObject *grp,
+                         std::list<libfwbuilder::FWObject*> &ol);
+
 	void expandGroupsInRuleElement(libfwbuilder::RuleElement *s);
 
         /**
