@@ -187,14 +187,70 @@ namespace fwcompiler {
                 expandMultipleAddressesInRE(n,libfwbuilder::RuleElementDst::TYPENAME) {}
         };
 
+
+
+
+        class ReplaceFirewallObjectWithSelfInSrc : public Compiler::ReplaceFirewallObjectWithSelfInRE
+        {
+            public:
+            ReplaceFirewallObjectWithSelfInSrc(const std::string &n) :
+                ReplaceFirewallObjectWithSelfInRE(
+                    n, libfwbuilder::RuleElementSrc::TYPENAME) {}
+        };
+
+        class ReplaceFirewallObjectWithSelfInDst : public Compiler::ReplaceFirewallObjectWithSelfInRE
+        {
+            public:
+            ReplaceFirewallObjectWithSelfInDst(const std::string &n) :
+                ReplaceFirewallObjectWithSelfInRE(
+                    n, libfwbuilder::RuleElementDst::TYPENAME) {}
+        };
+
+
 	/**
-	 * processes rules with negation in Itf
+         * single object negation in Src
 	 */
-        class ItfNegation : public Compiler::interfaceNegationInRE
+        class singleObjectNegationSrc : public singleObjectNegation
+        {
+            public:
+            singleObjectNegationSrc(const std::string &n):
+                singleObjectNegation(n, libfwbuilder::RuleElementSrc::TYPENAME)
+                {}
+        };
+
+	/**
+         * single object negation in Dst
+	 */
+        class singleObjectNegationDst : public Compiler::singleObjectNegation
+        {
+            public:
+            singleObjectNegationDst(const std::string &n):
+                singleObjectNegation(n, libfwbuilder::RuleElementDst::TYPENAME)
+                {}
+        };
+
+	/**
+         * single object negation in Itf
+	 */
+        class singleObjectNegationItf : public Compiler::singleObjectNegation
+        {
+            public:
+            singleObjectNegationItf(const std::string &n):
+                singleObjectNegation(n, libfwbuilder::RuleElementItf::TYPENAME)
+                {}
+        };
+
+        
+	/**
+	 * processes rules with negation in Itf.
+         * Compiler::fullInterfaceNegationInRE replaces interface object
+         * with a set of "other" interfaces of the firewall.
+	 */
+        class ItfNegation : public Compiler::fullInterfaceNegationInRE
         {
             public:
             ItfNegation(const std::string &name) :
-            interfaceNegationInRE(
+            fullInterfaceNegationInRE(
                 name, libfwbuilder::RuleElementItf::TYPENAME) {}
         };
            
@@ -464,6 +520,22 @@ namespace fwcompiler {
                 swapMultiAddressObjectsInRE(n,libfwbuilder::RuleElementDst::TYPENAME) {}
         };
 
+        class RegisterGroupsAndTablesInSrc : public RegisterGroupsAndTablesInRE
+        {
+            public:
+            RegisterGroupsAndTablesInSrc(const std::string &n) :
+            RegisterGroupsAndTablesInRE(n, libfwbuilder::RuleElementSrc::TYPENAME)
+            {}
+        };
+
+        class RegisterGroupsAndTablesInDst : public RegisterGroupsAndTablesInRE
+        {
+            public:
+            RegisterGroupsAndTablesInDst(const std::string &n) :
+            RegisterGroupsAndTablesInRE(n, libfwbuilder::RuleElementDst::TYPENAME)
+            {}
+        };
+
         
 	virtual int prolog();
 
