@@ -58,6 +58,7 @@ AddressRangeDialog::AddressRangeDialog(QWidget *parent):
     m_dialog->setupUi(this);
     obj=NULL;
 
+    connectSignalsOfAllWidgetsToSlotChange();
 }
 
 AddressRangeDialog::~AddressRangeDialog()
@@ -65,11 +66,6 @@ AddressRangeDialog::~AddressRangeDialog()
     delete m_dialog;
 }
 
-void AddressRangeDialog::getHelpName(QString *str)
-{
-    *str = "AddressRangeDialog";
-}
-
 void AddressRangeDialog::loadFWObject(FWObject *o)
 {
     obj=o;
@@ -81,7 +77,7 @@ void AddressRangeDialog::loadFWObject(FWObject *o)
     m_dialog->obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
     m_dialog->rangeStart->setText( s->getRangeStart().toString().c_str() );
     m_dialog->rangeEnd->setText( s->getRangeEnd().toString().c_str() );
-    m_dialog->comment->setText( QString::fromUtf8(s->getComment().c_str()) );
+    m_dialog->commentKeywords->loadFWObject(o);
     //apply->setEnabled( false );
 
     m_dialog->obj_name->setEnabled(!o->isReadOnly());
@@ -93,10 +89,6 @@ void AddressRangeDialog::loadFWObject(FWObject *o)
     m_dialog->rangeEnd->setEnabled(!o->isReadOnly());
     setDisabledPalette(m_dialog->rangeEnd);
 
-    m_dialog->comment->setReadOnly(o->isReadOnly());
-    setDisabledPalette(m_dialog->comment);
-
-
     init=false;
 }
 
@@ -140,7 +132,8 @@ void AddressRangeDialog::applyChanges()
 
     string oldname = obj->getName();
     new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    new_state->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
+    m_dialog->commentKeywords->applyChanges(new_state);
+
     try
     {
         InetAddr addr_start(m_dialog->rangeStart->text().toStdString());
