@@ -68,6 +68,8 @@ AddressTableDialog::AddressTableDialog(QWidget *parent) : BaseObjectDialog(paren
     m_dialog = new Ui::AddressTableDialog_q;
     m_dialog->setupUi(this);
     obj=NULL;
+
+    connectSignalsOfAllWidgetsToSlotChange();
 }
 
 AddressTableDialog::~AddressTableDialog()
@@ -75,11 +77,6 @@ AddressTableDialog::~AddressTableDialog()
     delete m_dialog;
 }
 
-void AddressTableDialog::getHelpName(QString *str)
-{
-    *str = "AddressTableDialog";
-}
-
 void AddressTableDialog::loadFWObject(FWObject *o)
 {
     obj=o;
@@ -90,7 +87,7 @@ void AddressTableDialog::loadFWObject(FWObject *o)
     init = true;
 
     m_dialog->obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
-    m_dialog->comment->setText( QString::fromUtf8(s->getComment().c_str()) );
+    m_dialog->commentKeywords->loadFWObject(o);
 
     m_dialog->filename->setText( s->getSourceName().c_str() );
     m_dialog->r_compiletime->setChecked(s->isCompileTime() );
@@ -106,9 +103,6 @@ void AddressTableDialog::loadFWObject(FWObject *o)
     m_dialog->filename->setEnabled(!o->isReadOnly());
     setDisabledPalette(m_dialog->filename);
 
-    m_dialog->comment->setReadOnly(o->isReadOnly());
-    setDisabledPalette(m_dialog->comment);
-
     updateButtons();
 
     init = false;
@@ -138,7 +132,8 @@ void AddressTableDialog::applyChanges()
 
     string oldname = obj->getName();
     new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    new_state->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
+    m_dialog->commentKeywords->applyChanges(new_state);
+
     QByteArray cs = m_dialog->filename->text().toLocal8Bit();
     s->setSourceName( (const char *)cs );
     s->setRunTime(m_dialog->r_runtime->isChecked() );
@@ -153,44 +148,96 @@ void AddressTableDialog::applyChanges()
    
 }
 
+
+static void doReminderAboutDataDir()
+{
+    if (st->isReminderAboutDataDirSuppressed()) return;
+
+    QMessageBox msgBox;
+    msgBox.setText("The file you selected is inside the "
+                   "'data directory' global preference.  The path of the "
+                   "file has been converted to use the variable %DATADIR% "
+                   "so that expansion will happen properly within rules.");
+
+    msgBox.setWindowModality(Qt::ApplicationModal);
+    msgBox.setWindowFlags(Qt::Window |
+                          Qt::WindowTitleHint |
+                          Qt::CustomizeWindowHint |
+#if QT_VERSION >= 0x040500
+                          Qt::WindowCloseButtonHint |
+#endif
+                          Qt::WindowSystemMenuHint);
+
+    msgBox.setWindowTitle("Data directory conversion");
+
+    QCheckBox cb("Do not show this again", &msgBox);
+    msgBox.addButton(&cb, QMessageBox::ResetRole);
+    msgBox.addButton(QMessageBox::Close);
+    msgBox.setDefaultButton(QMessageBox::Close);
+    msgBox.setIcon(QMessageBox::Information);
+
+    /* Hack alert!  Disconnect signals from the checkbox so that
+       QMessageBox doesn't know when it gets clicked, and treat it
+       like an "OK" action. */
+    cb.disconnect();
+
+    msgBox.exec();
+    if (cb.isChecked()) st->suppressReminderAboutDataDir(true);
+}
+
+
 void AddressTableDialog::browse()
 {
-
-    QString dir;
-    dir = st->getWDir();
-    if (dir.isEmpty())  dir = st->getOpenFileDir();
-    if (dir.isEmpty())  dir = "~";
-
     // build a dialog that will let user select existing file or enter
     // a name even if the file does not exist
 
-    QFileDialog fd(this);
-    fd.setWindowTitle(tr("Choose a file or type the name to create new"));
-    fd.setDirectory(dir);
-    fd.setFileMode(QFileDialog::AnyFile);
-    fd.setNameFilter(tr("All files (*)"));
-    fd.setViewMode(QFileDialog::Detail);
+    QString s = QFileDialog::getOpenFileName(
+        this,
+        tr("Choose a file or type the name to create new"),
+        st->getOpenFileDir(mw->getCurrentFileName()),
+        tr("All files (*)"));
 
-    if (fd.exec())
-    {
-        QString s = fd.selectedFiles()[0];
+    if (s.isEmpty()) return;
+    st->setOpenFileDir(s);
 
-        if (!s.isEmpty())
-        {
-            m_dialog->filename->setText(s);
-            // assign focus to the "file name" input field so that it
-            // generates signal editFinished when user clicks
-            // elsewhere. We use this signal to call changed() which in
-            // turn calls applyChanges() to save data
-            m_dialog->filename->setFocus(Qt::OtherFocusReason);
-            updateButtons();
+    QString dataDir = st->getDataDir();
+    if (!dataDir.isEmpty()) {
+        QString dataDirPath = QFileInfo(dataDir).canonicalFilePath();
+        QString filePath = QFileInfo(s).canonicalFilePath();
+        if (filePath.length() > 0 && filePath.startsWith(dataDirPath)) {
+            int truncateLen = dataDirPath.length();
+            if (dataDirPath.at(truncateLen-1) == '/' ||
+                dataDirPath.at(truncateLen-1) == '\\') {
+                truncateLen--;
+            }
+            s = filePath.replace(0, truncateLen, "%DATADIR%");
+            doReminderAboutDataDir();
         }
     }
+
+    m_dialog->filename->setText(s);
+    // assign focus to the "file name" input field so that it
+    // generates signal editFinished when user clicks
+    // elsewhere. We use this signal to call changed() which in
+    // turn calls applyChanges() to save data
+    m_dialog->filename->setFocus(Qt::OtherFocusReason);
+    updateButtons();
 }
 
 void AddressTableDialog::editFile( void )
 {
     QString filePath = m_dialog->filename->text();
+    if (filePath.startsWith("%DATADIR%")) {
+        QString dataDir = st->getDataDir();
+        if (dataDir.isEmpty()) {
+            QMessageBox::critical(this, "Firewall Builder",
+                                  tr("Data directory setting is blank "
+                                     "and path contains %DATADIR% variable"));
+            return;
+        }
+        filePath.replace(0, 9, dataDir);
+    }
+
     TextFileEditor editor(this, filePath);
     if (editor.load())
         editor.exec();  // its modal dialog
