@@ -40,6 +40,7 @@
 #include "PhysicalAddressDialog.h"
 #include "DNSNameDialog.h"
 #include "AddressTableDialog.h"
+#include "AttachedNetworksDialog.h"
 #include "NetworkDialog.h"
 #include "NetworkDialogIPv6.h"
 #include "UserDialog.h"
@@ -49,6 +50,7 @@
 #include "TCPServiceDialog.h"
 #include "UDPServiceDialog.h"
 #include "GroupObjectDialog.h"
+#include "DynamicGroupDialog.h"
 #include "HostDialog.h"
 #include "FirewallDialog.h"
 #include "ClusterDialog.h"
@@ -98,6 +100,7 @@
 #include "openaisOptionsDialog.h"
 #include "pfsyncOptionsDialog.h"
 
+#include "fwbuilder/AttachedNetworks.h"
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Cluster.h"
@@ -112,6 +115,7 @@
 #include "fwbuilder/AddressTable.h"
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/ObjectGroup.h"
+#include "fwbuilder/DynamicGroup.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/IPService.h"
@@ -172,6 +176,8 @@ BaseObjectDialog *DialogFactory::createDialog(QWidget *parent, const QString &ob
 
     if (objType==Interface::TYPENAME)     return new InterfaceDialog(parent);
 
+    if (objType==AttachedNetworks::TYPENAME) return new AttachedNetworksDialog(parent);
+
     if (objType==Network::TYPENAME)       return new NetworkDialog(parent);
 
     if (objType==NetworkIPv6::TYPENAME)   return new NetworkDialogIPv6(parent);
@@ -190,6 +196,8 @@ BaseObjectDialog *DialogFactory::createDialog(QWidget *parent, const QString &ob
 
     if (objType==ObjectGroup::TYPENAME)   return new GroupObjectDialog(parent);
 
+    if (objType==DynamicGroup::TYPENAME)   return new DynamicGroupDialog(parent);
+
     if (objType==ServiceGroup::TYPENAME)  return new GroupObjectDialog(parent);
 
     if (objType==TagService::TYPENAME) return new TagServiceDialog(parent);
@@ -286,7 +294,8 @@ QWidget *DialogFactory::createOSDialog(QWidget *parent,FWObject *o)
 QWidget *DialogFactory::createIfaceDialog(QWidget *parent,FWObject *o)
     throw(FWException)
 {
-    FWObject *h = Interface::cast(o)->getParentHost();
+    FWObject *h = Host::getParentHost(o);
+    //FWObject *h = Interface::cast(o)->getParentHost();
 
     string host_OS = h->getStr("host_OS");
     Resources *os = Resources::os_res[host_OS];
