@@ -69,8 +69,8 @@ const char* DTDSetpath = SETTINGS_PATH_PREFIX "/System/DTDPath";
 const char* ResSetpath = SETTINGS_PATH_PREFIX "/System/ResPath";
 const char* compression = SETTINGS_PATH_PREFIX "/DataFile/compression";
 const char* wdirSetpath = SETTINGS_PATH_PREFIX "/Environment/WDir";
+const char* datadirSetpath = SETTINGS_PATH_PREFIX "/Environment/DataDir";
 const char* ofdirSetpath = SETTINGS_PATH_PREFIX "/Environment/OpenFileDir";
-const char* sfdirSetpath = SETTINGS_PATH_PREFIX "/Environment/SaveFileDir";
 const char* startupActionSetpath =
     SETTINGS_PATH_PREFIX "/Environment/StartupAction";
 const char* labelColorPath = SETTINGS_PATH_PREFIX "/ColorLabels/color_";
@@ -115,6 +115,7 @@ const char* announcementLastTime =
 const char* checkUpdatesProxy = SETTINGS_PATH_PREFIX "/UI/CheckUpdatesProxy";
 const char* reminderAboutStandardLibSuppressed =
     SETTINGS_PATH_PREFIX "/UI/reminderAboutStandardLibSuppressed";
+const char* reminderDataDir = SETTINGS_PATH_PREFIX "/UI/reminderDataDir";
 const char* introDialogEnabled = SETTINGS_PATH_PREFIX "/UI/introDialogEnabled";
 const char* newFirewallPlatform =
     SETTINGS_PATH_PREFIX "/Objects/NewFireallPlatform";
@@ -399,6 +400,15 @@ void FWBSettings::init(bool force_first_time_run)
     if (!hasKey("Objects/PolicyRule/defaultLoggingState"))
         setBool("Objects/PolicyRule/defaultLoggingState", true);
 
+    if (!hasKey("Objects/PolicyRule/defaultStateful"))
+        setBool("Objects/PolicyRule/defaultStateful", true);
+
+    if (!hasKey("Objects/PolicyRule/defaultAction"))
+        setInt("Objects/PolicyRule/defaultAction", 0);
+
+    if (!hasKey("Objects/PolicyRule/defaultDirection"))
+        setInt("Objects/PolicyRule/defaultDirection", 0);
+
     if (!hasKey("Objects/Interface/autoconfigureInterfaces"))
         setBool("Objects/Interface/autoconfigureInterfaces", true);
 
@@ -414,6 +424,16 @@ void FWBSettings::suppressReminderAboutStandardLib(bool f)
     setValue(reminderAboutStandardLibSuppressed, f);
 }
 
+bool FWBSettings::isReminderAboutDataDirSuppressed()
+{
+    return value(reminderDataDir).toBool();
+}
+
+void FWBSettings::suppressReminderAboutDataDir(bool f)
+{
+    setValue(reminderDataDir, f);
+}
+
 bool FWBSettings::hasKey(const QString &attribute)
 {
     return QSettings::contains(SETTINGS_PATH_PREFIX "/" + attribute);
@@ -474,7 +494,12 @@ void FWBSettings::setList(const QString &attribute, QStringList &list)
 }
 
 QString FWBSettings::getWDir() { return value(wdirSetpath).toString();}
-void FWBSettings::setWDir( const QString &wd ) { setValue(wdirSetpath,wd);}
+void FWBSettings::setWDir(const QString &wd) { setValue(wdirSetpath, wd);}
+QString FWBSettings::getDataDir() { return value(datadirSetpath).toString();}
+void FWBSettings::setDataDir(const QString &d) {
+    setValue(datadirSetpath, d);
+    FWObject::setDataDir(d.toUtf8().constData());
+}
 int FWBSettings::getInfoStyle() { return value(infoStyleSetpath).toInt();}
 void FWBSettings::setInfoStyle(int s) { setValue(infoStyleSetpath,s);}
 int FWBSettings::getInfoWindowHeight() { return value(infoWindowHSetpath).toInt();}
@@ -506,24 +531,32 @@ void FWBSettings::setTooltipDelay(int v) { setValue( tooltipDelay, v); }
 QString FWBSettings::getLastEdited() { return value(lastEditedSetpath).toString();}
 void FWBSettings::setLastEdited(const QString &file) { setValue(lastEditedSetpath,file);}
 
-QString FWBSettings::getOpenFileDir()
+QString FWBSettings::getOpenFileDir(const QString &existingPath)
 {
-    return value(ofdirSetpath).toString();
+    QString ret = getWDir();
+    if (!ret.isEmpty() && QFileInfo(ret).isDir()) return ret;
+
+    ret = value(ofdirSetpath).toString();
+    if (!ret.isEmpty() && QFileInfo(ret).isDir()) return ret;
+
+    if (!existingPath.isEmpty()) {
+        ret = getFileDir(existingPath);
+        if (QFileInfo(ret).isDir()) return ret;
+    }
+
+    return userDataDir.c_str();
 }
 
-void FWBSettings::setOpenFileDir( const QString &d )
+void FWBSettings::setOpenFileDir(const QString &d)
 {
-     setValue(ofdirSetpath,d);
-}
+    QString dirPath = d;
 
-QString FWBSettings::getSaveFileDir()
-{
-    return value(sfdirSetpath).toString();
-}
+    QFileInfo info(d);
+    if (!info.isDir()) {
+        dirPath = info.dir().path();
+    }
 
-void FWBSettings::setSaveFileDir( const QString &d )
-{
-     setValue(sfdirSetpath,d);
+    setValue(ofdirSetpath, dirPath);
 }
 
 void FWBSettings::save()
