@@ -26,6 +26,7 @@
 #ifndef __FWBSETTINGS_H_
 #define __FWBSETTINGS_H_
 
+#include "../../VERSION.h"
 
 #include <qsettings.h>
 #include <qrect.h>
@@ -34,10 +35,11 @@
 
 #include <set>
 
-#define SETTINGS_PATH_PREFIX  "/4.2"
+#define SETTINGS_PATH_PREFIX  "/" GENERATION
 
 #include <fwbuilder/FWObject.h>
 
+
 class QWidget;
 
 /*
@@ -48,8 +50,8 @@ class QWidget;
 #define LoadStandardObjects 0
 #define LoadLastEditedFile  1
 
-class FWBSettings : public QSettings {
-
+class FWBSettings : public QSettings
+{
  public:
 
     enum LabelColors { RED, ORANGE, YELLOW, GREEN, BLUE, PURPLE, GRAY };
@@ -68,7 +70,10 @@ class FWBSettings : public QSettings {
     FWBSettings(bool testData = false);
     ~FWBSettings();
 
-    static QString getApplicationNameForSettings() { return "FirewallBuilder4.2"; }
+    static QString getApplicationNameForSettings()
+    {
+        return "FirewallBuilder" GENERATION;
+    }
 
     void init(bool force_first_time_run=false);
     void save();
@@ -76,14 +81,14 @@ class FWBSettings : public QSettings {
     bool isFirstRun() { return first_run; }
     
     QString getWDir();
-    void    setWDir( const QString &wd );
+    void    setWDir(const QString &wd);
 
-    QString getOpenFileDir();
+    QString getDataDir();
+    void    setDataDir(const QString &dataDir);
+
+    QString getOpenFileDir( const QString &existingPath = "");
     void    setOpenFileDir( const QString &d );
 
-    QString getSaveFileDir();
-    void    setSaveFileDir( const QString &d );
-
     int     getInfoStyle();
     void    setInfoStyle(int s);
 
@@ -178,6 +183,9 @@ class FWBSettings : public QSettings {
     bool isReminderAboutStandardLibSuppressed();
     void suppressReminderAboutStandardLib(bool f);
     
+    bool isReminderAboutDataDirSuppressed();
+    void suppressReminderAboutDataDir(bool f);
+    
     enum IconSize getIconsInRulesSize();
     void setIconsInRulesSize(enum IconSize size);
 
