@@ -44,6 +44,7 @@
 #include "fwbuilder/AddressTable.h"
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/DNSName.h"
+#include "fwbuilder/DynamicGroup.h"
 #include "fwbuilder/FWException.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/FWReference.h"
@@ -176,44 +177,49 @@ QString FWObjectPropertiesFactory::getObjectPropertiesBrief(FWObject *obj)
             if (!obj->isReadOnly()) intf->getOptionsObject();
             str << intf->getLabel().c_str() << " ";
 
-            FWObject *parent = intf->getParentHost();
-
-            bool supports_security_levels = false;
-            bool supports_network_zones   = false;
-            try
-            {
-                supports_security_levels =
-                    (!parent->getStr("platform").empty() &&
-                     Resources::getTargetCapabilityBool(
-                         parent->getStr("platform"), "security_levels"));
-                supports_network_zones =
-                    (!parent->getStr("platform").empty() &&
-                     Resources::getTargetCapabilityBool(
-                         parent->getStr("platform"), "network_zones"));
-            } catch (FWException &ex)  { }
-
             QStringList q;
-            if (supports_security_levels)
-            {
-                QString str;
-                str.setNum(intf->getSecurityLevel());
-                q.push_back(QString("sec level: %1").arg(str));
-            }
-            if (supports_network_zones)
-            {
-                int id = FWObjectDatabase::getIntId(intf->getStr("network_zone"));
-                if (id > 0)
-                {
-                    FWObject *nz_obj = obj->getRoot()->findInIndex(id);
-                    if (nz_obj)
-                        q.push_back(
-                            QString("network zone: %1")
-                            .arg(nz_obj->getName().c_str()));
-                    else
-                        q.push_back(QString("network zone: not configured"));
 
+            FWObject *parent = Host::getParentHost(intf);
+            //FWObject *parent = intf->getParentHost();
+            if (parent)
+            {
+                bool supports_security_levels = false;
+                bool supports_network_zones   = false;
+                try
+                {
+                    supports_security_levels =
+                        (!parent->getStr("platform").empty() &&
+                         Resources::getTargetCapabilityBool(
+                             parent->getStr("platform"), "security_levels"));
+                    supports_network_zones =
+                        (!parent->getStr("platform").empty() &&
+                         Resources::getTargetCapabilityBool(
+                             parent->getStr("platform"), "network_zones"));
+                } catch (FWException &ex)  { }
+
+                if (supports_security_levels)
+                {
+                    QString str;
+                    str.setNum(intf->getSecurityLevel());
+                    q.push_back(QString("sec level: %1").arg(str));
+                }
+                if (supports_network_zones)
+                {
+                    int id = FWObjectDatabase::getIntId(intf->getStr("network_zone"));
+                    if (id > 0)
+                    {
+                        FWObject *nz_obj = obj->getRoot()->findInIndex(id);
+                        if (nz_obj)
+                            q.push_back(
+                                QString("network zone: %1")
+                                .arg(nz_obj->getName().c_str()));
+                        else
+                            q.push_back(QString("network zone: not configured"));
+
+                    }
                 }
             }
+
             if (intf->isDyn())         q.push_back("dyn");
             if (intf->isUnnumbered())  q.push_back("unnum");
             if (intf->isDedicatedFailover())  q.push_back("failover");
@@ -394,11 +400,13 @@ QString FWObjectPropertiesFactory::getObjectProperties(FWObject *obj)
                 members.push_front(QObject::tr("Members:"));
                 str << members.join(" ");
             }
+        } else if (DynamicGroup::cast(obj) != 0) {
+            DynamicGroup *objGroup = DynamicGroup::cast(obj);
+            str << QObject::tr("%1 filters").arg(objGroup->getFilter().size());
         } else if (Group::cast(obj)!=NULL)   // just any group
         {
             Group *g=Group::cast(obj);
             str << QObject::tr("%1 objects").arg(g->size());
-
         } else if (Firewall::cast(obj))
         {
 
@@ -705,6 +713,9 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
                         arg(fw->getName().c_str()).arg(obj->getName().c_str());
                 }
             }
+        } else if (DynamicGroup::cast(obj) != 0) {
+            DynamicGroup *objGroup = DynamicGroup::cast(obj);
+            str += QObject::tr("%1 filters<br>\n").arg(objGroup->getFilter().size());
         } else if (Group::cast(obj)!=NULL)   // just any group
         {
             if (showPath && !tooltip) str += "<b>Path: </b>" + path + "<br>\n";
@@ -945,15 +956,6 @@ QString FWObjectPropertiesFactory::getRuleActionProperties(Rule *rule)
             case PolicyRule::Reject:
                 par = ropt->getStr("action_on_reject").c_str();
                 break;
-            case PolicyRule::Tag:
-            {
-                FWObject *tag_object = PolicyRule::cast(rule)->getTagObject();
-                if (tag_object)
-                    par = QString::fromUtf8(tag_object->getName().c_str());
-                else
-                    par = QString::fromUtf8(PolicyRule::cast(rule)->getTagValue().c_str());
-                break;
-            }
             case PolicyRule::Accounting :
                 par = ropt->getStr("rule_name_accounting").c_str();
                 break;
@@ -968,24 +970,6 @@ QString FWObjectPropertiesFactory::getRuleActionProperties(Rule *rule)
                 // ropt->getStr("branch_name").c_str();
                 break;
             }
-            case PolicyRule::Classify:
-                if (platform=="ipfw")
-                {
-                    if (ropt->getInt("ipfw_classify_method") == DUMMYNETPIPE)
-                    {
-                        par = "pipe";
-                    } else {
-                        par = "queue";
-                    }
-                    par = par + " " + ropt->getStr("ipfw_pipe_queue_num").c_str();
-                } else
-                {
-                    if (platform=="pf")
-                        par = ropt->getStr("pf_classify_str").c_str();
-                    else
-                        par = ropt->getStr("classify_str").c_str();
-                }
-                break;
             case PolicyRule::Pipe :
                 if (platform=="ipfw")
                 {
@@ -993,44 +977,6 @@ QString FWObjectPropertiesFactory::getRuleActionProperties(Rule *rule)
                         ropt->getStr("ipfw_pipe_port_num").c_str();
                 }
                 break;
-            case PolicyRule::Route :
-                if (platform=="iptables")
-                {
-                    string a;
-                    a = ropt->getStr("ipt_gw");
-                    if (!a.empty()) par = par + " gw: " + a.c_str();
-                    a = ropt->getStr("ipt_iif");
-                    if (!a.empty()) par = par + " iif: " + a.c_str();
-                    a = ropt->getStr("ipt_oif");
-                    if (!a.empty()) par = par + " oif: " + a.c_str();
-                }
-                if (platform=="ipf")
-                {
-                    string a;
-                    a = ropt->getStr("ipf_route_option");
-                    if (!a.empty())
-                    {
-                        par = par + " "+ 
-                            getScreenName(a.c_str(),
-                                          getRouteOptions_pf_ipf( platform.c_str() ));
-                    }
-                    a = ropt->getStr("ipf_route_opt_if");
-                    if (!a.empty()) par = par + " "+ a.c_str();
-                    a = ropt->getStr("ipf_route_opt_addr");
-                    if (!a.empty()) par = par + " "+ a.c_str();
-                }
-                if (platform=="pf")
-                {
-                    string a;
-                    a = ropt->getStr("pf_route_option");
-                    if (!a.empty()) par = par + " "+ a.c_str();
-                    a = ropt->getStr("pf_route_opt_if");
-                    if (!a.empty()) par = par + " "+ a.c_str();
-                    a = ropt->getStr("pf_route_opt_addr");
-                    if (!a.empty()) par = par + " "+ a.c_str();
-                }
-                break;
-
 
             default : {}
             }
@@ -1080,120 +1026,251 @@ QString FWObjectPropertiesFactory::getRuleActionPropertiesRich(Rule *rule)
 
 QString FWObjectPropertiesFactory::getPolicyRuleOptions(Rule *rule)
 {
+    if (rule == NULL) return "";
+
     QList<QPair<QString,QString> > options;
 
-    if (rule!=NULL)
-    {
-        FWObject *o = rule;
-        while (o!=NULL && Firewall::cast(o)==NULL) o = o->getParent();
-        assert(o!=NULL);
-        Firewall *f = Firewall::cast(o);
-        string platform = f->getStr("platform");
-        FWOptions *ropt = rule->getOptionsObject();
+    PolicyRule *prule = PolicyRule::cast(rule);
+    FWObject *o = rule;
+    while (o!=NULL && Firewall::cast(o)==NULL) o = o->getParent();
+    assert(o!=NULL);
+    Firewall *f = Firewall::cast(o);
+    string platform = f->getStr("platform");
+    FWOptions *ropt = rule->getOptionsObject();
 
-        if (platform!="iosacl" && platform!="procurve_acl")
+    if (platform!="iosacl" && platform!="procurve_acl")
+    {
+        if (ropt->getBool("stateless"))
         {
-            if (ropt->getBool("stateless"))
+            options << qMakePair(QObject::tr("Stateless"), QString(""));
+        } else
+        {
+            options << qMakePair(QObject::tr("Stateful"), QString(""));
+        }
+    }
+
+    if (platform=="iptables")
+    {
+        if (prule)
+        {
+            if (prule->getTagging())
             {
-                options << qMakePair(QObject::tr("Stateless"), QString(""));
-            } else
+                options << qMakePair(QObject::tr("tag:"), 
+                                     QString(prule->getTagValue().c_str()));
+            }
+
+            if (prule->getClassification())
             {
-                options << qMakePair(QObject::tr("Stateful"), QString(""));
+                options << qMakePair(QObject::tr("class:"),
+                                     QString(ropt->getStr("classify_str").c_str()));
+            }
+        }
+            
+        if (!ropt->getStr("log_prefix").empty())
+        {
+            options << qMakePair(QObject::tr("Log prefix:"),
+                                 QString(ropt->getStr("log_prefix").c_str()));
+        }
+
+        if (!ropt->getStr("log_level").empty())
+        {
+            options << qMakePair(
+                QObject::tr("Log Level:"),
+                QString(getScreenName(ropt->getStr("log_level").c_str(),
+                                      getLogLevels(platform.c_str()))));
+        }
+
+        if (ropt->getInt("ulog_nlgroup")>1)
+        {
+            options << qMakePair(
+                QObject::tr("Netlink group:"),
+                QString(ropt->getStr("ulog_nlgroup").c_str()));
+        }
+            
+        if (ropt->getInt("limit_value")>0)
+        {
+            QString arg;
+            if (ropt->getBool("limit_value_not")) arg = " ! ";
+            arg += QString(ropt->getStr("limit_value").c_str());
+            if (!ropt->getStr("limit_suffix").empty())
+            {
+                arg += getScreenName(ropt->getStr("limit_suffix").c_str(),
+                                     getLimitSuffixes(platform.c_str()));
+            }
+            options << qMakePair(QString("Limit value:"), arg);
+        }
+
+        if (ropt->getInt("limit_burst")>0)
+        {
+            options << qMakePair(QString("Limit burst:"),
+                                 QString(ropt->getStr("limit_burst").c_str()));
+        }
+
+        if (ropt->getInt("connlimit_value")>0)
+        {
+            QString arg;
+
+            if (ropt->getBool("connlimit_above_not")) arg = " ! ";
+            arg += QString(ropt->getStr("connlimit_value").c_str());
+
+            options << qMakePair(QObject::tr("connlimit value:"), arg);
+        }
+
+        if (ropt->getInt("hashlimit_value")>0)
+        {
+            QString arg;
+            if (ropt->getBool("hashlimit_value_not")) arg = " ! ";
+            arg += QString(ropt->getStr("hashlimit_value").c_str());
+            if (!ropt->getStr("hashlimit_suffix").empty())
+            {
+                arg += getScreenName(ropt->getStr("limit_suffix").c_str(),
+                                     getLimitSuffixes(platform.c_str()));
+            }
+
+            options << qMakePair(
+                QString("hashlimit name:"),
+                QString(ropt->getStr("hashlimit_name").c_str()));
+            options << qMakePair(QString("hashlimit value:"), arg);
+
+            if (ropt->getInt("hashlimit_burst")>0)
+            {
+                options << qMakePair(
+                    QString("haslimit burst:"),
+                    QString(ropt->getStr("hashlimit_burst").c_str()));
+            }
+        }
+            
+        if (ropt->getBool("firewall_is_part_of_any_and_networks"))
+        {
+            options << qMakePair(QObject::tr("Part of Any"), QString(""));
+        }
+
+            
+    } else if (platform=="ipf") 
+    {
+        if (!ropt->getStr("ipf_log_facility").empty())
+        {
+            options << qMakePair(
+                QObject::tr("Log facility:"),
+                QString(getScreenName(ropt->getStr("ipf_log_facility").c_str(),
+                                      getLogFacilities(platform.c_str()))));
+        }
+            
+        if (!ropt->getStr("log_level").empty())
+        {
+            options << qMakePair(
+                QObject::tr("Log level:"),
+                QString(getScreenName(ropt->getStr("log_level").c_str(),
+                                      getLogLevels(platform.c_str()))));
+        }
+            
+        if (ropt->getBool("ipf_return_icmp_as_dest"))
+        {
+            options << qMakePair(
+                QObject::tr("Send 'unreachable'"), QString(""));
+        }
+
+        if (ropt->getBool("ipf_keep_frags"))
+        {
+            options << qMakePair(
+                QObject::tr("Keep information on fragmented packets"),
+                QString(""));
+        }
+            
+    }else if (platform=="pf")
+    {
+
+        if (prule)
+        {
+            if (prule->getTagging())
+            {
+                options << qMakePair(QObject::tr("tag:"), 
+                                     QString(prule->getTagValue().c_str()));
+            }
+
+            if (prule->getClassification())
+            {
+                options << qMakePair(QObject::tr("queue:"),
+                                     QString(ropt->getStr("pf_classify_str").c_str()));
             }
         }
 
-        if (platform=="iptables")
+        if (!ropt->getStr("log_prefix").empty())
         {
-            if (!ropt->getStr("log_prefix").empty())
-            {
-                options << qMakePair(QObject::tr("Log prefix:"),
-                                     QString(ropt->getStr("log_prefix").c_str()));
-            }
-
-            if (!ropt->getStr("log_level").empty())
-            {
-                options << qMakePair(
-                    QObject::tr("Log Level:"),
-                    QString(getScreenName(ropt->getStr("log_level").c_str(),
-                                          getLogLevels(platform.c_str()))));
-            }
-
-            if (ropt->getInt("ulog_nlgroup")>1)
-            {
-                options << qMakePair(
-                    QObject::tr("Netlink group:"),
-                    QString(ropt->getStr("ulog_nlgroup").c_str()));
-            }
+            options << qMakePair(QObject::tr("Log prefix:"), 
+                                 QString(ropt->getStr("log_prefix").c_str()));
+        }
             
-            if (ropt->getInt("limit_value")>0)
-            {
-                QString arg;
-                if (ropt->getBool("limit_value_not")) arg = " ! ";
-                arg += QString(ropt->getStr("limit_value").c_str());
-                if (!ropt->getStr("limit_suffix").empty())
-                {
-                    arg += getScreenName(ropt->getStr("limit_suffix").c_str(),
-                                         getLimitSuffixes(platform.c_str()));
-                }
-                options << qMakePair(QString("Limit value:"), arg);
-            }
-
-            if (ropt->getInt("limit_burst")>0)
-            {
-                options << qMakePair(QString("Limit burst:"),
-                                     QString(ropt->getStr("limit_burst").c_str()));
-            }
-
-            if (ropt->getInt("connlimit_value")>0)
-            {
-                QString arg;
-
-                if (ropt->getBool("connlimit_above_not")) arg = " ! ";
-                arg += QString(ropt->getStr("connlimit_value").c_str());
-
-                options << qMakePair(QObject::tr("connlimit value:"), arg);
-            }
-
-            if (ropt->getInt("hashlimit_value")>0)
-            {
-                QString arg;
-                if (ropt->getBool("hashlimit_value_not")) arg = " ! ";
-                arg += QString(ropt->getStr("hashlimit_value").c_str());
-                if (!ropt->getStr("hashlimit_suffix").empty())
-                {
-                    arg += getScreenName(ropt->getStr("limit_suffix").c_str(),
-                                         getLimitSuffixes(platform.c_str()));
-                }
-
-                options << qMakePair(
-                    QString("hashlimit name:"),
-                    QString(ropt->getStr("hashlimit_name").c_str()));
-                options << qMakePair(QString("hashlimit value:"), arg);
-
-                if (ropt->getInt("hashlimit_burst")>0)
-                {
-                    options << qMakePair(
-                        QString("haslimit burst:"),
-                        QString(ropt->getStr("hashlimit_burst").c_str()));
-                }
-            }
-            
-            if (ropt->getBool("firewall_is_part_of_any_and_networks"))
-            {
-                options << qMakePair(QObject::tr("Part of Any"), QString(""));
-            }
-
-            
-        } else if (platform=="ipf") 
+        if (ropt->getInt("pf_rule_max_state")>0)
         {
-            if (!ropt->getStr("ipf_log_facility").empty())
-            {
-                options << qMakePair(
-                    QObject::tr("Log facility:"),
-                    QString(getScreenName(ropt->getStr("ipf_log_facility").c_str(),
-                                          getLogFacilities(platform.c_str()))));
-            }
+            options << qMakePair(
+                QObject::tr("Max state:"),
+                QString(ropt->getStr("pf_rule_max_state").c_str()));
+        }
             
+        if (ropt->getBool("pf_keep_state"))
+        {
+            options << qMakePair(
+                QObject::tr("Force 'keep-state'"), QString(""));
+        }
+        if (ropt->getBool("pf_no_sync"))
+        {
+            options << qMakePair(QString("no-sync"), QString(""));
+        }
+        if (ropt->getBool("pf_pflow"))
+        {
+            options << qMakePair(QString("pflow"), QString(""));
+        }
+        if (ropt->getBool("pf_sloppy_tracker"))
+        {
+            options << qMakePair(QString("sloppy-tracker"), QString(""));
+        }
+            
+        if (ropt->getBool("pf_source_tracking"))
+        {
+            options << qMakePair(
+                QObject::tr("Source tracking"), QString(""));
+                
+            options << qMakePair(
+                QObject::tr("Max src nodes:"),
+                QString(ropt->getStr("pf_max_src_nodes").c_str()));
+                
+            options << qMakePair(
+                QObject::tr("Max src states:"),
+                QString(ropt->getStr("pf_max_src_states").c_str()));
+        }
+
+        if (ropt->getBool("pf_synproxy"))
+        {
+            options << qMakePair(QString("synproxy"), QString(""));
+        }
+
+        if (ropt->getBool("pf_modulate_state"))
+        {
+            options << qMakePair(QString("modulate_state"), QString(""));
+        }
+            
+    }else if (platform=="ipfw")
+    {
+        ;
+    }else if (platform == "iosacl" || platform == "procurve_acl")
+    {
+        if (ropt->getBool("iosacl_add_mirror_rule"))
+        {
+            options << qMakePair(
+                QObject::tr("Add mirrored rule"), QString(""));
+        }
+            
+    }else if (platform=="pix" || platform=="fwsm")
+    {
+        string vers = "version_"+f->getStr("version");
+            
+        options << qMakePair(QObject::tr("Version:"), QString(vers.c_str()));
+            
+        if ( Resources::platform_res[platform]->getResourceBool(
+                 "/FWBuilderResources/Target/options/"+vers+"/pix_rule_syslog_settings"))
+        {
+                
             if (!ropt->getStr("log_level").empty())
             {
                 options << qMakePair(
@@ -1201,124 +1278,22 @@ QString FWObjectPropertiesFactory::getPolicyRuleOptions(Rule *rule)
                     QString(getScreenName(ropt->getStr("log_level").c_str(),
                                           getLogLevels(platform.c_str()))));
             }
-            
-            if (ropt->getBool("ipf_return_icmp_as_dest"))
+            if (ropt->getInt("log_interval")>0)
             {
                 options << qMakePair(
-                    QObject::tr("Send 'unreachable'"), QString(""));
+                    QObject::tr("Log interval:"),
+                    QString(ropt->getStr("log_interval").c_str()));
             }
-
-            if (ropt->getBool("ipf_keep_frags"))
-            {
-                options << qMakePair(
-                    QObject::tr("Keep information on fragmented packets"),
-                    QString(""));
-            }
-            
-        }else if (platform=="pf")
-        {
-            
-            if (!ropt->getStr("log_prefix").empty())
-            {
-                options << qMakePair(QObject::tr("Log prefix:"), 
-                                     QString(ropt->getStr("log_prefix").c_str()));
-            }
-            
-            if (ropt->getInt("pf_rule_max_state")>0)
-            {
-                options << qMakePair(
-                    QObject::tr("Max state:"),
-                    QString(ropt->getStr("pf_rule_max_state").c_str()));
-            }
-            
-            if (ropt->getBool("pf_keep_state"))
-            {
-                options << qMakePair(
-                    QObject::tr("Force 'keep-state'"), QString(""));
-            }
-            if (ropt->getBool("pf_no_sync"))
-            {
-                options << qMakePair(QString("no-sync"), QString(""));
-            }
-            if (ropt->getBool("pf_pflow"))
-            {
-                options << qMakePair(QString("pflow"), QString(""));
-            }
-            if (ropt->getBool("pf_sloppy_tracker"))
-            {
-                options << qMakePair(QString("sloppy-tracker"), QString(""));
-            }
-            
-            if (ropt->getBool("pf_source_tracking"))
-            {
-                options << qMakePair(
-                    QObject::tr("Source tracking"), QString(""));
                 
-                options << qMakePair(
-                    QObject::tr("Max src nodes:"),
-                    QString(ropt->getStr("pf_max_src_nodes").c_str()));
-                
-                options << qMakePair(
-                    QObject::tr("Max src states:"),
-                    QString(ropt->getStr("pf_max_src_states").c_str()));
-            }
-
-            if (ropt->getBool("pf_synproxy"))
-            {
-                options << qMakePair(QString("synproxy"), QString(""));
-            }
-
-            if (ropt->getBool("pf_modulate_state"))
-            {
-                options << qMakePair(QString("modulate_state"), QString(""));
-            }
-            
-        }else if (platform=="ipfw")
-        {
-            ;
-        }else if (platform == "iosacl" || platform == "procurve_acl")
-        {
-            if (ropt->getBool("iosacl_add_mirror_rule"))
+            if (ropt->getBool("disable_logging_for_this_rule"))
             {
                 options << qMakePair(
-                    QObject::tr("Add mirrored rule"), QString(""));
+                    QObject::tr("Disable logging for this rule"), QString(""));
             }
-            
-        }else if (platform=="pix" || platform=="fwsm")
-        {
-            string vers = "version_"+f->getStr("version");
-            
-            options << qMakePair(QObject::tr("Version:"), QString(vers.c_str()));
-            
-            if ( Resources::platform_res[platform]->getResourceBool(
-                  "/FWBuilderResources/Target/options/"+vers+"/pix_rule_syslog_settings"))
-            {
                 
-                if (!ropt->getStr("log_level").empty())
-                {
-                    options << qMakePair(
-                        QObject::tr("Log level:"),
-                        QString(getScreenName(ropt->getStr("log_level").c_str(),
-                                              getLogLevels(platform.c_str()))));
-                }
-                if (ropt->getInt("log_interval")>0)
-                {
-                    options << qMakePair(
-                        QObject::tr("Log interval:"),
-                        QString(ropt->getStr("log_interval").c_str()));
-                }
-                
-                if (ropt->getBool("disable_logging_for_this_rule"))
-                {
-                    options << qMakePair(
-                        QObject::tr("Disable logging for this rule"), QString(""));
-                }
-                
-            } 
-        }
+        } 
     }
 
-    PolicyRule *prule = PolicyRule::cast(rule);
     if (prule)
         options << qMakePair(
             QObject::tr("Logging: "),
