@@ -69,8 +69,6 @@
 #include "HttpGet.h"
 #include "StartTipDialog.h"
 
-#include "transferDialog.h"
-
 #include "events.h"
 
 #include "importAddressListWizard/ImportAddressListWizard.h"
@@ -231,7 +229,6 @@ FWWindow::FWWindow() : QMainWindow(),   // QMainWindow(NULL, Qt::Desktop),
     m_mainWindow->editorDockWidget->hide();
 
     oe  = new ObjectEditor((QWidget*)m_mainWindow->objectEditorStack);
-    oe->setHelpButton(m_mainWindow->helpObjectEditorButton);
 
     m_mainWindow->editorDockWidget->setupEditor(oe);
     m_mainWindow->editorDockWidget->hide();
@@ -262,12 +259,6 @@ FWWindow::FWWindow() : QMainWindow(),   // QMainWindow(NULL, Qt::Desktop),
     connect(current_version_http_getter, SIGNAL(done(const QString&)),
             this, SLOT(checkForUpgrade(const QString&)));
 
-    // connect( m_mainWindow->newObjectAction, SIGNAL( triggered() ),
-    //          this, SLOT(newObject() ) );
-
-    connect( m_mainWindow->backAction, SIGNAL( triggered() ),
-             this, SLOT(back() ) );
-
     connect( m_mainWindow->findAction, SIGNAL( triggered() ),
              this, SLOT(search()) );
 
@@ -606,10 +597,16 @@ void FWWindow::showIntroDialog()
             msg_box.addButton(tr("Watch the guide"), QMessageBox::AcceptRole);
         msg_box.addButton(QMessageBox::Close);
 
+        /* Hack alert!  Disconnect signals from the checkbox so that
+           QMessageBox doesn't know when it gets clicked.  We check it
+           directly ourselves to see what state it's in when the user
+           clicks "close" or "watch". */
+        cb.disconnect();
+
         msg_box.setDefaultButton(watch_button);
         msg_box.exec();
 
-        if (msg_box.clickedButton() == &cb)
+        if (cb.isChecked())
         {
             st->setIntroDialogEnabled(false);
         }
@@ -680,39 +677,20 @@ void FWWindow::fileOpen()
     QString dir;
     QMdiSubWindow *last_active_window = m_mainWindow->m_space->activeSubWindow();
 
-/*
- * Pick default directory where to look for the file to open.
- * 1) if "work directory" is configured in preferences, always use it
- * 2) if it is blank, use the same directory where currently opened file is
- * 3) if this is the first file to be opened, get directory where the user opened
- *    during last session from settings using st->getOpenFileDir
- */
-
-    dir = st->getWDir();
-    if (fwbdebug) qDebug("Choosing directory for file open 1: %s",
-                         dir.toStdString().c_str());
-
-    if (dir.isEmpty() && !mw->getCurrentFileName().isEmpty())
-        dir = getFileDir(mw->getCurrentFileName());
-    if (fwbdebug) qDebug("Choosing directory for file open 2: %s",
-                         dir.toStdString().c_str());
-
-    if (dir.isEmpty()) dir = st->getOpenFileDir();
-    if (fwbdebug) qDebug("Choosing directory for file open 3: %s",
-                         dir.toStdString().c_str());
-
     QString file_name = QFileDialog::getOpenFileName(
         this,
         tr("Open File"),
-        dir,
+        st->getOpenFileDir(mw->getCurrentFileName()),
         "FWB files (*.fwb *.fwl *.xml);;All Files (*)");
 
     if (file_name.isEmpty())
     {
         m_mainWindow->m_space->setActiveSubWindow(last_active_window);
-        return ;
+        return;
     }
 
+    st->setOpenFileDir(file_name);
+
     // Using absoluteFilePath(), see #1334
     QFileInfo fi(file_name);
     QString file_path = fi.absoluteFilePath();
@@ -778,32 +756,23 @@ QMdiSubWindow* FWWindow::alreadyOpened(const QString &file_name)
 
 bool FWWindow::loadFile(const QString &file_name, bool load_rcs_head)
 {
-    ProjectPanel *proj;
-    // if the only project panel window that we have shows
-    // default object tree (i.e. its filename is empty), then load file
-    // into. Otherwise create new project window.
-    //
-    // However if the only project panel has default tree with unsaved
-    // changes then we open new project window.
+    /* We always create new project panel, deleting the old one if
+       it's the default (standard objects) one with no changes.
+       Otherwise we leave behind traces of the old project panel. */
 
-    proj = activeProject();
-
-    if (proj && proj->getFileName().isEmpty() &&
-        (proj->db() == NULL || !proj->db()->isDirty()))
-    {
-        if (!proj->loadFile(file_name, load_rcs_head)) return false;
-    } else
-    {
-        proj = newProjectPanel();
-        if (proj->loadFile(file_name, load_rcs_head))
-        {
-            showSub(proj);
-        } else
-        {
-            delete proj;
-            return false;
+    ProjectPanel *oldProj = activeProject();
+    ProjectPanel *proj = newProjectPanel();
+    if (proj->loadFile(file_name, load_rcs_head)) {
+        if (oldProj != 0 && oldProj->getFileName().isEmpty() &&
+            (oldProj->db() == NULL || !oldProj->db()->isDirty())) {
+            oldProj->fileClose();
         }
+        showSub(proj);
+    } else {
+        delete proj;
+        return false;
     }
+
     proj->readyStatus(true);
     proj->loadState(true);
     return true;
@@ -949,7 +918,7 @@ void FWWindow::prepareViewMenu()
     m_mainWindow->actionEditor_panel->setEnabled(true);
 
     m_mainWindow->actionObject_Tree->setChecked(
-        activeProject()->m_panel->treeDockWidget->isVisible());
+        activeProject()->m_panel->treePanelFrame->isVisible());
     m_mainWindow->actionEditor_panel->setChecked(
         m_mainWindow->editorDockWidget->isVisible());
     m_mainWindow->actionUndo_view->setChecked(
@@ -1784,16 +1753,15 @@ void FWWindow::inspect()
     }
 }
 
-void FWWindow::transferfw(set<Firewall*> vf)
+void FWWindow::addNewObjectMenu(QMenu *m)
 {
-    transferDialog *ed = new transferDialog(NULL, vf);
-    ed->show();
+    QMenu *old_menu = m_mainWindow->newObjectAction->menu();
+    if (old_menu) delete old_menu;
+    m_mainWindow->newObjectAction->setMenu( m );
 }
 
-void FWWindow::transferfw()
+void FWWindow::showNewObjectMenu()
 {
-    std::set<Firewall*> emp;
-    transferDialog *ed = new transferDialog(NULL, emp);
-    ed->show();
+    m_mainWindow->newObjectAction->menu()->popup(QCursor::pos());
 }
 
