@@ -26,16 +26,20 @@
 #include "../../config.h"
 #include "global.h"
 #include "utils.h"
+#include "events.h"
 
 #include <ui_FWBMainWindow_q.h>
 
+#include "FWBTree.h"
 #include "FWWindow.h"
 #include "FindObjectWidget.h"
 #include "FindWhereUsedWidget.h"
-#include "FWBTree.h"
+#include "RuleSetModel.h"
+#include "RuleSetView.h"
 #include "platforms.h"
 
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Policy.h"
 #include "fwbuilder/RuleSet.h"
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/Library.h"
@@ -115,6 +119,8 @@ void FWWindow::openEditorPanel()
 
 void FWWindow::openEditor(FWObject *obj)
 {
+    if (FWBTree().isSystem(obj)) return;
+
     attachEditorToProjectPanel(activeProject());
 
     /*
@@ -168,6 +174,46 @@ void FWWindow::openEditor(FWObject *obj)
     oe->open(obj);
     m_mainWindow->objectEditorStack->resize(old_size);
 
+    // #2465 If the object we are about to open in the editor is a
+    // firewall and if a ruleset visible in RuleSetView belongs to
+    // another firewall, switch ruleset to the ruleset of the new
+    // firewall which we looked at last time.
+    //
+    FWObject *parent_fw = Host::getParentHost(obj);
+
+    if (fwbdebug)
+        qDebug() << "parent firewall:" << parent_fw
+                 << QString((parent_fw)? parent_fw->getName().c_str() : "");
+
+    if (parent_fw != NULL)  // this includes Cluster
+    {
+        RuleSetView* rsv = activeProject()->getCurrentRuleSetView();
+        if (rsv)
+        {
+            RuleSet* current_ruleset = NULL;
+            RuleSetModel* md = NULL;
+            if (rsv)
+            {
+                md = (RuleSetModel*)rsv->model();
+                current_ruleset = md->getRuleSet();
+            }
+            if (parent_fw != current_ruleset->getParent())
+            {
+                FWObject *old_rs =
+                    activeProject()->m_panel->om->findRuleSetInHistoryByParentFw(
+                        parent_fw);
+
+                if (old_rs == NULL)
+                    old_rs = parent_fw->getFirstByType(Policy::TYPENAME);
+
+                if (old_rs != NULL)
+                    QCoreApplication::postEvent(
+                        activeProject(), new openRulesetImmediatelyEvent(
+                            activeProject()->getFileName(), old_rs->getId()));
+            }
+        }
+    }
+
     if (reopen)
     {
         if (fwbdebug)
