@@ -83,6 +83,8 @@ InterfaceDialog::InterfaceDialog(QWidget *parent) :
     netzone->hide();     netzoneLabel->hide();
 */
     obj=NULL;
+
+    connectSignalsOfAllWidgetsToSlotChange();
 }
 
 InterfaceDialog::~InterfaceDialog()
@@ -91,11 +93,6 @@ InterfaceDialog::~InterfaceDialog()
     delete m_dialog;
 }
 
-void InterfaceDialog::getHelpName(QString *str)
-{
-    *str = "InterfaceDialog";
-}
-
 void InterfaceDialog::loadFWObject(FWObject *o)
 {
     obj=o;
@@ -122,11 +119,17 @@ void InterfaceDialog::loadFWObject(FWObject *o)
          * something relevant in the interface to complement their changes
          * and right after the interface has been created.
          */
-        interfaceProperties *int_prop =
-            interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-                s->getParentHost());
-        int_prop->guessSubInterfaceTypeAndAttributes(s);
-        delete int_prop;
+        FWObject *parent_host = Host::getParentHost(s);
+        if (parent_host)
+        {
+            // parent_host may be NULL if interface object is located
+            // in the Deleted Objects library
+            interfaceProperties *int_prop =
+                interfacePropertiesObjectFactory::getInterfacePropertiesObject(
+                    parent_host);
+            int_prop->guessSubInterfaceTypeAndAttributes(s);
+            delete int_prop;
+        }
     }
 
     m_dialog->obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
@@ -142,14 +145,147 @@ void InterfaceDialog::loadFWObject(FWObject *o)
 
     m_dialog->management->setChecked( s->isManagement() );
 
-    m_dialog->comment->setText( QString::fromUtf8(s->getComment().c_str()) );
+    m_dialog->commentKeywords->loadFWObject(s);
+
+    if (s->isBridgePort())
+    {
+        m_dialog->regular->hide();
+        m_dialog->dynamic->hide();
+        m_dialog->unnumbered->hide();
+        m_dialog->management->hide();
+        m_dialog->unprotected->hide();
+        m_dialog->dedicated_failover->hide();
+        m_dialog->bridge_port_label->show();
+    } else
+    {
+        m_dialog->regular->show();
+        m_dialog->dynamic->show();
+        m_dialog->unnumbered->show();
+        m_dialog->management->show();
+        m_dialog->unprotected->show();
+        m_dialog->dedicated_failover->show();
+        m_dialog->bridge_port_label->hide();
+    }
+
+    FWObject *f = Host::getParentHost(s);
+    if (f)
+    {
+        m_dialog->advancedconfig->setEnabled(true);
+
+/* if parent is a host, hide firewall related settings */
+        if (Host::isA(f))
+        {
+            m_dialog->management->setEnabled(false);
+            m_dialog->unprotected->setEnabled(false);
+            m_dialog->dedicated_failover->setEnabled(false);
+            m_dialog->seclevel->setEnabled(false);
+            m_dialog->seclevelLabel->setEnabled(false);
+            m_dialog->netzone->setEnabled(false);
+            m_dialog->netzoneLabel->setEnabled(false);
+            // Can;t let user try to open "advanced interface settings"
+            // dialog because Host does not have "platform" and "host_OS"
+            // attributes but that dialog depends on them.
+            m_dialog->advancedconfig->setEnabled(false);
+        }
+
+        bool supports_security_levels = false;
+        bool supports_network_zones   = false;
+        bool supports_unprotected     = false;
+        bool supports_advanced_ifaces = false;
+
+        try 
+        {
+            // platform specific
+            supports_security_levels =
+                Resources::getTargetCapabilityBool(
+                    f->getStr("platform"), "security_levels");
+            supports_network_zones =
+                Resources::getTargetCapabilityBool(
+                    f->getStr("platform"), "network_zones");
+            supports_unprotected =
+                Resources::getTargetCapabilityBool(
+                    f->getStr("platform"), "unprotected_interfaces");
+
+            // OS specific
+            supports_advanced_ifaces =
+                Resources::getTargetCapabilityBool(
+                    f->getStr("host_OS"), "supports_advanced_interface_options");
+
+            // disable advanced options dialog if this is main interface of a cluster
+            if (Cluster::isA(s->getParent())) supports_advanced_ifaces = false;
+
+        } catch (FWException &ex)  { }
+
+        if (fwbdebug)
+            qDebug() << "parent=" << f->getName().c_str()
+                     << "Firewall::isA(f)=" << Firewall::isA(f)
+                     << "host_OS=" << f->getStr("host_OS").c_str()
+                     << "supports_advanced_ifaces=" << supports_advanced_ifaces;
+
+
+/* if parent is a firewall or a fw cluster, it is more complex ... */
+        if (Firewall::isA(f) || Cluster::isA(f))
+        {
+            if (supports_security_levels)
+            {
+                m_dialog->seclevel->setEnabled(true);
+                m_dialog->seclevelLabel->setEnabled(true);
+                m_dialog->seclevel->setValue( obj->getInt("security_level") );
+            } else
+            {
+                m_dialog->seclevel->setEnabled(false);
+                m_dialog->seclevelLabel->setEnabled(false);
+                m_dialog->seclevel->setValue(0);
+            }
+
+            if (supports_unprotected)
+            {
+                m_dialog->unprotected->setEnabled(true);
+                m_dialog->unprotected->setChecked( obj->getBool("unprotected") );
+            } else
+            {
+                m_dialog->unprotected->setEnabled(false);
+            }
+
+            if (supports_advanced_ifaces)
+            {
+                m_dialog->advancedconfig->setEnabled(!o->isReadOnly());
+            } else
+            {
+                m_dialog->advancedconfig->setEnabled(false);
+            }
+
+            // disable interface options group if this is main interface
+            // of a cluster. This applies to subinterfaces as
+            // well. Current implementation can not generate configuration
+            // code for interfaces and subinterfaces of member firewalls
+            // from cluster interface or subinterface objects
+            m_dialog->interfaceOptionsGroup->setEnabled(!Cluster::isA(f));
+
+            if (supports_network_zones)
+            {
+                m_dialog->netzone->setEnabled(true);
+                m_dialog->netzoneLabel->setEnabled(true);
+
+                netzone_manager->load(m_project->db());
+
+                int id = FWObjectDatabase::getIntId(obj->getStr("network_zone"));
+                if (id==-1) id = 0;
+
+                netzone_manager->packComboBox(m_dialog->netzone, id);
+            }
+            else
+            {
+                m_dialog->netzone->setEnabled(false);
+                m_dialog->netzoneLabel->setEnabled(false);
+                m_dialog->netzone->clear();
+            }
+        }
+    }
 
     m_dialog->obj_name->setEnabled(!o->isReadOnly());
     setDisabledPalette(m_dialog->obj_name);
 
-    m_dialog->comment->setReadOnly(o->isReadOnly());
-    setDisabledPalette(m_dialog->comment);
-
     m_dialog->label->setEnabled(!o->isReadOnly());
     setDisabledPalette(m_dialog->label);
 
@@ -177,140 +313,6 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     m_dialog->netzone->setEnabled(!o->isReadOnly());
     setDisabledPalette(m_dialog->netzone);
 
-    if (s->isBridgePort())
-    {
-        m_dialog->regular->hide();
-        m_dialog->dynamic->hide();
-        m_dialog->unnumbered->hide();
-        m_dialog->management->hide();
-        m_dialog->unprotected->hide();
-        m_dialog->dedicated_failover->hide();
-        m_dialog->bridge_port_label->show();
-    } else
-    {
-        m_dialog->regular->show();
-        m_dialog->dynamic->show();
-        m_dialog->unnumbered->show();
-        m_dialog->management->show();
-        m_dialog->unprotected->show();
-        m_dialog->dedicated_failover->show();
-        m_dialog->bridge_port_label->hide();
-    }
-
-    FWObject *f = s->getParentHost();
-
-    m_dialog->advancedconfig->setEnabled(true);
-
-/* if parent is a host, hide firewall related settings */
-    if (Host::isA(f))
-    {
-        m_dialog->management->setEnabled(false);
-        m_dialog->unprotected->setEnabled(false);
-        m_dialog->dedicated_failover->setEnabled(false);
-        m_dialog->seclevel->setEnabled(false);
-        m_dialog->seclevelLabel->setEnabled(false);
-        m_dialog->netzone->setEnabled(false);
-        m_dialog->netzoneLabel->setEnabled(false);
-        // Can;t let user try to open "advanced interface settings"
-        // dialog because Host does not have "platform" and "host_OS"
-        // attributes but that dialog depends on them.
-        m_dialog->advancedconfig->setEnabled(false);
-    }
-
-    bool supports_security_levels = false;
-    bool supports_network_zones   = false;
-    bool supports_unprotected     = false;
-    bool supports_advanced_ifaces = false;
-
-    try 
-    {
-        // platform specific
-        supports_security_levels =
-            Resources::getTargetCapabilityBool(
-                f->getStr("platform"), "security_levels");
-        supports_network_zones =
-            Resources::getTargetCapabilityBool(
-                f->getStr("platform"), "network_zones");
-        supports_unprotected =
-            Resources::getTargetCapabilityBool(
-                f->getStr("platform"), "unprotected_interfaces");
-
-        // OS specific
-        supports_advanced_ifaces =
-            Resources::getTargetCapabilityBool(
-                f->getStr("host_OS"), "supports_advanced_interface_options");
-
-        // disable advanced options dialog if this is main interface of a cluster
-        if (Cluster::isA(s->getParent())) supports_advanced_ifaces = false;
-
-    } catch (FWException &ex)  { }
-
-    if (fwbdebug)
-        qDebug() << "parent=" << f->getName().c_str()
-                 << "Firewall::isA(f)=" << Firewall::isA(f)
-                 << "host_OS=" << f->getStr("host_OS").c_str()
-                 << "supports_advanced_ifaces=" << supports_advanced_ifaces;
-
-
-/* if parent is a firewall or a fw cluster, it is more complex ... */
-    if (Firewall::isA(f) || Cluster::isA(f))
-    {
-        if (supports_security_levels)
-        {
-            m_dialog->seclevel->setEnabled(true);
-            m_dialog->seclevelLabel->setEnabled(true);
-            m_dialog->seclevel->setValue( obj->getInt("security_level") );
-        } else
-        {
-            m_dialog->seclevel->setEnabled(false);
-            m_dialog->seclevelLabel->setEnabled(false);
-            m_dialog->seclevel->setValue(0);
-        }
-
-        if (supports_unprotected)
-        {
-            m_dialog->unprotected->setEnabled(true);
-            m_dialog->unprotected->setChecked( obj->getBool("unprotected") );
-        } else
-        {
-            m_dialog->unprotected->setEnabled(false);
-        }
-
-        if (supports_advanced_ifaces)
-        {
-            m_dialog->advancedconfig->setEnabled(!o->isReadOnly());
-        } else
-        {
-            m_dialog->advancedconfig->setEnabled(false);
-        }
-
-        // disable interface options group if this is main interface
-        // of a cluster. This applies to subinterfaces as
-        // well. Current implementation can not generate configuration
-        // code for interfaces and subinterfaces of member firewalls
-        // from cluster interface or subinterface objects
-        m_dialog->interfaceOptionsGroup->setEnabled(!Cluster::isA(f));
-
-        if (supports_network_zones)
-        {
-            m_dialog->netzone->setEnabled(true);
-            m_dialog->netzoneLabel->setEnabled(true);
-
-            netzone_manager->load(m_project->db());
-
-            int id = FWObjectDatabase::getIntId(obj->getStr("network_zone"));
-            if (id==-1) id = 0;
-
-            netzone_manager->packComboBox(m_dialog->netzone, id);
-        }
-        else
-        {
-            m_dialog->netzone->setEnabled(false);
-            m_dialog->netzoneLabel->setEnabled(false);
-            m_dialog->netzone->clear();
-        }
-    }
-
     //apply->setEnabled( false );
 
     init = false;
@@ -329,9 +331,10 @@ void InterfaceDialog::validate(bool *res)
         return;
     }
 
+    FWObject *parent_host = Host::getParentHost(obj);
     interfaceProperties *int_prop =
         interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-            Interface::cast(obj)->getParentHost());
+            parent_host);
     QString err;
 
     /*
@@ -438,7 +441,7 @@ void InterfaceDialog::applyChanges()
     string oldname = obj->getName();
     string oldlabel = intf->getLabel();
     new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    new_state->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
+    m_dialog->commentKeywords->applyChanges(new_state);
 
     intf->setLabel( string(m_dialog->label->text().toUtf8().constData()) );
     intf->setDyn( m_dialog->dynamic->isChecked() );
@@ -448,7 +451,8 @@ void InterfaceDialog::applyChanges()
     // NOTE: new_state is a copy of the interface but it is not attached to
     // the tree and therefore has no parent. Need to use original object obj
     // to get the pointer to the parent firewall.
-    FWObject *f = Interface::cast(obj)->getParentHost();
+    FWObject *f = Host::getParentHost(obj);
+    //FWObject *f = Interface::cast(obj)->getParentHost();
     bool supports_security_levels = false;
     bool supports_network_zones   = false;
     bool supports_unprotected     = false;
@@ -503,9 +507,10 @@ void InterfaceDialog::applyChanges()
         {
             // ticket #328: automatically assign vlan id to interface based on
             // interface name
+            FWObject *parent_host = Host::getParentHost(obj);
             interfaceProperties *int_prop =
                 interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-                    Interface::cast(obj)->getParentHost());
+                    parent_host);
             int_prop->setPerformVlanChecks(true);
             int_prop->guessSubInterfaceTypeAndAttributes(intf);
             delete int_prop;
