@@ -63,6 +63,7 @@
 #include "fwbuilder/StateSyncClusterGroup.h"
 #include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/Host.h"
+#include "fwbuilder/AttachedNetworks.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/NetworkIPv6.h"
 #include "fwbuilder/IPv4.h"
@@ -71,6 +72,7 @@
 #include "fwbuilder/AddressTable.h"
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/ObjectGroup.h"
+#include "fwbuilder/DynamicGroup.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/NAT.h"
 #include "fwbuilder/Routing.h"
@@ -106,7 +108,6 @@ using namespace libfwbuilder;
 ObjectEditor::ObjectEditor( QWidget *parent):
     QObject(parent), opened(0), current_dialog_idx(-1), current_dialog_name(""),
     editorStack(dynamic_cast<QStackedWidget*>(parent)),
-    helpButton(0),
     m_project(0),
     openedOpt(optNone)
 {
@@ -140,6 +141,7 @@ ObjectEditor::ObjectEditor( QWidget *parent):
     registerObjectDialog(editorStack, StateSyncClusterGroup::TYPENAME,
                          "w_StateSyncClusterGroupDialog");
     registerObjectDialog(editorStack, Host::TYPENAME, "w_HostDialog");
+    registerObjectDialog(editorStack, AttachedNetworks::TYPENAME, "w_AttachedNetworksDialog");
     registerObjectDialog(editorStack, Network::TYPENAME, "w_NetworkDialog");
     registerObjectDialog(editorStack, NetworkIPv6::TYPENAME, "w_NetworkDialogIPv6");
     registerObjectDialog(editorStack, CustomService::TYPENAME, "w_CustomServiceDialog");
@@ -151,6 +153,7 @@ ObjectEditor::ObjectEditor( QWidget *parent):
     registerObjectDialog(editorStack, TagService::TYPENAME, "w_TagServiceDialog");
     registerObjectDialog(editorStack, ServiceGroup::TYPENAME, "w_ServiceGroupDialog");
     registerObjectDialog(editorStack, ObjectGroup::TYPENAME, "w_ObjectGroupDialog");
+    registerObjectDialog(editorStack, DynamicGroup::TYPENAME, "w_DynamicGroupDialog");
     registerObjectDialog(editorStack, IntervalGroup::TYPENAME, "w_IntervalGroupDialog");
     registerObjectDialog(editorStack, Interval::TYPENAME, "w_TimeDialog");
 
@@ -252,7 +255,6 @@ void ObjectEditor::activateDialog(const QString &dialog_name,
     openedOpt = opt;
 
     load();
-    findAndLoadHelp();
 
     //show();
 
@@ -267,10 +269,6 @@ void ObjectEditor::activateDialog(const QString &dialog_name,
     connect(dialogs[ current_dialog_idx ], SIGNAL(changed_sign()),
             this,
             SLOT(changed()));
-
-    connect(this, SIGNAL(getHelpName_sign(QString*)),
-            dialogs[ current_dialog_idx ],
-            SLOT(getHelpName(QString*)));
 }
 
 void ObjectEditor::open(FWObject *obj)
@@ -309,7 +307,6 @@ void ObjectEditor::disconnectSignals()
     disconnect( SIGNAL(loadObject_sign(libfwbuilder::FWObject*)) );
     disconnect( SIGNAL(validate_sign(bool*)) );
     disconnect( SIGNAL(applyChanges_sign()) );
-    disconnect( SIGNAL(getHelpName_sign(QString*)) );
     if (current_dialog_idx>=0) dialogs[current_dialog_idx]->disconnect( this );
 }
 
@@ -320,35 +317,6 @@ void ObjectEditor::purge()
     openedOpt = optNone;
 }
 
-void ObjectEditor::setHelpButton(QPushButton * b)
-{
-    helpButton=b;
-    helpButton->setEnabled(true);
-    connect((QWidget*)helpButton,SIGNAL(clicked()),this,SLOT(help()));
-}
-
-void ObjectEditor::help()
-{
-    QString help_name;
-    emit getHelpName_sign(&help_name);
-    if (fwbdebug)
-        qDebug("ObjectEditor::help: %s", help_name.toStdString().c_str());
-    Help *h = Help::getHelpWindow(mw);
-    h->setSource(QUrl(help_name + ".html"));
-    h->raise();
-    h->show();
-}
-
-void ObjectEditor::findAndLoadHelp()
-{
-    QString help_name;
-    emit getHelpName_sign(&help_name);
-
-    // is help available?
-    Help *h = Help::getHelpWindow(mw);
-    helpButton->setEnabled(!h->findHelpFile(help_name + ".html").isEmpty());
-}
-
 void ObjectEditor::load()
 {
     emit loadObject_sign(opened);
