@@ -52,9 +52,11 @@
 
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/AttachedNetworks.h"
 #include "fwbuilder/Cluster.h"
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/DNSName.h"
+#include "fwbuilder/DynamicGroup.h"
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/Firewall.h"
@@ -130,7 +132,9 @@ ObjectManipulator::ObjectManipulator(QWidget *parent):
     // used in duplicateWithDependencies()
     dedup_marker_global_counter = time(NULL);
 
-    buildNewObjectMenu();
+    popup_menu = NULL;
+
+//    buildNewObjectMenu();
 
 }
 
@@ -341,11 +345,76 @@ QString ObjectManipulator::makeNameUnique(FWObject* parent,
     return newname;
 }
 
+static void addKeywordsMenu(ObjectManipulator *om, QMenu *menu)
+{
+    QMenu *keywordsMenu = menu->addMenu(om->tr("Keywords"));
+    QMenu *addKeywords = keywordsMenu->addMenu(om->tr("Add"));
+
+    addKeywords->addAction(om->tr("New Keyword..."),
+                           om, SLOT(addNewKeywordSlot()));
+    addKeywords->addSeparator();
+
+    QStringList addList;
+    const set<string> &allKeywords = om->getSelectedObject()->getAllKeywords();
+    set<string>::const_iterator iterz;
+    for (iterz = allKeywords.begin(); iterz != allKeywords.end(); ++iterz) {
+        addList.append(QString::fromUtf8((*iterz).c_str()));
+    }
+    addList = sortStrings(addList);
+
+    QStringList data;
+    data << "add" << "";
+    foreach (QString add, addList) {
+        QAction *act =
+            addKeywords->addAction(add, om, SLOT(processKeywordSlot()));
+        data[1] = add;
+        act->setData(data);
+    }
+
+    bool allLocked = true;
+    QMenu *removeKeywords = keywordsMenu->addMenu(om->tr("Remove"));
+    QSet<QString> toRemove;
+    foreach (FWObject *obj, om->getCurrentObjectTree()->getSelectedObjects()) {
+        if (obj->isReadOnly()) continue;
+        allLocked = false;
+
+        const set<string> &keywords = obj->getKeywords();
+        set<string>::const_iterator iter;
+        for (iter = keywords.begin(); iter != keywords.end(); ++iter) {
+            toRemove.insert(QString::fromUtf8((*iter).c_str()));
+        }
+    }
+
+    if (toRemove.isEmpty()) {
+        removeKeywords->setDisabled(true);
+    } else {
+        data[0] = "remove";
+        foreach (QString str, sortStrings(toRemove.toList())) {
+            QAction *act =
+                removeKeywords->addAction(str, om, SLOT(processKeywordSlot()));
+            data[1] = str;
+            act->setData(data);
+        }
+    }
+
+    if (allLocked) {
+        keywordsMenu->setDisabled(true);
+    }
+}
+
+
 void ObjectManipulator::contextMenuRequested(const QPoint &pos)
 {
+    if (popup_menu == NULL)
+    {
+        popup_menu = new QMenu(this);
+        popup_menu->setObjectName("objectTreeContextMenu");
+    } else
+        popup_menu->clear();
+
 /* in extended selection mode there may be several selected items */
 
-    QTreeWidget *objTreeView = getCurrentObjectTree();
+    ObjectTreeView *objTreeView = getCurrentObjectTree();
     QTreeWidgetItem *item = objTreeView->itemAt(pos);//clicked item
 
     if (fwbdebug)
@@ -355,14 +424,25 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
     ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(item);
     if (otvi==NULL)  return;  // happens when user clicks outside an item
 
+    FWObject *obj = otvi->getFWObject();
+    if (obj == 0) {
+        assert(otvi->getUserFolderParent() != 0);
+        QAction *action =
+            popup_menu->addAction(tr("Delete"), this, SLOT(removeUserFolder()));
+        /* The user-defined folder doesn't get counted as a selected obj */
+        if (objTreeView->getNumSelected() > 0) {
+            action->setEnabled(false);
+        }
+        popup_menu->exec(QCursor::pos());
+        return;
+    }
+
     if (!getCurrentObjectTree()->isSelected(otvi->getFWObject()))
         openObjectInTree( otvi, true );
 
     //if (currentObj==NULL)  currentObj=otvi->getFWObject();
     FWObject *currentObj = getSelectedObject();
 
-    popup_menu->clear();
-
     if (item->childCount() > 0)
     {
         if (item->isExpanded())
@@ -468,7 +548,7 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
     
     if (getCurrentObjectTree()->getNumSelected()==1)
     {
-
+        bool addSubfolder = false;
         if ( (Firewall::isA(currentObj) || Host::isA(currentObj)) &&
              ! currentObj->isReadOnly() )
         {
@@ -488,7 +568,8 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
         if (Interface::isA(currentObj) && ! currentObj->isReadOnly())
         {
             Interface *iface = Interface::cast(currentObj);
-            FWObject *h = iface->getParentHost();
+            FWObject *h = Host::getParentHost(iface);
+            //FWObject *h = iface->getParentHost();
 
             bool supports_advanced_ifaces = false;
             try {
@@ -524,6 +605,20 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, physAddress::TYPENAME));
 
+            /*
+             * Add menu item to let user add AttachedNetworks object
+             * to an interface, but only if this object does not exist yet.
+             *
+             * Actions added to AddObjectActions are
+             * enabled and disabled all together based on the decision
+             * made in getMenuState() (argument newMenuItem). But we
+             * should always allow the user to add AttachedNetworks
+             * object to an interface.
+             */
+            FWObject *att = currentObj->getFirstByType(AttachedNetworks::TYPENAME);
+            if (att == NULL)
+                addNewObjectMenuItem(popup_menu, AttachedNetworks::TYPENAME);
+
 
             // Check if we should add menu item that creates failover
             // group. if parent is a cluster, allow one vrrp type
@@ -557,16 +652,21 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
                     popup_menu, StateSyncClusterGroup::TYPENAME));
         }
 
-        if (currentObj->getPath(true)=="Firewalls")
+        if (currentObj->getPath(true)=="Firewalls") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, Firewall::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Clusters")
+        if (currentObj->getPath(true)=="Clusters") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, Cluster::TYPENAME));
+        }
 
         if (currentObj->getPath(true)=="Objects/Addresses")
         {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, IPv4::TYPENAME));
             AddObjectActions.append(
@@ -575,75 +675,112 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
 
         if (currentObj->getPath(true)=="Objects/DNS Names")
         {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, DNSName::TYPENAME));
         }
 
         if (currentObj->getPath(true)=="Objects/Address Tables")
         {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, AddressTable::TYPENAME));
         }
 
-        if (currentObj->getPath(true)=="Objects/Address Ranges")
+        if (currentObj->getPath(true)=="Objects/Address Ranges") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, AddressRange::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Objects/Hosts")
+        if (currentObj->getPath(true)=="Objects/Hosts") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, Host::TYPENAME));
+        }
 
         if (currentObj->getPath(true)=="Objects/Networks")
         {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, Network::TYPENAME));
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, NetworkIPv6::TYPENAME));
         }
 
-        if (currentObj->getPath(true)=="Objects/Groups")
+        if (currentObj->getPath(true)=="Objects/Groups") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, ObjectGroup::TYPENAME));
+            AddObjectActions.append(
+                addNewObjectMenuItem(popup_menu, DynamicGroup::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Services/Custom")
+        if (currentObj->getPath(true)=="Services/Custom") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, CustomService::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Services/IP")
+        if (currentObj->getPath(true)=="Services/IP") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, IPService::TYPENAME));
+        }
 
         if (currentObj->getPath(true)=="Services/ICMP")
         {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, ICMPService::TYPENAME));
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, ICMP6Service::TYPENAME));
         }
 
-        if (currentObj->getPath(true)=="Services/TCP")
+        if (currentObj->getPath(true)=="Services/TCP") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, TCPService::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Services/UDP")
+        if (currentObj->getPath(true)=="Services/UDP") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, UDPService::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Services/TagServices")
+        if (currentObj->getPath(true)=="Services/TagServices") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, TagService::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Services/Groups")
+        if (currentObj->getPath(true)=="Services/Groups") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, ServiceGroup::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Services/Users")
+        if (currentObj->getPath(true)=="Services/Users") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, UserService::TYPENAME));
+        }
 
-        if (currentObj->getPath(true)=="Time")
+        if (currentObj->getPath(true)=="Time") {
+            addSubfolder = true;
             AddObjectActions.append(
                 addNewObjectMenuItem(popup_menu, Interval::TYPENAME));
+        }
+
+        if (addSubfolder) {
+            QAction *action =
+                popup_menu->addAction(QIcon(":/Icons/SystemGroup/icon-tree"),
+                                      tr("New Subfolder"), this,
+                                      SLOT(addSubfolderSlot()));
+            action->setData(currentObj->getId());
+            AddObjectActions.append(action);
+        }
 
         popup_menu->addSeparator();
 
@@ -657,11 +794,13 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
         findID->setEnabled( !FWBTree().isStandardFolder(currentObj));
         whereUsedID->setEnabled( !FWBTree().isStandardFolder(currentObj));
     }
-    
+
     popup_menu->addSeparator();
     popup_menu->addAction( tr("Group"), this, SLOT( groupObjects() ) )
             ->setDisabled(getCurrentObjectTree()->getNumSelected()==1);
 
+    addKeywordsMenu(this, popup_menu);
+
     if (Firewall::cast(currentObj)!=NULL ||
         (ObjectGroup::cast(currentObj)!=NULL &&
          currentObj->getName()=="Firewalls"))
@@ -684,38 +823,15 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
         popup_menu->addAction( tr("Compile"), this, SLOT( compile()));
         popup_menu->addAction( tr("Install"), this, SLOT( install()));
         popup_menu->addAction( tr("Inspect"), this, SLOT( inspect()));
+    }
 
-        if (Firewall::cast(currentObj)!=NULL)
-        {
-            Resources* os_res = Resources::os_res[currentObj->getStr("host_OS")];
-            if (os_res)
-            {
-                string transfer = os_res->getTransferAgent();
-                if (!transfer.empty())
-                    popup_menu->addAction( tr("Transfer"), this, SLOT(transferfw()));
-            }
-        }
-
-        if (ObjectGroup::cast(currentObj)!=NULL &&
-            currentObj->getName()=="Firewalls")
-        {
-            // Config transfer is currently only supported for Secuwall.
-            // Check if we have any
-            bool have_transfer_support = false;
-            for (FWObject::iterator it=currentObj->begin();
-                 it!=currentObj->end(); ++it)
-            {
-                FWObject *fw = *it;
-                Resources* os_res = Resources::os_res[fw->getStr("host_OS")];
-                if (os_res)
-                {
-                    string transfer = os_res->getTransferAgent();
-                    have_transfer_support = have_transfer_support || (!transfer.empty());
-                }
-            }
-            if (have_transfer_support)
-                popup_menu->addAction( tr("Transfer"), this, SLOT(transferfw()));
-        }
+    if (Interface::cast(currentObj)!=NULL)
+    {
+        popup_menu->addSeparator();
+        FWObject *h = Host::getParentHost(currentObj);
+        list<FWObject*> top_level_interfaces = h->getByType(Interface::TYPENAME);
+        top_level_interfaces.sort(FWObjectNameCmpPredicate());
+        addSubinterfaceSubmenu(popup_menu, top_level_interfaces);
     }
 
     popup_menu->addSeparator();
@@ -768,7 +884,66 @@ void ObjectManipulator::contextMenuRequested(const QPoint &pos)
 
 //    if (inDeletedObjects) movID->setText( tr("Undelete...") );
 
-    popup_menu->exec( objTreeView->mapToGlobal( pos ) );
+    popup_menu->exec(QCursor::pos());
+}
+
+/*
+ * Add menu item "Make subinterface of ..." and submenu with list of
+ * top level interfaces.
+ */
+void ObjectManipulator::addSubinterfaceSubmenu(
+    QMenu *menu,
+    const list<FWObject*> &top_level_interfaces)
+{
+    QMenu *submenu = menu->addMenu( tr("Make subinterface of..."));
+    
+    int submenu_items_counter = 0;
+    list<FWObject*>::const_iterator it;
+    for (it=top_level_interfaces.begin(); it!=top_level_interfaces.end(); ++it)
+    {
+        Interface *intf = Interface::cast(*it);
+
+        bool skip_interface = false;
+        foreach(FWObject *obj, getCurrentObjectTree()->getSelectedObjects())
+        {
+            if (obj == intf)
+            {
+                skip_interface = true;
+                break;
+            }
+
+            if (!intf->validateChild(obj))
+            {
+                skip_interface = true;
+                break;
+            }
+        }
+        if (skip_interface) continue;
+        if (intf->isLoopback()) continue;
+
+        // can not add interfaces to a read-only parent interface
+        if (intf->isReadOnly()) continue; 
+
+        QString itf_name = QString::fromUtf8(intf->getName().c_str());
+        FWObject *parent_fw = Host::getParentHost(intf);
+
+        std::auto_ptr<interfaceProperties> int_prop(
+            interfacePropertiesObjectFactory::getInterfacePropertiesObject(
+                parent_fw));
+        if (int_prop->looksLikeVlanInterface(itf_name)) continue;
+
+        QAction *a = submenu->addAction(
+            QIcon(":/Icons/Interface/icon-tree"), itf_name);
+
+        a->setData(intf->getId());
+
+        connect( submenu, SIGNAL(triggered(QAction*)),
+                 this, SLOT(makeSubinterface(QAction*)));
+
+        submenu_items_counter++;
+    }
+
+    submenu->setEnabled(submenu_items_counter != 0);
 }
 
 bool ObjectManipulator::getDeleteMenuState(FWObject *obj)
@@ -850,6 +1025,16 @@ void ObjectManipulator::getMenuState(bool haveMoveTargets,
             }
         }
 
+        if (AttachedNetworks::isA(obj))
+        {
+            dupMenuItem = false;
+            moveMenuItem = false;
+            copyMenuItem = false;
+            pasteMenuItem = false;
+            newMenuItem = false;
+            continue;
+        }
+
         copyMenuItem = copyMenuItem && FWBTree().getCopyMenuState(object_path);
         pasteMenuItem = pasteMenuItem &&
             FWBTree().getPasteMenuState(object_path) &&
@@ -1146,10 +1331,10 @@ void ObjectManipulator::selectionChanged(QTreeWidgetItem *cur)
     FWObject *o = obj;
     //if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
-    if (history.empty() || otvi!=history.back().item() )
+    if (history.empty() || otvi != getCurrentHistoryItem() )
     {
         mw->enableBackAction();
-        history.push_back( HistoryItem(otvi, o->getId()) );
+        addObjectToHistory(otvi, o);
     }
 
     //currentObj = obj;
@@ -1224,8 +1409,12 @@ void ObjectManipulator::openLibForObject(FWObject *obj)
         m_objectManipulator->libs->setCurrentIndex(
             libs_model->getIdxForLib(obj->getLibrary()).row());
     else
+    {
+        FWObject *parent = obj->getParent();
+        FWObject *lib = parent->getLibrary();
         m_objectManipulator->libs->setCurrentIndex(
-            libs_model->getIdxForLib(obj->getParent()->getLibrary()).row());
+            libs_model->getIdxForLib(lib).row());
+    }
 }
 
 void ObjectManipulator::showObjectInTree(ObjectTreeViewItem *otvi)
@@ -1248,7 +1437,7 @@ void ObjectManipulator::showObjectInTree(ObjectTreeViewItem *otvi)
     otvi->getTree()->scrollToItem(otvi);
     otvi->getTree()->setCurrentItem(otvi);
     otvi->setSelected(true);
-    otvi->getTree()->setFocus(Qt::OtherFocusReason);
+//    otvi->getTree()->setFocus(Qt::OtherFocusReason);
 }
 
 void ObjectManipulator::libChangedById(int obj_id)
@@ -1341,7 +1530,7 @@ void ObjectManipulator::updateCreateObjectMenu(FWObject* lib)
     );
     bool new_object_op_possible = !f;
     emit libraryAccessChanged(new_object_op_possible);
-    m_objectManipulator->newButton->setEnabled(new_object_op_possible);
+//    m_objectManipulator->newButton->setEnabled(new_object_op_possible);
     QAction *noa = (QAction*)(mw->findChild<QAction*>("newObjectAction"));
     noa->setEnabled(new_object_op_possible);
 }
@@ -1365,12 +1554,13 @@ ObjectTreeView* ObjectManipulator::getCurrentObjectTree()
 
 void ObjectManipulator::openLib(FWObject *obj)
 {
-    openObjectInTree(obj->getLibrary(),false);
+    openObjectInTree(obj->getLibrary(), false);
 }
 
 void ObjectManipulator::newObject()
 {
-    m_objectManipulator->newButton->showMenu();
+//     m_objectManipulator->newButton->showMenu();
+    buildNewObjectMenu();
 }
 
 void ObjectManipulator::select()
