@@ -65,7 +65,8 @@ namespace libfwbuilder
 }
 
 
-class HistoryItem {
+class HistoryItem
+{
     ObjectTreeViewItem *itm;
     int                 objId;
 
@@ -76,7 +77,9 @@ class HistoryItem {
     int id() const { return objId; }
 };
 
-class FindHistoryItemByObjectId {
+
+class FindHistoryItemByObjectId
+{
     int id;
 public:
     FindHistoryItemByObjectId(int i) { id = i; }
@@ -94,7 +97,21 @@ class ObjectManipulator : public QWidget
 
     int previous_lib_index;
     QSet<int> ids ;
+
+    /*
+     * this is where we keep information about user's object browsing
+     * history. Even though latest versions of the GUI do not offer
+     * buttons "Back" and "Forward", history is still used to
+     * determine which rule set object to open when user double clicks
+     * on a firewall. We want to open the rule set they looked at
+     * last. This whole browsing history infrastructure might be an
+     * overkill for the simple purpose like that, but I'll keep it
+     * around beause I do not want to make drastic changes so close to
+     * the release date and it may become useful again in the
+     * future. Otherwise, we can get rid of it later. --vk 07/22/2011
+     */
     std::list<HistoryItem> history;
+    std::list<HistoryItem>::iterator current_history_item;
     int cacheHits;
 
     //libfwbuilder::FWObject *currentObj;
@@ -117,7 +134,7 @@ class ObjectManipulator : public QWidget
     int dedup_marker_global_counter;
 
     void buildNewObjectMenu();
-
+    
     ObjectTreeViewItem* insertObject(ObjectTreeViewItem *itm,
                                      libfwbuilder::FWObject *obj);
     void insertSubtree( ObjectTreeViewItem *itm,libfwbuilder::FWObject *obj );
@@ -172,6 +189,7 @@ class ObjectManipulator : public QWidget
     libfwbuilder::FWObject* newPhysicalAddress(QUndoCommand* macro);
     libfwbuilder::FWObject* newPolicyRuleSet (QUndoCommand* macro);
     libfwbuilder::FWObject* newNATRuleSet (QUndoCommand* macro);
+    libfwbuilder::FWObject* newAttachedNetworks(QUndoCommand* macro);
 
    
 public slots:
@@ -186,6 +204,10 @@ public slots:
      
      void selectionChanged(QTreeWidgetItem *cur);
 
+     void removeUserFolder();
+     void moveItems(ObjectTreeViewItem *dest,
+                    const std::list<libfwbuilder::FWObject *> &items);
+
      /**
       * open object obj in the editor. Does not open editor panel
       * if it is closed. Asks FWWindow permission to own editor.
@@ -233,7 +255,8 @@ public slots:
      void newObject();
 
      void createNewObject();
-
+     void newFirewallSlot();
+     
      void copyObj();
      void cutObj();
      void pasteObj();
@@ -242,7 +265,6 @@ public slots:
      void compile();
      void install();
      void inspect();
-     void transferfw();
 
      void duplicateObj(QAction*);
      void moveObj(QAction*);
@@ -257,13 +279,17 @@ public slots:
      void find();
      void findObject();
 
-     virtual void back();
      virtual void lockObject();
      virtual void unlockObject();
      virtual void simulateInstall();
      virtual void findWhereUsedSlot();
 
+     void addSubfolderSlot();
+     void addNewKeywordSlot();
+     void processKeywordSlot();
 
+     void makeSubinterface(QAction*);
+     
 public:
 
      Ui::ObjectManipulator_q *m_objectManipulator;
@@ -301,6 +327,16 @@ public:
      void loadObjects();
      void clearObjects();
 
+     /* Add/remove user folder from tree */
+     void addUserFolderToTree(libfwbuilder::FWObject *obj,
+                              const QString &folder);
+     void removeUserFolderFromTree(libfwbuilder::FWObject *obj,
+                                   const QString &folder);
+     void moveToFromUserFolderInTree(libfwbuilder::FWObject *parent,
+                                     libfwbuilder::FWObject *objToMove,
+                                     const QString &oldFolder,
+                                     const QString &newFolder);
+
      void reopenCurrentItemParent();
 
      void insertSubtree(libfwbuilder::FWObject *parent,
@@ -315,7 +351,18 @@ public:
      void openObjectInTree(ObjectTreeViewItem *otvi,    bool register_in_history);
 
      void removeObjectFromHistory(libfwbuilder::FWObject *obj);
-
+     void addObjectToHistory(ObjectTreeViewItem* otvi,
+                             libfwbuilder::FWObject *obj);
+     ObjectTreeViewItem* getCurrentHistoryItem();
+     
+     /*
+      * searches history trying to find an object that has given
+      * parent. Used to find which rule set of the firewall user
+      * looked at last.
+      */
+     libfwbuilder::FWObject* findRuleSetInHistoryByParentFw(
+         libfwbuilder::FWObject* parent);
+         
      void expandObjectInTree(libfwbuilder::FWObject *obj);
      
      libfwbuilder::FWObject* duplicateObject(libfwbuilder::FWObject *target,
@@ -431,6 +478,10 @@ public:
                                    const QString &text=QString(),
                                    int add_to_group_id=-1);
 
+     void addSubinterfaceSubmenu(
+         QMenu *menu,
+         const std::list<libfwbuilder::FWObject*> &top_level_interfaces);
+     
      bool isObjectAllowed(const QString &type_name);
      bool isObjectAllowed(libfwbuilder::FWObject *target,
                           libfwbuilder::FWObject *obj);
