@@ -45,6 +45,7 @@
 
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/AttachedNetworks.h"
 #include "fwbuilder/Cluster.h"
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/DNSName.h"
@@ -90,10 +91,7 @@ using namespace libfwbuilder;
 
 void ObjectManipulator::buildNewObjectMenu()
 {
-    popup_menu = new QMenu(this);
-    popup_menu->setObjectName("objectTreeContextMenu");
-
-    QMenu* newObjectPopup = new QMenu( this );
+    QMenu* newObjectPopup = new QMenu( mw );
 
     newObjectPopup->setObjectName("newObjectPopup");
 
@@ -101,35 +99,22 @@ void ObjectManipulator::buildNewObjectMenu()
 
     newObjectPopup->addSeparator();
 
-    addNewObjectMenuItem(newObjectPopup, Firewall::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, Cluster::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, Host::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, Network::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, NetworkIPv6::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, IPv4::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, IPv6::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, DNSName::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, AddressTable::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, AddressRange::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, ObjectGroup::TYPENAME);
+    foreach (const char *type, FWBTree::getObjectTypes()) {
+        addNewObjectMenuItem(newObjectPopup, type);
+    }
 
     newObjectPopup->addSeparator();
 
-    addNewObjectMenuItem(newObjectPopup, CustomService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, IPService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, ICMPService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, ICMP6Service::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, TCPService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, UDPService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, TagService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, UserService::TYPENAME);
-    addNewObjectMenuItem(newObjectPopup, ServiceGroup::TYPENAME);
+    foreach (const char *type, FWBTree::getServiceTypes()) {
+        addNewObjectMenuItem(newObjectPopup, type);
+    }
 
     newObjectPopup->addSeparator();
 
     addNewObjectMenuItem(newObjectPopup, Interval::TYPENAME);
 
-    m_objectManipulator->newButton->setMenu( newObjectPopup );
+    mw->addNewObjectMenu(newObjectPopup);
+    mw->showNewObjectMenu();
 }
 
 QAction* ObjectManipulator::addNewObjectMenuItem(QMenu *menu,
@@ -212,6 +197,7 @@ void ObjectManipulator::createNewObject()
     if (type_name ==  Policy::TYPENAME) new_obj = newPolicyRuleSet(macro);
     if (type_name ==  NAT::TYPENAME) new_obj = newNATRuleSet(macro);
     //if (type_name ==  Routing::TYPENAME) new_obj = newRoutingRuleSet();
+    if (type_name ==  AttachedNetworks::TYPENAME) new_obj = newAttachedNetworks(macro);
 
     if (new_obj == NULL) new_obj = createObject(type_name, descr, NULL, macro);
 
@@ -263,6 +249,35 @@ void ObjectManipulator::createNewObject()
     m_project->undoStack->push(macro);
 }
 
+void ObjectManipulator::newFirewallSlot()
+{
+    QString descr = FWBTree().getTranslatableObjectTypeName(Firewall::TYPENAME);
+    // FWCmdMacro should be used for commands grouping
+    FWCmdMacro* macro = 0;
+    macro = new FWCmdMacro(
+        FWBTree().getTranslatableNewObjectMenuText(Firewall::TYPENAME));
+
+    FWObject *new_obj = newFirewall(macro);
+
+    if (new_obj == NULL)
+    {
+        delete macro;
+        return;
+    }
+
+    QCoreApplication::postEvent(
+        m_project, new expandObjectInTreeEvent(
+            m_project->getFileName(), new_obj->getId()));
+
+    FWObject *ruleset = new_obj->getFirstByType(Policy::TYPENAME);
+    if (ruleset)
+        QCoreApplication::postEvent(
+            m_project, new openRulesetEvent(
+                m_project->getFileName(), ruleset->getId()));
+
+    m_project->undoStack->push(macro);
+}
+
 FWObject* ObjectManipulator::createObject(const QString &objType,
                                           const QString &objName,
                                           FWObject *copyFrom,
@@ -652,6 +667,31 @@ FWObject* ObjectManipulator::newFailoverClusterGroup(QUndoCommand* macro)
     return o;
 }
 
+/*
+ * Creates new AttachedNetworks object; this method is called by
+ * context menu item associated with Interface object
+ */
+FWObject* ObjectManipulator::newAttachedNetworks(QUndoCommand* macro)
+{
+    FWObject *currentObj = getSelectedObject();
+    if ( currentObj->isReadOnly() ) return NULL;
+
+    if (Interface::isA(currentObj))
+    {
+        FWObject *no = createObject(currentObj, AttachedNetworks::TYPENAME,
+                                    tr("Attached Networks"), NULL, macro);
+        FWObject *parent_host = Host::getParentHost(currentObj);
+        string name = parent_host->getName() +
+            ":" + currentObj->getName() + ":attached";
+        no->setName(name);
+        return no;
+    } else
+    {
+        qWarning("newAttachedNetworks: invalid currentObj");
+        return NULL;
+    }
+}
+
 FWObject* ObjectManipulator::newHost(QUndoCommand* macro)
 {
     FWObject *parent =
@@ -708,7 +748,8 @@ FWObject* ObjectManipulator::newInterface(QUndoCommand* macro)
 
     if (Interface::isA(currentObj))
     {
-        FWObject *h = Interface::cast(currentObj)->getParentHost();
+        FWObject *h = Host::getParentHost(currentObj);
+        //FWObject *h = Interface::cast(currentObj)->getParentHost();
 
         bool supports_advanced_ifaces = false;
         supports_advanced_ifaces =
@@ -738,9 +779,10 @@ FWObject* ObjectManipulator::newInterface(QUndoCommand* macro)
 
     if (Interface::isA(parent))
     {
+        FWObject *parent_host = Host::getParentHost(parent);
         interfaceProperties *int_prop =
             interfacePropertiesObjectFactory::getInterfacePropertiesObject(
-                Interface::cast(parent)->getParentHost());
+                parent_host);
         int_prop->guessSubInterfaceTypeAndAttributes(new_interface);
         delete int_prop;
         //guessSubInterfaceTypeAndAttributes(new_interface);
