@@ -45,9 +45,11 @@
 #include "FWCmdMoveObject.h"
 #include "FWBTree.h"
 #include "FWWindow.h"
+#include "KeywordsDialog.h"
 #include "ProjectPanel.h"
 #include "ConfirmDeleteObjectDialog.h"
 
+#include "fwbuilder/AttachedNetworks.h"
 #include "fwbuilder/Cluster.h"
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/IPv6.h"
@@ -61,6 +63,7 @@
 
 #include <memory>
 
+#include <QInputDialog>
 #include <QMessageBox>
 #include <QTextEdit>
 #include <QTime>
@@ -120,9 +123,10 @@ void ObjectManipulator::autorename(FWObject *obj)
         list<FWObject*> ipv6_list = obj->getByType(IPv6::TYPENAME);
         list<FWObject*> pa_list = obj->getByType(physAddress::TYPENAME);
         list<FWObject*> failover_list = obj->getByType(FailoverClusterGroup::TYPENAME);
+        list<FWObject*> attached_list = obj->getByType(AttachedNetworks::TYPENAME);
 
         if (ipv4_list.size() || ipv6_list.size() || pa_list.size() ||
-            failover_list.size() || subinterfaces.size())
+            failover_list.size() || subinterfaces.size() || attached_list.size())
         {
             list<FWObject*> vlans;
             for (list<FWObject*>::iterator j=subinterfaces.begin();
@@ -143,6 +147,7 @@ void ObjectManipulator::autorename(FWObject *obj)
             autorename(ipv6_list, IPv6::TYPENAME, "ip6");
             autorename(pa_list, physAddress::TYPENAME, "mac");
             autorename(failover_list, FailoverClusterGroup::TYPENAME, "members");
+            autorename(attached_list, AttachedNetworks::TYPENAME, "attached");
         }
     }
 }
@@ -214,6 +219,12 @@ FWObject* ObjectManipulator::duplicateObject(FWObject *targetLib, FWObject *obj)
 {
     if (!isTreeReadWrite(this, targetLib)) return NULL;
 
+    // we disable copy/cut/paste/duplicate menu items for objects that
+    // can't be copied or duplicated in
+    // ObjectManipulator::getMenuState() but will check here just in
+    // case
+    if (AttachedNetworks::isA(obj)) return NULL;
+
     openLib(targetLib);
     FWObject *new_parent = FWBTree().getStandardSlotForObject(
         targetLib, obj->getTypeName().c_str());
@@ -295,6 +306,12 @@ FWObject* ObjectManipulator::actuallyPasteTo(FWObject *target,
 
     if (!isObjectAllowed(ta, obj)) return NULL;
 
+    // we disable copy/cut/paste/duplicate menu items for objects that
+    // can't be copied or duplicated in
+    // ObjectManipulator::getMenuState() but will check here just in
+    // case
+    if (AttachedNetworks::isA(obj)) return NULL;
+
     if (fwbdebug)
         qDebug() << "ObjectManipulator::actuallyPasteTo"
                  << "target=" << target->getPath().c_str()
@@ -426,7 +443,8 @@ void ObjectManipulator::lockObject()
             if (lib->getId()!=FWObjectDatabase::STANDARD_LIB_ID)
             {
                 std::auto_ptr<FWCmdLockObject> cmd(
-                    new FWCmdLockObject(m_project, obj, QObject::tr("Lock object")));
+                    new FWCmdLockObject(m_project, obj, tr("Lock object ") +
+                                        QString::fromUtf8(obj->getName().c_str())));
                 FWObject* new_state = cmd->getNewState();
                 new_state->setReadOnly(true);
                 if (!cmd->getOldState()->cmp(new_state, true))
@@ -463,7 +481,8 @@ void ObjectManipulator::unlockObject()
             if (lib->getId()!=FWObjectDatabase::STANDARD_LIB_ID)
             {
                 std::auto_ptr<FWCmdLockObject> cmd(
-                    new FWCmdLockObject(m_project, obj, QObject::tr("Unlock object")));
+                    new FWCmdLockObject(m_project, obj, tr("Unlock object ") +
+                                        QString::fromUtf8(obj->getName().c_str())));
                 FWObject* new_state = cmd->getNewState();
                 new_state->setReadOnly(false);
                 if (!cmd->getOldState()->cmp(new_state, true))
@@ -670,3 +689,158 @@ void ObjectManipulator::groupObjects()
     }
 }
 
+
+static void doKeyword(vector<FWObject *> objs, bool doAdd,
+                      const string &keyword, ProjectPanel *project)
+{
+    vector<FWObject *>::const_iterator iter;
+    for (iter = objs.begin(); iter != objs.end(); ++iter) {
+        FWObject *obj = *iter;
+
+        FWCmdChange *cmd = new FWCmdChange(project, obj);
+        FWObject *newObj = cmd->getNewState();
+        if (doAdd) {
+            newObj->addKeyword(keyword);
+        } else {
+            newObj->removeKeyword(keyword);
+        }
+
+        if (!obj->cmp(newObj)) {
+            project->undoStack->push(cmd);
+        } else {
+            delete cmd;
+        }
+    }
+}
+
+
+void ObjectManipulator::addNewKeywordSlot()
+{
+    QString keyword = QInputDialog::getText(0, tr("Add New Keyword"),
+                          tr("Enter new keyword to add to selected objects"));
+    keyword = keyword.simplified();
+    if (fwbdebug) {
+        qDebug() << "ObjectManipulator::addNewKeyword: " << keyword;
+    }
+
+    if (!KeywordsDialog::validateKeyword(0, keyword)) return;
+
+    doKeyword(getCurrentObjectTree()->getSelectedObjects(), true, 
+              keyword.toUtf8().constData(), m_project);
+}
+
+
+void ObjectManipulator::processKeywordSlot()
+{
+    const QObject *qObj = sender();
+    if (qObj == 0) return;
+    const QAction *qAct = dynamic_cast<const QAction *>(qObj);
+    QStringList list = qAct->data().toStringList();
+    if (list.size() != 2) return;
+
+    if (fwbdebug) {
+        qDebug() << "ObjectManipulator::processKeyword:" << list;
+    }
+
+    doKeyword(getCurrentObjectTree()->getSelectedObjects(), (list[0] == "add"),
+              list[1].toUtf8().constData(), m_project);
+}
+
+
+void ObjectManipulator::addSubfolderSlot()
+{
+    const QAction *qAct = dynamic_cast<const QAction *>(sender());
+    if (qAct == 0) return;
+
+    FWObject *obj = getCurrentObjectTree()->getCurrentObject();
+    assert(obj->getId() == qAct->data().toInt());
+
+    QString folder = QInputDialog::getText(0, tr("Add Subfolder"),
+                                           tr("Enter new subfolder name"));
+    folder = folder.simplified();
+    if (folder.isEmpty()) return;
+    if (folder.contains(',')) {
+        QMessageBox::warning(this, "Firewall Builder",
+                             tr("Subfolder cannot contain a comma"), "&OK",
+                             QString::null, QString::null, 0, 1);
+        return;
+    }
+
+    /* See if the subfolder already exists */
+    string folderStr = folder.toUtf8().constData();
+    set<string> folders = stringToSet(obj->getStr("subfolders"));
+    if (folders.find(folderStr) != folders.end()) return;
+    folders.insert(folderStr);
+
+    if (fwbdebug) {
+        qDebug() << "ObjectManipulator::addSubfolder: " << folder;
+    }
+
+    FWCmdAddUserFolder *cmd = new FWCmdAddUserFolder(m_project, obj, folder,
+                                                     tr("Add subfolder"));
+    FWObject *newObj = cmd->getNewState();
+    newObj->setStr("subfolders", setToString(folders));
+
+    m_project->undoStack->push(cmd);
+}
+
+
+void ObjectManipulator::removeUserFolder()
+{
+    ObjectTreeViewItem *item = dynamic_cast<ObjectTreeViewItem *>
+        (getCurrentObjectTree()->currentItem());
+    if (item == 0 || item->getUserFolderParent() == 0) return;
+    ObjectTreeViewItem *parent = dynamic_cast<ObjectTreeViewItem *>
+        (item->parent());
+    assert(parent != 0);
+
+    vector<FWObject *> objs;
+    for (int ii = 0; ii < item->childCount(); ii++) {
+        ObjectTreeViewItem *child = dynamic_cast<ObjectTreeViewItem *>
+            (item->child(ii));
+
+        FWObject *obj = child->getFWObject();
+        if (obj->getRO()) {
+            QMessageBox::critical(this, "Firewall Builder",
+                                  tr("Folder with locked object "
+                                     "cannot be deleted"));
+            return;
+        }
+
+        objs.push_back(obj);
+    }
+
+    if (objs.size() > 0) {
+        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+        ConfirmDeleteObjectDialog confirm(this);
+        confirm.load(objs);
+        QApplication::restoreOverrideCursor();
+        if (confirm.exec() == QDialog::Rejected) return;
+    }
+
+    FWCmdMacro *macro = new FWCmdMacro(tr("Delete user folder"));
+
+    QList<QTreeWidgetItem *> children = item->takeChildren();
+    while (!children.isEmpty()) {
+        ObjectTreeViewItem *child = dynamic_cast<ObjectTreeViewItem *>
+            (children.takeFirst());
+        assert(child != 0);
+
+        FWObject *obj = child->getFWObject();
+        if (mw->isEditorVisible() && mw->getOpenedEditor() == obj) {
+            mw->hideEditor();
+        }
+
+        deleteObject(obj, macro);
+    }
+
+    FWCmdRemoveUserFolder *cmd =
+        new FWCmdRemoveUserFolder(m_project, parent->getFWObject(),
+                                  item->getUserFolderName(), "", macro);
+    FWObject *newObj = cmd->getNewState();
+    set<string> folders = stringToSet(newObj->getStr("subfolders"));
+    folders.erase(item->getUserFolderName().toUtf8().constData());
+    newObj->setStr("subfolders", setToString(folders));
+
+    m_project->undoStack->push(macro);
+}
