@@ -412,17 +412,6 @@ void ObjectManipulator::inspect()
     m_project->inspect(fwset);
 }
 
-void ObjectManipulator::transferfw()
-{
-    if (getCurrentObjectTree()->getNumSelected()==0) return;
-
-    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
-    set<Firewall*> fo;
-    filterFirewallsFromSelection(so, fo);
-
-    m_project->transferfw(fo);
-}
-
 void ObjectManipulator::find()
 {
     if (getCurrentObjectTree()->getNumSelected()==0) return;
@@ -441,42 +430,6 @@ void ObjectManipulator::findObject()
     mw->findObject( obj );
 }
 
-void ObjectManipulator::back()
-{
-    if (!history.empty())
-    {
-        history.pop_back();
-
-/* skip objects that have been deleted.
- *
- * But see removeObjectFromHistory() which is now called by
- * removeObjectFromTreeView() it may not be necessary to do this
- * additional check here, especially since according to #1661 it
- * probably does not work anyway.
- *
- */
-        while ( ! history.empty())
-        {
-            if (m_project->db()->findInIndex( history.back().id() )!=NULL) break;
-            history.pop_back();
-        }
-
-        if (history.empty())
-        {
-            mw->enableBackAction();
-            return;
-        }
-
-        openObjectInTree( history.back().item(), false );
-
-        if (mw->isEditorVisible())
-        {
-            ObjectTreeViewItem *otvi=history.back().item();
-            switchObjectInEditor(otvi->getFWObject());
-        }
-    }
-}
-
 void ObjectManipulator::findWhereUsedSlot()
 {
     if (getCurrentObjectTree()->getNumSelected()==0) return;
@@ -487,3 +440,45 @@ void ObjectManipulator::findWhereUsedSlot()
 
 }
 
+void ObjectManipulator::makeSubinterface(QAction *act)
+{
+    int intf_id = act->data().toInt();
+    FWObject *new_parent_interface = m_project->db()->findInIndex(intf_id);
+    assert(new_parent_interface!=NULL);
+
+    if (getCurrentObjectTree()->getNumSelected()==0) return;
+
+    ObjectTreeView* ot = getCurrentObjectTree();
+    ot->freezeSelection(true);
+    FWObject *obj;
+
+    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
+    for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
+    {
+        obj = *i;
+
+        if (obj->getParent() == new_parent_interface) continue;
+
+        if (fwbdebug)
+            qDebug() << "ObjectManipulator::makeSubinterface"
+                     << "obj=" << obj
+                     << obj->getName().c_str()
+                     << "new parent:" << new_parent_interface->getName().c_str();
+
+        // new_parent_interface->reparent(obj);
+
+        map<int, set<FWObject*> > reference_holders;
+        FWCmdMoveObject *cmd = new FWCmdMoveObject(
+            m_project,
+            obj->getParent(),
+            new_parent_interface,
+            obj,
+            reference_holders,
+            QString("Make an interface a subinterface"),
+            0);
+        m_project->undoStack->push(cmd);
+    }
+
+    ot->freezeSelection(false);
+}
+
