@@ -52,6 +52,7 @@
 #include "events.h"
 #include "FWCmdChange.h"
 #include "FWCmdAddObject.h"
+#include "FWCmdMoveObject.h"
 #include "IconSetter.h"
 
 #include <QTextEdit>
@@ -99,6 +100,7 @@
 #include "fwbuilder/StateSyncClusterGroup.h"
 #include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/Host.h"
+#include "fwbuilder/AttachedNetworks.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/NetworkIPv6.h"
 #include "fwbuilder/IPv4.h"
@@ -218,19 +220,48 @@ void ObjectManipulator::collapseCurrentTreeNode()
 }
 
 
+static ObjectTreeViewItem *findUserFolder(ObjectTreeViewItem *parent,
+                                          const QString &folder)
+{
+    if (folder.isEmpty()) return parent;
+
+    for (int ii = 0; ii < parent->childCount(); ii++) {
+        ObjectTreeViewItem *sub =
+            dynamic_cast<ObjectTreeViewItem *>(parent->child(ii));
+        if (sub != 0 &&
+            sub->getUserFolderParent() != 0 &&
+            sub->getUserFolderName() == folder) {
+            return sub;
+        }
+    }
+
+    return 0;
+}
+
+
 ObjectTreeViewItem* ObjectManipulator::insertObject(ObjectTreeViewItem *itm,
                                                     FWObject *obj)
 {
     if (FWReference::cast(obj)!=NULL) return NULL;
     if (Resources::global_res->getObjResourceBool(obj,"hidden") ) return NULL;
 
-    ObjectTreeViewItem *nitm = NULL;
-
     if (Resources::global_res->getResourceBool(
             string("/FWBuilderResources/Type/") +
             obj->getTypeName() + "/hidden")) return NULL;
 
-    nitm = new ObjectTreeViewItem( itm );
+    ObjectTreeViewItem *item = itm;
+    if (!obj->getStr("folder").empty()) {
+        item = findUserFolder(itm, obj->getStr("folder").c_str());
+
+        /* If we can't find the user folder, put it under the system
+           folder and get rid of the folder attribute */
+        if (item == 0) {
+            item = itm;
+            obj->setStr("folder", "");
+        }
+    }
+
+    ObjectTreeViewItem *nitm = new ObjectTreeViewItem(item);
 
     nitm->setLib("");
     nitm->setText( 0, getTreeLabel(obj, 0) );
@@ -241,7 +272,11 @@ ObjectTreeViewItem* ObjectManipulator::insertObject(ObjectTreeViewItem *itm,
 
     nitm->setIcon( 0, QIcon(pm) );
 //    nitm->setIcon( 1, QIcon(pm) );
-    nitm->setFlags(nitm->flags() | Qt::ItemIsDragEnabled);
+    if (FWBTree().isSystem(obj)) {
+        nitm->setFlags(nitm->flags() & ~Qt::ItemIsDragEnabled);
+    } else {
+        nitm->setFlags(nitm->flags() | Qt::ItemIsDragEnabled);
+    }
 
     nitm->setProperty("type", obj->getTypeName().c_str() );
     nitm->setFWObject( obj );
@@ -264,12 +299,23 @@ void ObjectManipulator::insertSubtree(FWObject *parent, FWObject *obj)
 
 void ObjectManipulator::insertSubtree(ObjectTreeViewItem *itm, FWObject *obj)
 {
-    this->m_objectManipulator->filter->clearEditText();
+    this->m_objectManipulator->filter->clear();
     ObjectTreeViewItem *nitm = insertObject(itm, obj);
     if (nitm==NULL) return;
 
     if (FWBTree().isStandardFolder(obj)) nitm->setExpanded( st->getExpandTree());
 
+    set<string> subfolders = stringToSet(obj->getStr("subfolders"));
+    set<string>::const_iterator iter;
+    for (iter = subfolders.begin(); iter != subfolders.end(); ++iter) {
+        ObjectTreeViewItem *sub = new ObjectTreeViewItem(nitm);
+        sub->setUserFolderParent(obj);
+        QString name = QString::fromUtf8((*iter).c_str());
+        sub->setUserFolderName(name);
+        sub->setText(0, name);
+        sub->setIcon(0, QIcon(LoadPixmap(":/Icons/SystemGroup/icon-tree")));
+    }
+
     if (Cluster::isA(obj))
     {
          for (FWObjectTypedChildIterator it = obj->findByType(StateSyncClusterGroup::TYPENAME);
@@ -278,9 +324,6 @@ void ObjectManipulator::insertSubtree(ObjectTreeViewItem *itm, FWObject *obj)
 
     if (Cluster::isA(obj) || Firewall::isA(obj))
     {
-         for (FWObjectTypedChildIterator it = obj->findByType(Interface::TYPENAME);
-              it != it.end(); ++it) insertSubtree( nitm, *it );
-
          for (FWObjectTypedChildIterator it = obj->findByType(Policy::TYPENAME);
               it != it.end(); ++it) insertSubtree( nitm, *it );
 
@@ -290,6 +333,9 @@ void ObjectManipulator::insertSubtree(ObjectTreeViewItem *itm, FWObject *obj)
          for (FWObjectTypedChildIterator it = obj->findByType(Routing::TYPENAME);
               it != it.end(); ++it) insertSubtree( nitm, *it );
 
+         for (FWObjectTypedChildIterator it = obj->findByType(Interface::TYPENAME);
+              it != it.end(); ++it) insertSubtree( nitm, *it );
+
          return ;
     }
 
@@ -306,15 +352,18 @@ void ObjectManipulator::insertSubtree(ObjectTreeViewItem *itm, FWObject *obj)
          for (FWObjectTypedChildIterator it = obj->findByType(FailoverClusterGroup::TYPENAME);
               it != it.end(); ++it) insertSubtree( nitm, *it );
 
+         for (FWObjectTypedChildIterator it = obj->findByType(AttachedNetworks::TYPENAME);
+              it != it.end(); ++it) insertSubtree( nitm, *it );
+
          return;
     }
 
-
     for (list<FWObject*>::iterator m=obj->begin(); m!=obj->end(); m++)
     {
         FWObject *o1=*m;
         if (FWReference::cast(o1)!=NULL) continue;
-        insertSubtree( nitm, o1 );
+
+        insertSubtree(nitm, o1);
     }
 }
 
@@ -347,6 +396,22 @@ bool FindHistoryItemByObjectId::operator()(const HistoryItem &itm)
     return (itm.id() == id);
 }
 
+FWObject* ObjectManipulator::findRuleSetInHistoryByParentFw(FWObject* parent)
+{
+    list<HistoryItem>::reverse_iterator it = history.rbegin();
+    for (; it!=history.rend(); ++it)
+    {
+        FWObject *obj = mw->activeProject()->db()->findInIndex(it->id());
+        if (RuleSet::cast(obj))
+        {
+            FWObject *parent_fw = Host::getParentHost(obj);
+            if (parent_fw != NULL && parent_fw == parent) return obj;
+        }
+    }
+
+    return NULL;
+}
+
 void ObjectManipulator::removeObjectFromHistory(FWObject *obj)
 {
     if (fwbdebug)
@@ -364,6 +429,19 @@ void ObjectManipulator::removeObjectFromHistory(FWObject *obj)
     if (history.empty()) mw->enableBackAction();
 }
 
+void ObjectManipulator::addObjectToHistory(ObjectTreeViewItem* otvi,
+                                           FWObject *obj)
+{
+    history.push_back( HistoryItem(otvi, obj->getId()) );
+    current_history_item = history.end();
+    current_history_item--;
+}
+
+ObjectTreeViewItem* ObjectManipulator::getCurrentHistoryItem()
+{
+    return current_history_item->item();
+}
+
 void ObjectManipulator::updateLibColor(FWObject *lib)
 {
     QString clr = lib->getStr("color").c_str();
@@ -499,13 +577,17 @@ void ObjectManipulator::loadObjects()
 
         addLib( lib );
 
-        if (fwbdebug) qDebug("ObjectManipulator::loadObjects %p added lib %s",
-                             this, lib->getName().c_str());
+        if (fwbdebug)
+            qDebug() << "ObjectManipulator::loadObjects"
+                     << this << "added lib" << lib->getName().c_str();
+
     }
 
-    if (firstUserLib==NULL) firstUserLib=ll.front();
+    if (firstUserLib==NULL) firstUserLib = ll.front();
     openLib( firstUserLib );
-    if (fwbdebug) qDebug("ObjectManipulator::loadObjects %p done", this);
+
+    if (fwbdebug)
+        qDebug() << "ObjectManipulator::loadObjects done" << this;
 }
 
 void ObjectManipulator::addLib(FWObject *lib)
@@ -571,16 +653,15 @@ void ObjectManipulator::addLib(FWObject *lib)
             SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*) ),
             this, SLOT(selectionChanged(QTreeWidgetItem*)));
 
+    connect(objTreeView, SIGNAL(moveItems_sign(ObjectTreeViewItem *, const std::list<libfwbuilder::FWObject *> &)),
+            this, SLOT(moveItems(ObjectTreeViewItem *, const std::list<libfwbuilder::FWObject *> &)));
 
     ObjectTreeViewItem *itm1=new ObjectTreeViewItem( objTreeView );
 
     itm1->setLib("");
     itm1->setExpanded(TRUE);
 
-/* need to enable dragging in order to avoid object highlighting in
- * the tree when user drags mouse cursor */
-
-    itm1->setFlags(itm1->flags() | Qt::ItemIsDragEnabled);
+    itm1->setFlags(itm1->flags() & ~Qt::ItemIsDragEnabled);
 
     itm1->setText( 0, getTreeLabel(lib, 0) );
     itm1->setText( 1, getTreeLabel(lib, 1) );
@@ -599,7 +680,7 @@ void ObjectManipulator::addLib(FWObject *lib)
     objTreeView->header()->resizeSections(QHeaderView::ResizeToContents);
 
     m_objectManipulator->filter->connect(m_objectManipulator->filter,
-                                         SIGNAL(editTextChanged(QString)),
+                                         SIGNAL(textChanged(QString)),
                                          objTreeView, SLOT(setFilter(QString)));
 }
 
@@ -662,3 +743,95 @@ void ObjectManipulator::refreshSubtree(QTreeWidgetItem *parent, QTreeWidgetItem
     getCurrentObjectTree()->update();
 }
 
+void ObjectManipulator::moveItems(ObjectTreeViewItem *dest,
+                                  const list<FWObject *> &items)
+{
+    string folder;
+    if (dest->getUserFolderParent() != 0) {
+        folder = dest->getUserFolderName().toUtf8().constData();
+    } else {
+        folder = dest->getFWObject()->getStr("folder");
+    }
+
+    FWCmdMacro *macro = new FWCmdMacro(tr("Move objects"));
+
+    list<FWObject *>::const_iterator iter;
+    for (iter = items.begin(); iter != items.end(); ++iter) {
+        FWObject *obj = *iter;
+
+        FWCmdMoveToFromUserFolder *cmd = new FWCmdMoveToFromUserFolder
+            (m_project, obj->getParent(), obj,
+             obj->getStr("folder").c_str(), folder.c_str(), "", macro);
+
+        FWObject *newObj = cmd->getNewState();
+        newObj->setStr("folder", folder);
+    }
+
+    m_project->undoStack->push(macro);
+}
+
+
+void ObjectManipulator::addUserFolderToTree(FWObject *obj,
+                                            const QString &folder)
+{
+    ObjectTreeViewItem *item = allItems[obj];
+    if (item == 0) return;
+
+    ObjectTreeViewItem *sub = new ObjectTreeViewItem(item);
+    sub->setUserFolderParent(obj);
+    sub->setUserFolderName(folder);
+    sub->setText(0, folder);
+    sub->setIcon(0, QIcon(LoadPixmap(":/Icons/SystemGroup/icon-tree")));
+    refreshSubtree(item, sub);
+}
+
+
+void ObjectManipulator::removeUserFolderFromTree(FWObject *obj,
+                                                 const QString &folder)
+{
+    ObjectTreeViewItem *item = allItems[obj];
+    if (item == 0) return;
+
+    ObjectTreeViewItem *sub = findUserFolder(item, folder);
+    if (sub == 0) return;
+
+    QList<QTreeWidgetItem *> children = sub->takeChildren();
+    while (!children.isEmpty()) {
+        ObjectTreeViewItem *child = dynamic_cast<ObjectTreeViewItem *>
+            (children.takeFirst());
+        assert(child != 0);
+
+        FWObject *obj = child->getFWObject();
+        if (mw->isEditorVisible() && mw->getOpenedEditor() == obj) {
+            mw->hideEditor();
+        }
+
+        delete child;
+    }
+
+    item->removeChild(sub);
+    delete sub;
+
+    refreshSubtree(item, 0);
+}
+
+
+void ObjectManipulator::moveToFromUserFolderInTree(FWObject *obj,
+                                                   FWObject *objToMove,
+                                                   const QString &oldFolder,
+                                                   const QString &newFolder)
+{
+    ObjectTreeViewItem *parent = allItems[obj];
+    ObjectTreeViewItem *toMove = allItems[objToMove];
+    if (parent == 0 || toMove == 0) return;
+
+    ObjectTreeViewItem *oldItem = findUserFolder(parent, oldFolder);
+    ObjectTreeViewItem *newItem = findUserFolder(parent, newFolder);
+
+    oldItem->removeChild(toMove);
+    newItem->addChild(toMove);
+
+    refreshSubtree(newItem, 0);
+}
+
+                                                   
