@@ -28,41 +28,46 @@
 #include "global.h"
 #include "utils.h"
 
-#include "FWWindow.h"
+#include "FWBSettings.h"
 #include "FWBTree.h"
+#include "FWObjectDrag.h"
+#include "FWObjectPropertiesFactory.h"
+#include "FWWindow.h"
+#include "IconSetter.h"
+#include "ObjectManipulator.h"
 #include "ObjectTreeView.h"
 #include "ObjectTreeViewItem.h"
-#include "ObjectManipulator.h"
-#include "FWObjectDrag.h"
-#include "FWBSettings.h"
-#include "IconSetter.h"
-#include "FWObjectPropertiesFactory.h"
+#include "ProjectPanel.h"
 
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/Firewall.h"
-#include "fwbuilder/Resources.h"
 #include "fwbuilder/Group.h"
 #include "fwbuilder/Interface.h"
+#include "fwbuilder/NAT.h"
+#include "fwbuilder/Routing.h"
+#include "fwbuilder/Policy.h"
+#include "fwbuilder/Resources.h"
 
 #include <QAbstractItemView>
-#include <QHeaderView>
 #include <QBitmap>
-#include <qpainter.h>
-#include <qpixmapcache.h>
-#include <QMimeData>
+#include <QDragEnterEvent>
+#include <QDragLeaveEvent>
 #include <QDragMoveEvent>
 #include <QDropEvent>
-#include <QDragLeaveEvent>
 #include <QFocusEvent>
-#include <QPixmap>
-#include <QMouseEvent>
+#include <QHeaderView>
 #include <QKeyEvent>
-#include <QDragEnterEvent>
+#include <QMimeData>
+#include <QMouseEvent>
+#include <QPixmap>
+#include <QStyle>
+#include <QStyleOption>
 #include <QtDebug>
+#include <qpainter.h>
+#include <qpixmapcache.h>
 
 #include <iostream>
 #include <algorithm>
-#include "ProjectPanel.h"
 
 using namespace std;
 using namespace libfwbuilder;
@@ -84,9 +89,19 @@ ObjectTreeView::ObjectTreeView(ProjectPanel* project,
     this->setParent(parent, f);
     setFont(st->getTreeFont());
 
+    QPalette updated_palette = palette();
+    updated_palette.setColor(
+        QPalette::Inactive,
+        QPalette::Highlight,
+        QColor("silver"));
+
+//        palette().color(QPalette::Highlight).lighter(300));
+
+    setPalette(updated_palette);
+
     setExpandsOnDoubleClick(false);
 
-//    setAcceptDrops( TRUE );
+    setDragEnabled(true);
     item_before_drag_started=NULL;
     lastSelected = NULL;
     second_click = false;
@@ -139,13 +154,16 @@ ObjectTreeView::ObjectTreeView(ProjectPanel* project,
 
     setMinimumSize( QSize( 100, 0 ) );
 
-    setAutoScroll( TRUE );
+    setAutoScroll(true);
+    setAutoScrollMargin(50);
     setAllColumnsShowFocus( TRUE );
     setSelectionMode( ExtendedSelection );
     setAcceptDrops( true );
     setDragDropMode( QAbstractItemView::DragDrop );
     setRootIsDecorated( TRUE );
 
+    setFocusPolicy(Qt::StrongFocus);
+
     connect(this, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this,
             SLOT(updateFilter()));
 
@@ -153,7 +171,34 @@ ObjectTreeView::ObjectTreeView(ProjectPanel* project,
 // QT 4.3.4 (discovered on Ubuntu Hardy). Crash happened when
 // second object was added to any branch of the tree.
 //
+// This causes crash with Qt 4.6 as well
+//
 //    setSortingEnabled(true);
+
+}
+
+void ObjectTreeView::paintEvent(QPaintEvent *ev)
+{
+    QTreeWidget::paintEvent(ev);
+}
+
+void ObjectTreeView::drawRow(QPainter *painter, const QStyleOptionViewItem &option,
+                             const QModelIndex &index ) const
+{
+    // qDebug() << "ObjectTreeView::drawRow"
+    //          << "QStyleOptionViewItem.state=" << int(option.state)
+    //          << "hasFocus()=" << hasFocus()
+    //          << "isActiveWindow()=" << isActiveWindow();
+
+    // QWidget *fw = QApplication::focusWidget();
+    // qDebug() << "Currently has focus:" << fw;
+
+    // this is a patch for #2475
+    // Looks like the state remains State_Active even when the tree view widget
+    // loses focus (as long as parent window is active).
+    QStyleOptionViewItem new_opt = option;
+    if ( ! hasFocus()) new_opt.state &= ~QStyle::State_Active;
+    QTreeWidget::drawRow(painter, new_opt, index);
 }
 
 bool ObjectTreeView::event( QEvent *event )
@@ -287,20 +332,19 @@ void ObjectTreeView::focusOutEvent(QFocusEvent* ev)
 
 void ObjectTreeView::updateTreeIcons()
 {
-    if (fwbdebug) qDebug("ObjectTreeView::updateTreeIcons ");
-
     QTreeWidgetItemIterator it(this);
-    while ( *it )
+    for ( ; *it; ++it)
     {
         QTreeWidgetItem *itm = *it;
         ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem*>(itm);
         FWObject *obj = otvi->getFWObject();
 
+        /* We can have obj==0 if it's a user-create subfolder */
+        if (obj == 0) continue;
+
         QPixmap pm_obj;
         IconSetter::setObjectIcon(obj, &pm_obj, 0);
         itm->setIcon(0, pm_obj );
-
-        ++it;
     }
     update();
 }
@@ -310,10 +354,11 @@ void ObjectTreeView::startDrag(Qt::DropActions supportedActions)
     QTreeWidgetItem *ovi = currentItem();
     if (ovi==NULL) return;
 
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-
     FWObject *current_obj = getCurrentObject();
 
+    /* User-defined folders can't be dragged */
+    if (current_obj == 0) return;
+
     if (fwbdebug) qDebug("ObjectTreeView::startDrag: this: %p current_obj: %s",
                          this, current_obj->getName().c_str());
 
@@ -411,10 +456,6 @@ void ObjectTreeView::startDrag(Qt::DropActions supportedActions)
     if (fwbdebug) qDebug("ObjectTreeView::dragObject()  this=%p visible=%d",
                          this,visible);
 
-    FWObject *edit_obj = mw->getOpenedEditor();
-
-    if (fwbdebug) qDebug("ObjectTreeView::dragObject()  returns !NULL");
-
     drag->start(supportedActions);
 }
 
@@ -424,79 +465,95 @@ void ObjectTreeView::dragEnterEvent( QDragEnterEvent *ev)
     ev->setDropAction(Qt::MoveAction);
 }
 
-bool ObjectTreeView::isCurrReadOnly(QDragMoveEvent *ev)
-{
-// the tree can accept drop only if it goes into a group and if that group
-//  validates the object and tree is not read-only
-    QTreeWidgetItem *ovi = itemAt(ev->pos());
 
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-    FWObject *trobj;
-    if (otvi && (trobj = otvi->getFWObject()))
-        return trobj->isReadOnly();
-    return false;
+static bool isValidDropTarget(QTreeWidgetItem *item, list<FWObject *> &objs)
+{
+    ObjectTreeViewItem *dest = dynamic_cast<ObjectTreeViewItem *>(item);
+    if (dest == 0) return false;
+
+    bool dragIsNoop = true;
+    list<FWObject *>::const_iterator iter;
+    for (iter = objs.begin(); iter != objs.end(); ++iter) {
+        FWObject *dragobj = *iter;
+        assert(dragobj != 0);
+
+        if (Interface::cast(dragobj) != 0 ||
+            Interface::cast(dragobj->getParent()) != 0 ||
+            Policy::cast(dragobj) != 0 ||
+            NAT::cast(dragobj) != 0 ||
+            Routing::cast(dragobj) != 0) return false;
+
+        /* See if destination is a user folder */
+        if (dest->getUserFolderParent() != 0) {
+            /* Dragged object has to match parent of user folder */
+            if (dest->getUserFolderParent() != dragobj->getParent()) {
+                return false;
+            }
+
+            /* Are we dragging within the same user folder? */
+            if (dest->getUserFolderName() !=
+                QString::fromUtf8(dragobj->getStr("folder").c_str())) {
+                dragIsNoop = false;
+            }
+        } else {
+            /* OK to drag onto parent itself, or object that shares parent */
+            if (dragobj->getParent() != dest->getFWObject() &&
+                dragobj->getParent() != dest->getFWObject()->getParent()) {
+                return false;
+            }
+
+            /* Are we dragging to a new place? */
+            if ((FWBTree().isSystem(dest->getFWObject()) &&
+                 dragobj->getStr("folder") != "") ||
+                (dest->getFWObject()->getStr("folder") !=
+                 dragobj->getStr("folder"))) {
+                dragIsNoop = false;
+            }
+        }
+    }
+
+    return !dragIsNoop;
 }
 
-void ObjectTreeView::dragMoveEvent( QDragMoveEvent *ev)
+
+void ObjectTreeView::dragMoveEvent(QDragMoveEvent *ev)
 {
-    QWidget *fromWidget = ev->source();
+    /* Call the parent so that auto-scrolling works properly */
+    QTreeWidget::dragMoveEvent(ev);
 
-    // The source of DnD object must be the same instance of fwbuilder
-    if (!fromWidget)
-    {
-        ev->setAccepted(false);
-        return;
-    }
-   
-    if (isCurrReadOnly(ev) ||
-        !ev->mimeData()->hasFormat(FWObjectDrag::FWB_MIME_TYPE))
-    {
+    list<FWObject*> objs;
+    if (ev->source() != this || !FWObjectDrag::decode(ev, objs) ||
+        !isValidDropTarget(itemAt(ev->pos()), objs)) {
         ev->setAccepted(false);
         return;
     }
 
-    list<FWObject*> dragol;
-    if (!FWObjectDrag::decode(ev, dragol))
-        ev->setAccepted(false);
-    for (list<FWObject*>::iterator i=dragol.begin();i!=dragol.end(); ++i)
-    {
-        FWObject *dragobj = *i;
-        assert(dragobj!=NULL);
-
-        if (FWBTree().isSystem(dragobj))
-        {
-// can not drop system folder anywhere 
-            ev->setAccepted(false);
-            return;
-        }
-
-        // see #1976 do not allow pasting object that has been deleted
-        if (dragobj->getLibrary()->getId() == FWObjectDatabase::DELETED_OBJECTS_ID)
-        {
-            ev->setAccepted(false);
-            return;
-        }
-    }
-
+    ev->setDropAction(Qt::MoveAction);
     ev->setAccepted(true);
 }
 
-/*
- * See ticket #483: d&d of objects inside the tree should not be allowed
- */
+
 void ObjectTreeView::dropEvent(QDropEvent *ev)
 {
-    ev->setAccepted(false);
-    return;
-}
+    // only accept drops from the same instance of fwbuilder
+    if (ev->source() == NULL) return;
 
-FWObject *ObjectTreeView::getDropTarget(QDropEvent *ev, FWObject*)
-{
-    QTreeWidgetItem *ovi = itemAt(ev->pos());
+    ObjectTreeViewItem *dest =
+        dynamic_cast<ObjectTreeViewItem *>(itemAt(ev->pos()));
+    if (dest == 0) {
+    notWanted:
+        ev->setAccepted(false);
+        return;
+    }
 
-    ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem*>(ovi);
-    if (otvi) return otvi->getFWObject();
-    else return NULL;
+    list<FWObject*> objs;
+    if (!FWObjectDrag::decode(ev, objs)) goto notWanted;
+
+    /* Make sure the drop event is on an object that can handle it */
+    if (ev->source() != this || !isValidDropTarget(dest, objs)) goto notWanted;
+
+    emit moveItems_sign(dest, objs);
+    ev->setAccepted(true);
 }
 
 void ObjectTreeView::dragLeaveEvent( QDragLeaveEvent *ev)
@@ -508,6 +565,9 @@ void ObjectTreeView::dragLeaveEvent( QDragLeaveEvent *ev)
 
 void ObjectTreeView::mouseMoveEvent( QMouseEvent * e )
 {
+    /* This stops highlighting of stuff in the tree when the user
+       clicks and tries to drag something non-draggable. */
+    if (state() == DragSelectingState) return;
     QTreeWidget::mouseMoveEvent(e);
     if (e==NULL)  return;
 }
@@ -706,6 +766,9 @@ void ObjectTreeView::itemSelectionChanged()
         QTreeWidgetItem *itm = (*it);
         ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem*>(itm);
 
+        FWObject *obj = otvi->getFWObject();
+        if (obj == 0) continue;
+
         selectedObjects.push_back(otvi->getFWObject());
 
         if (fwbdebug) qDebug(
@@ -744,14 +807,14 @@ void ObjectTreeView::ExpandTreeItems(const set<int> &ids)
         qDebug() << "ObjectTreeView::ExpandTreeItems()";
 
     QTreeWidgetItemIterator it(this);
-    while ( *it )
+    for ( ; *it; ++it)
     {
         QTreeWidgetItem *itm = *it;
         ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
         FWObject *obj = otvi->getFWObject();
+        if (obj == 0) continue;
         if (ids.count(obj->getId()))
             itm->setExpanded(true);
-        ++it;
     }
 }
 
@@ -788,30 +851,100 @@ void ObjectTreeView::updateFilter()
     setFilter(filter);
 }
 
+static bool filterMatches(const QString &text,
+                          ObjectTreeViewItem *item)
+{
+    if (text.isEmpty()) return true;
+    if (item->text(0).contains(text, Qt::CaseInsensitive)) return true;
+
+    if (item->getUserFolderParent() != 0) return false;
+    FWObject *obj = item->getFWObject();
+
+    QByteArray utf8 = text.toUtf8();
+    set<string> keys = obj->getKeywords();
+    set<string>::const_iterator iter;
+    for (iter = keys.begin(); iter != keys.end(); ++iter) {
+        QString keyword = QString::fromUtf8((*iter).c_str());
+        if (keyword.contains(text, Qt::CaseInsensitive)) return true;
+    }
+
+    return false;
+}
+
+static uint qHash(const QStringList &list)
+{
+    uint ret = 0;
+    for (int ii = 0; ii < list.size(); ii++) {
+        ret += qHash(list.at(ii));
+    }
+    return ret;
+}
+
+void ObjectTreeView::doExpandedState(bool save, QStringList &list,
+                                     QTreeWidgetItem *item)
+{
+    list.append(item->text(0));
+    if (save) {
+        if (item->isExpanded()) expandedState.insert(list);
+    } else {
+        if (expandedState.contains(list)) item->setExpanded(true);
+    }
+
+    for (int ii = 0; ii < item->childCount(); ii++) {
+        doExpandedState(save, list, item->child(ii));
+    }
+    list.removeLast();
+}
+
+
 void ObjectTreeView::setFilter(QString text)
 {
+    if (filter.isEmpty() && !text.isEmpty()) {
+        QStringList list;
+        for (int ii = 0; ii < topLevelItemCount(); ii++) {
+            doExpandedState(true, list, topLevelItem(ii));
+        }
+    } else if (text.isEmpty() && !filter.isEmpty()) {
+        QStringList list;
+        for (int ii = 0; ii < topLevelItemCount(); ii++) {
+            doExpandedState(false, list, topLevelItem(ii));
+        }
+        expandedState.clear();
+    }
+    filter = text;
+
     if (fwbdebug)
         qDebug() << "ObjectTreeView::setFilter " << text;
 
-    if (filter.isEmpty() && text.isEmpty()) return;
-    filter = text;
-    QSet<QTreeWidgetItem *> items = this->findItems(text, Qt::MatchContains|Qt::MatchRecursive, 0).toSet();
-    QSet<QTreeWidgetItem *> children, parents;
-    QSet<QTreeWidgetItem *>::iterator iter;
-    for(iter=items.begin(); iter!=items.end(); iter++)
-    {
-        children.unite(resolveChildren((*iter)));
-        parents.unite(resolveParents((*iter)));
+    list<QTreeWidgetItem *> expand;
+    for (QTreeWidgetItemIterator wit(this); *wit; ++wit) {
+        ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem *>(*wit);
+
+        if (filterMatches(text, otvi)) {
+            (*wit)->setHidden(false);
+
+            if (Firewall::cast(otvi->getFWObject()) != 0) {
+                expand.push_back(otvi);
+            }
+
+            QTreeWidgetItem *parent = (*wit)->parent();
+            while (parent != 0) {
+                parent->setHidden(false);
+                parent = parent->parent();
+            }
+        } else {
+            (*wit)->setHidden(true);
+        }
     }
-    items.unite(children);
-    items.unite(parents);
-    QTreeWidgetItemIterator witer(this);
-    while (*witer)
-    {
-        if (items.contains((*witer))) (*witer)->setHidden(false);
-        else (*witer)->setHidden(true);
-        witer++;
+
+    list<QTreeWidgetItem *>::const_iterator iter;
+    for (iter = expand.begin(); iter != expand.end(); ++iter) {
+        QTreeWidgetItem *item = *iter;
+        item->setHidden(false);
+        for (int ii = 0; ii < item->childCount(); ii++) {
+            expand.push_back(item->child(ii));
+        }
     }
+
     if (!text.isEmpty()) this->expandAll();
 }
-
