@@ -112,7 +112,8 @@ PrefsDialog::PrefsDialog(QWidget *parent) : QDialog(parent)
 
     m_dialog->tabWidget->setCurrentIndex(0);
 
-    m_dialog->wDir->setText( st->getWDir() );
+    m_dialog->wDir->setText(st->getWDir());
+    m_dialog->dataDir->setText(st->getDataDir());
 
     m_dialog->objTooltips->setChecked( st->getObjTooltips() );
     m_dialog->advTooltipMode->setChecked(st->getBool("UI/AdvancedTooltips"));
@@ -161,6 +162,15 @@ PrefsDialog::PrefsDialog(QWidget *parent) : QDialog(parent)
     m_dialog->rulesLoggingOn->setChecked(
         st->getBool("Objects/PolicyRule/defaultLoggingState") );
 
+    m_dialog->rulesDefaultStateful->setChecked(
+        st->getBool("Objects/PolicyRule/defaultStateful"));
+
+    m_dialog->rulesDefaultAction->setCurrentIndex(
+        st->getInt("Objects/PolicyRule/defaultAction"));
+
+    m_dialog->rulesDefaultDirection->setCurrentIndex(
+        st->getInt("Objects/PolicyRule/defaultDirection"));
+
     m_dialog->autoconfigure_interfaces->setChecked(
         st->getBool("Objects/Interface/autoconfigureInterfaces") );
 
@@ -388,29 +398,60 @@ void PrefsDialog::changeFont(QFont &font)
 
 void PrefsDialog::findWDir()
 {
-    QString wd = st->getWDir();
+    QString wd = m_dialog->wDir->text();
+    if (wd.isEmpty()) wd = st->getWDir();
+    if (wd.isEmpty()) wd = st->getOpenFileDir();
     QString dir = QFileDialog::getExistingDirectory(
-            this, tr("Find working directory"), wd, QFileDialog::ShowDirsOnly );
+            this, tr("Find working directory"), wd, QFileDialog::ShowDirsOnly);
 
-    if (!dir.isEmpty()) m_dialog->wDir->setText(dir);
+    if (dir.isEmpty()) return;
+    st->setOpenFileDir(dir);
+
+    m_dialog->wDir->setText(dir);
+}
+
+void PrefsDialog::findDataDir()
+{
+    QString dataDir = m_dialog->dataDir->text();
+    if (dataDir.isEmpty()) dataDir = st->getDataDir();
+    if (dataDir.isEmpty()) dataDir = st->getOpenFileDir();
+    QString dir = QFileDialog::getExistingDirectory(
+        this, tr("Find data directory"), dataDir, QFileDialog::ShowDirsOnly);
+
+    if (dir.isEmpty()) return;
+    st->setOpenFileDir(dir);
+
+    m_dialog->dataDir->setText(dir);
 }
 
 void PrefsDialog::findSSH()
 {
-    QString fp = QFileDialog::getOpenFileName(
-        this,
-        tr("Find Secure Shell utility") );
+    QString sshPath = m_dialog->sshPath->text();
+    if (!QFileInfo(sshPath).isFile()) sshPath = st->getSSHPath();
+    if (!QFileInfo(sshPath).isFile()) sshPath = st->getOpenFileDir();
 
-    if (!fp.isEmpty()) m_dialog->sshPath->setText(fp);
+    QString fp = QFileDialog::getOpenFileName(
+        this, tr("Find Secure Shell utility"), sshPath);
+
+    if (fp.isEmpty()) return;
+    st->setOpenFileDir(fp);
+
+    m_dialog->sshPath->setText(fp);
 }
 
 void PrefsDialog::findSCP()
 {
-    QString fp = QFileDialog::getOpenFileName(
-        this,
-        tr("Find SCP utility") );
+    QString scpPath = m_dialog->scpPath->text();
+    if (!QFileInfo(scpPath).isFile()) scpPath = st->getSCPPath();
+    if (!QFileInfo(scpPath).isFile()) scpPath = st->getOpenFileDir();
 
-    if (!fp.isEmpty()) m_dialog->scpPath->setText(fp);
+    QString fp = QFileDialog::getOpenFileName(
+        this, tr("Find SCP utility"), scpPath);
+
+    if (fp.isEmpty()) return;
+    st->setOpenFileDir(fp);
+
+    m_dialog->scpPath->setText(fp);
 }
 
 void PrefsDialog::accept()
@@ -420,6 +461,7 @@ void PrefsDialog::accept()
 /* check if the default working directory does not exist yet */
 
     st->setWDir( wd );
+    st->setDataDir(m_dialog->dataDir->text());
 
     st->setObjTooltips( m_dialog->objTooltips->isChecked() );
     st->setBool("UI/AdvancedTooltips", m_dialog->advTooltipMode->isChecked());
@@ -454,6 +496,17 @@ void PrefsDialog::accept()
     st->setBool("Objects/PolicyRule/defaultLoggingState",
                 m_dialog->rulesLoggingOn->isChecked());
 
+    st->setBool("Objects/PolicyRule/defaultStateful",
+                m_dialog->rulesDefaultStateful->isChecked());
+
+    st->setInt("Objects/PolicyRule/defaultAction",
+                m_dialog->rulesDefaultAction->currentIndex());
+
+    st->setInt("Objects/PolicyRule/defaultDirection",
+                m_dialog->rulesDefaultDirection->currentIndex());
+
+
+
     st->setBool("Objects/Interface/autoconfigureInterfaces",
                 m_dialog->autoconfigure_interfaces->isChecked());
 
