@@ -33,6 +33,7 @@
 #include "FWBSettings.h"
 #include "RCS.h"
 #include "RuleSetView.h"
+#include "ObjectTreeView.h"
 
 #include <QtDebug>
 #include <QMdiSubWindow>
@@ -145,6 +146,8 @@ void ProjectPanel::saveMainSplitter()
 {
     QString fileName ;
     if (rcs!=NULL) fileName = rcs->getFileName();
+
+#ifdef TREE_IS_DOCKABLE
     // Save position of splitters regardless of the window state
     // Do not save if one of tree panel is floating
     if (!m_panel->treeDockWidget->isWindow())
@@ -161,6 +164,20 @@ void ProjectPanel::saveMainSplitter()
             qDebug() << out1;
         }
     }
+#else
+    QList<int> sl = m_panel->topSplitter->sizes();
+    QString arg = QString("%1,%2").arg(sl[0]).arg(sl[1]);
+    if (sl[0] || sl[1])
+        st->setStr("Window/" + fileName + "/MainWindowSplitter", arg );
+
+    if (fwbdebug)
+    {
+        QString out1 = " save Window/" + fileName + "/MainWindowSplitter";
+        out1+= " " + arg;
+        qDebug() << out1;
+    }
+
+#endif
 }
 
 void ProjectPanel::loadMainSplitter()
@@ -306,10 +323,13 @@ void ProjectPanel::loadLastOpenedLib()
         if (last_lib_id > 0 && last_lib_id != FWObjectDatabase::DELETED_OBJECTS_ID)
         {
             m_panel->om->libChangedById(last_lib_id);
+            m_panel->om->getCurrentObjectTree()->setFocus(Qt::OtherFocusReason);
             return;
         }
     }
     loadFirstNonStandardLib();
+
+    m_panel->om->getCurrentObjectTree()->setFocus(Qt::OtherFocusReason);
 }
 
 void ProjectPanel::loadFirstNonStandardLib()
