@@ -24,6 +24,7 @@
 */
 
 #include "../../config.h"
+#include "../../definitions.h"
 #include "global.h"
 #include "utils.h"
 
@@ -1202,7 +1203,6 @@ bool RuleSetModel::isGroup(const QModelIndex &index) const
 
 void RuleSetModel::resetAllSizes()
 {
-    qDebug() << "resetAllSizes()";
     emit layoutAboutToBeChanged ();
     root->resetAllSizes();
     emit layoutChanged ();
@@ -1326,7 +1326,7 @@ QModelIndexList RuleSetModel::findObject(FWObject* object)
                         {
                             list.append(this->index(rule, column));
                         }
-                    } else if (pr->getAction() == PolicyRule::Tag)
+                    } else if (pr->getTagging())
                     {
                         if (pr->getTagObject() == object)
                         {
@@ -1443,8 +1443,63 @@ QStringList PolicyModel::getRuleOptions(Rule* r) const
 {
     QStringList res;
     PolicyRule  *policyRule  = PolicyRule::cast( r );
+    FWOptions *ropt = policyRule->getOptionsObject();
+
     if (policyRule->getLogging()) res << "Log";
-    if (!isDefaultPolicyRuleOptions(r->getOptionsObject())) res << "Options";
+
+    if ( ! isDefaultPolicyRuleOptions(r->getOptionsObject())) res << "Options";
+
+    FWObject *firewall = r;
+    // use Firewall::cast to match both Firewall and Cluster
+    while (!Firewall::cast(firewall)) firewall = firewall->getParent();
+    string platform = firewall->getStr("platform");
+
+    if (policyRule->getTagging())
+    {
+        FWObject *tobj = PolicyRule::cast(policyRule)->getTagObject();
+        if (tobj)
+        {
+            if (platform=="iptables")
+            {
+                res << tr("Tag %1%2")
+                    .arg(QString::fromUtf8(tobj->getName().c_str()))
+                    .arg(
+                        QString(
+                            (ropt->getBool("ipt_mark_connections"))?" (conn)":""));
+            } else
+                res << tr("Tag %1").arg(
+                    QString::fromUtf8(tobj->getName().c_str()));
+        }
+    }
+
+    if (policyRule->getClassification())
+    {
+        QString par;
+
+        if (platform=="iptables")
+            par = ropt->getStr("classify_str").c_str();
+
+        if (platform=="pf")
+            par = ropt->getStr("pf_classify_str").c_str();
+
+        if (platform=="ipfw")
+        {
+            if (ropt->getInt("ipfw_classify_method") == DUMMYNETPIPE)
+            {
+                par = "pipe";
+            } else {
+                par = "queue";
+            }
+            par = par + " " + ropt->getStr("ipfw_pipe_queue_num").c_str();
+        }
+
+        if (!par.isEmpty()) res << tr("Classify %1").arg(par);
+    }
+
+    if (policyRule->getRouting())
+    {
+        res << "Route";
+    }
 
     return res;
 }
@@ -1455,12 +1510,30 @@ void PolicyModel::initRule(Rule *new_rule, Rule *old_rule)
     PolicyRule *newrule_as_policy_rule = PolicyRule::cast(new_rule);
     if (newrule_as_policy_rule)
     {
+        FWOptions *ruleopt = newrule_as_policy_rule->getOptionsObject();
         newrule_as_policy_rule->setLogging(supports_logging &&
                      st->getBool("Objects/PolicyRule/defaultLoggingState"));
-        newrule_as_policy_rule->setAction(PolicyRule::Deny);
-        newrule_as_policy_rule->setDirection(PolicyRule::Both);
-        FWOptions *ruleopt = newrule_as_policy_rule->getOptionsObject();
+
+        switch (st->getInt("Objects/PolicyRule/defaultAction"))
+        {
+        case 1:
+            newrule_as_policy_rule->setAction(PolicyRule::Accept); break;
+        default:
+            newrule_as_policy_rule->setAction(PolicyRule::Deny); break;
+        }
+
+        switch (st->getInt("Objects/PolicyRule/defaultDirection"))
+        {
+        case 0:
+            newrule_as_policy_rule->setDirection(PolicyRule::Both); break;
+        case 1:
+            newrule_as_policy_rule->setDirection(PolicyRule::Inbound); break;
+        case 2:
+            newrule_as_policy_rule->setDirection(PolicyRule::Outbound); break;
+        }
+
         ruleopt->setBool("stateless",
+                         ! st->getBool("Objects/PolicyRule/defaultStateful") ||
                          getStatelessFlagForAction(newrule_as_policy_rule));
     }
 
