@@ -6,8 +6,6 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id$
-
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
   of that license as published by the Free Software Foundation; either
@@ -48,19 +46,25 @@
 #include <QtGui>
 #include<QStringList>
 
+
 using namespace libfwbuilder;
 using namespace std;
 
-//////////////////////////////////////////////////////////////////////////////////////////////////////////
-// RuleSetViewDelegate
-//////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-RuleSetViewDelegate::RuleSetViewDelegate(QObject *parent, FWObjectSelectionModel *selectionModel) : QItemDelegate(parent)
+////////////////////////////////////////////////////////////////////////////
+// RuleSetViewDelegate
+////////////////////////////////////////////////////////////////////////////
+
+RuleSetViewDelegate::RuleSetViewDelegate(QObject *parent,
+                                         FWObjectSelectionModel *selectionModel)
+    : QItemDelegate(parent)
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::RuleSetViewDelegate";
     this->sectionModel = selectionModel;
 }
-void RuleSetViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+void RuleSetViewDelegate::paint(QPainter *painter,
+                                const QStyleOptionViewItem &option,
+                                const QModelIndex &index) const
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paint";
 
@@ -96,7 +100,14 @@ void RuleSetViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
     painter->restore();
 }
 
-void RuleSetViewDelegate::paintRowHeader (QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index, RuleNode * node) const
+/*
+ * This paints the leftmost column in the rule set view (where rule
+ * number appears)
+ */
+void RuleSetViewDelegate::paintRowHeader(QPainter *painter,
+                                         const QStyleOptionViewItem &option,
+                                         const QModelIndex &index,
+                                         RuleNode * node) const
 {
     Q_UNUSED(node);
     QStyleOptionViewItem newOpt = option;
@@ -104,9 +115,13 @@ void RuleSetViewDelegate::paintRowHeader (QPainter *painter, const QStyleOptionV
     QItemDelegate::paint(painter, newOpt, index);
     painter->setPen( QColor("lightgray") );
     painter->drawRect(option.rect);
+
 }
 
-void RuleSetViewDelegate::paintGroup(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index, RuleNode * node) const
+void RuleSetViewDelegate::paintGroup(QPainter *painter,
+                                     const QStyleOptionViewItem &option,
+                                     const QModelIndex &index,
+                                     RuleNode * node) const
 {
     Q_UNUSED(node);
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintGroup";
@@ -114,7 +129,8 @@ void RuleSetViewDelegate::paintGroup(QPainter *painter, const QStyleOptionViewIt
     QItemDelegate::paint(painter, option, index);
 }
 
-void RuleSetViewDelegate::drawIcons(QPainter *painter, QRect rect,
+void RuleSetViewDelegate::drawIcons(QPainter *painter,
+                                    QRect rect,
                                     const QStringList &icons) const
 {
     int x = rect.left();
@@ -142,22 +158,38 @@ void RuleSetViewDelegate::drawIconAndText(QPainter *painter,
     int y = rect.top();
     int iconWidth = 0;
 
-    if (!icon.isEmpty() && st->getShowIconsInRules())
+    if (st->getShowIconsInRules())
     {
-        QPixmap pm;
-        if (!icon.isEmpty()) LoadPixmap(calculateIconName(icon, negation), pm);
-        painter->drawPixmap(x,y,pm);
-        iconWidth = pm.width();
+        if (!icon.isEmpty())
+        {
+            QPixmap pm;
+            LoadPixmap(calculateIconName(icon, negation), pm);
+            painter->drawPixmap(x,y,pm);
+            iconWidth = pm.width();
+        } else
+        {
+            if (FWBSettings::SIZE25X25 == st->getIconsInRulesSize())
+                iconWidth = 25;
+            else
+                iconWidth = 16;
+        }
+
         x += iconWidth + ICON_TEXT_GAP;
     }
 
     if (!text.isEmpty())
     {
-        painter->drawText( x, y , rect.width() - iconWidth - ICON_TEXT_GAP, rect.height(), Qt::AlignLeft|Qt::AlignVCenter, text );
+        painter->drawText(x, y,
+                          rect.width() - iconWidth - ICON_TEXT_GAP,
+                          rect.height(),
+                          Qt::AlignLeft|Qt::AlignVCenter,
+                          text );
     }
 }
 
-void RuleSetViewDelegate::drawSelectedFocus(QPainter *painter, const QStyleOptionViewItem &option,QRect &rect) const
+void RuleSetViewDelegate::drawSelectedFocus(QPainter *painter,
+                                            const QStyleOptionViewItem &option,
+                                            QRect &rect) const
 {
     if (option.state & QStyle::State_HasFocus)
     {
@@ -178,110 +210,144 @@ void RuleSetViewDelegate::paintRule(QPainter *painter,
     QVariant v = index.data(Qt::DisplayRole);
     if (!v.isValid()) return;
 
-    // if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintRule"
-    //                        << v;
-
     if (node != 0)
     {
         FWOptions *ropt = node->rule->getOptionsObject();
         QString color = ropt->getStr("color").c_str();
         if (!color.isEmpty())
-            painter->fillRect(option.rect,QColor(color));
+        {
+            painter->fillRect(option.rect, QColor(color));
+        } else
+        {
+            if (option.state & QStyle::State_Selected)
+            {
+                painter->fillRect(option.rect,
+                                  option.palette.color(QPalette::Highlight));
+            }
+        }
     }
+
+    QStyleOptionViewItem new_opt = option;
+    new_opt.palette.setColor(QPalette::Highlight, standard_highlight);
+
     ColDesc colDesc = index.data(Qt::UserRole).value<ColDesc>();
     switch (colDesc.type)
     {
         case ColDesc::Object :
-            paintObject(painter, option, v);
+            paintObject(painter, new_opt, v);
             break;
         case ColDesc::Direction :
-            paintDirection(painter, option, v);
+            paintDirection(painter, new_opt, v);
             break;
         case ColDesc::Action :
-            paintAction(painter, option, v);
+            paintAction(painter, new_opt, v);
             break;
         case ColDesc::Time :
-            paintObject(painter, option, v);
+            paintObject(painter, new_opt, v);
             break;
         case ColDesc::Options :
-            paintOptions(painter, option, v);
+            paintOptions(painter, new_opt, v);
             break;
         case ColDesc::Comment :
-            paintComment(painter, option, v);
+            paintComment(painter, new_opt, v);
             break;
         case ColDesc::Metric :
-            paintMetric(painter, option, v);
+            paintMetric(painter, new_opt, v);
             break;
         default:
-            QItemDelegate::paint(painter, option, index);
+            QItemDelegate::paint(painter, new_opt, index);
     }
+
     painter->setPen( QColor("lightgray") );
-    painter->drawRect(option.rect);
+    painter->drawRect(new_opt.rect);
 }
 
-void RuleSetViewDelegate::paintDirection(QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
+void RuleSetViewDelegate::paintDirection(
+    QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintDirection";
-    DrawingContext ctx = initContext(option.rect);
+    DrawingContext ctx = initContext(option.rect, true); // useEnireSpace=true
     QString dir = v.value<QString>();
-    drawSelectedFocus(painter, option, ctx.objectRect);
+    if (option.state & QStyle::State_Selected)
+        drawSelectedFocus(painter, option, ctx.objectRect);
     QString text = (st->getShowDirectionText())?dir:"";
-    drawIconAndText(painter, ctx.drawRect, dir,text);
+    ctx = initContext(option.rect, false); // useEnireSpace=false
+    drawIconAndText(painter, ctx.drawRect, dir, text);
 }
 
-void RuleSetViewDelegate::paintAction(QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
+void RuleSetViewDelegate::paintAction(
+    QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintAction";
-    DrawingContext ctx = initContext(option.rect);
+    DrawingContext ctx = initContext(option.rect, true); // useEnireSpace=true
     ActionDesc actionDesc = v.value<ActionDesc>();
-    drawSelectedFocus(painter, option, ctx.objectRect);
+    if (option.state & QStyle::State_Selected)
+        drawSelectedFocus(painter, option, ctx.objectRect);
     QString text = constructActionText(actionDesc);
+    ctx = initContext(option.rect, false); // useEnireSpace=false
     drawIconAndText(painter, ctx.drawRect, actionDesc.name, text);
 }
 
-void RuleSetViewDelegate::paintOptions(QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
+void RuleSetViewDelegate::paintOptions(
+    QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
 {
-    //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintOptions";
-    DrawingContext ctx = initContext(option.rect);
-    drawSelectedFocus(painter, option, ctx.objectRect);
-    QStringList icons;
-    icons = v.value<QStringList>();
-    QStringList text;
+    DrawingContext ctx = initContext(option.rect, true); // useEnireSpace=true
 
-    if (st->getShowIconsInRules())
+    if (option.state & QStyle::State_Selected)
+        drawSelectedFocus(painter, option, ctx.objectRect);
+
+    QStringList icons = v.value<QStringList>();
+
+    // draw option icons vertically instead of horizontally  #2367
+    // for options "Tag", "Classify" and "Route" the "icon name" actually
+    // consists of the name of the icon, plus space and parameter
+
+    int y = ctx.objectRect.top();
+    foreach(QString icon, icons)
     {
-        drawIcons(painter, ctx.drawRect, icons);
-    } else
-    {
-        for (int i=0; i<icons.size(); i++)
-        {
-            if (icons[i].contains("Log")) text.push_back(tr("log"));
-            if (icons[i].contains("Options")) text.push_back(tr("(options)"));
-        }
-        drawIconAndText(painter, ctx.drawRect, "", text.join(", "), false);
+        QRect itemRect = QRect(ctx.objectRect.left(), y,
+                               ctx.objectRect.width(), ctx.itemHeight);
+
+        QString parameter = icon.section(" ", 1);
+        if (icon.contains("Log")) parameter = tr("log");
+        if (icon.contains("Options")) parameter = tr("(options)");
+
+        drawIconAndText(painter,
+                        itemRect.adjusted(
+                            HORIZONTAL_MARGIN, VERTICAL_MARGIN,
+                            -HORIZONTAL_MARGIN, -VERTICAL_MARGIN),
+                        icon, parameter, false);
+
+        y += ctx.itemHeight;
+
     }
 }
 
-
-void RuleSetViewDelegate::paintComment(QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
+void RuleSetViewDelegate::paintComment(
+    QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintComment";
     DrawingContext ctx = initContext(option.rect, true);
     QString comment = v.value<QString>();
-    drawSelectedFocus(painter, option, ctx.objectRect);
+    if (option.state & QStyle::State_Selected)
+        drawSelectedFocus(painter, option, ctx.objectRect);
     painter->drawText( ctx.drawRect, Qt::AlignLeft|Qt::AlignTop, comment);
 }
 
-void RuleSetViewDelegate::paintMetric(QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
+void RuleSetViewDelegate::paintMetric(
+    QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintMetric";
     DrawingContext ctx = initContext(option.rect, true);
     QString metric = v.value<QString>();
-    drawSelectedFocus(painter, option, ctx.objectRect);
+    if (option.state & QStyle::State_Selected)
+        drawSelectedFocus(painter, option, ctx.objectRect);
+    ctx = initContext(option.rect, false); // useEnireSpace=false
     drawIconAndText(painter, ctx.drawRect, QString(), metric);
 }
 
-void RuleSetViewDelegate::paintObject(QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
+void RuleSetViewDelegate::paintObject(
+    QPainter *painter, const QStyleOptionViewItem &option, const QVariant &v) const
 {
     //if (fwbdebug) qDebug() << "RuleSetViewDelegate::paintObject";
     RuleElement *re = (RuleElement *)v.value<void *>();
@@ -294,11 +360,8 @@ void RuleSetViewDelegate::paintObject(QPainter *painter, const QStyleOptionViewI
 
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
     {
-        FWObject *o1= *i;
-        if (FWReference::cast(o1)!=NULL)
-            o1=FWReference::cast(o1)->getPointer();
-        if (o1==NULL)
-            continue ;
+        FWObject *o1 = FWReference::getObject(*i);
+        if (o1==NULL) continue ;
 
         QRect itemRect = QRect(ctx.objectRect.left(), y, ctx.objectRect.width(), ctx.itemHeight);
 
@@ -329,7 +392,8 @@ void RuleSetViewDelegate::paintObject(QPainter *painter, const QStyleOptionViewI
     }
 }
 
-QSize RuleSetViewDelegate::drawIconInRule(QPainter *p, int x, int y, QString name, bool neg) const
+QSize RuleSetViewDelegate::drawIconInRule(
+    QPainter *p, int x, int y, QString name, bool neg) const
 {
     if (!st->getShowIconsInRules()) return QSize();
 
@@ -472,7 +536,7 @@ QSize RuleSetViewDelegate::calculateCellSizeForRule(
             calculated = calculateCellSizeForIconAndText(index);
             break;
         case ColDesc::Options :
-            calculated = calculateCellSizeForIconAndText(index);
+            calculated = calculateCellSizeForOptions(index);
             break;
         default :
             calculated = QSize(0,0);
@@ -563,6 +627,30 @@ QSize RuleSetViewDelegate::calculateCellSizeForIconAndText(
     return QSize(w, h);
 }
 
+QSize RuleSetViewDelegate::calculateCellSizeForOptions(
+    const QModelIndex & index) const
+{
+    QVariant v = index.data(Qt::DisplayRole);
+    QStringList icons = v.value<QStringList>();
+
+    // for options "Tag", "Classify" and "Route" the "icon name" actually
+    // consists of the name of the icon, plus space and parameter
+
+    int itemHeight = getItemHeight();
+    QSize iconSize = getIconSize();
+
+    int h = 0;
+    int w = 0;
+
+    foreach(QString icon, icons)
+    {
+        QString parameter = icon.section(" ", 1);
+        QSize size = getTextSize(parameter, Qt::TextSingleLine);
+        h += itemHeight;
+        w = qMax(w, iconSize.width() + size.width() + ICON_TEXT_GAP);
+    }
+    return QSize(w+HORIZONTAL_MARGIN*2, h);
+}
 
 QString RuleSetViewDelegate::objectText(RuleElement *re,FWObject *obj) const
 {
@@ -575,7 +663,6 @@ QString RuleSetViewDelegate::objectText(RuleElement *re,FWObject *obj) const
         if (RuleElementRDst::isA(re)) return QString(tr("Default"));
         if (RuleElementRGtw::isA(re) ||
             RuleElementRItf::isA(re))  return QString("");
-        if (RuleElementItf::isA(re))   return QString(tr("All"));
         if (RuleElementItfInb::isA(re))   return QString(tr("Auto"));
         if (RuleElementItfOutb::isA(re))   return QString(tr("Auto"));
 
@@ -591,14 +678,21 @@ QString RuleSetViewDelegate::objectText(RuleElement *re,FWObject *obj) const
     else return QString::fromUtf8(obj->getName().c_str());
 }
 
-DrawingContext RuleSetViewDelegate::initContext(QRect rect, bool useEnireSpace) const
+DrawingContext RuleSetViewDelegate::initContext(
+    QRect rect, bool useEnireSpace) const
 {
     DrawingContext ctx;
 
     ctx.iconSize = getIconSize();
     ctx.itemHeight = getItemHeight();
-    ctx.objectRect = QRect(rect.left()+1, rect.top()+1, rect.width()-1, useEnireSpace?rect.height()-1:ctx.itemHeight);
-    ctx.drawRect = ctx.objectRect.adjusted(HORIZONTAL_MARGIN, VERTICAL_MARGIN, -HORIZONTAL_MARGIN, -VERTICAL_MARGIN);
+
+    ctx.objectRect =
+        QRect(rect.left()+1, rect.top()+1,
+              rect.width()-1, useEnireSpace?rect.height()-1:ctx.itemHeight);
+
+    ctx.drawRect =
+        ctx.objectRect.adjusted(HORIZONTAL_MARGIN, VERTICAL_MARGIN,
+                                -HORIZONTAL_MARGIN, -VERTICAL_MARGIN);
 
     return ctx;
 }
