@@ -116,7 +116,7 @@ instDialog::instDialog(QWidget *p) : QDialog(p)
     err_re.push_back("(fwb_[^:]*: \\S*\\.cpp:\\d{1,}: .*: Assertion .* failed.)");
     foreach(string re, err_re)
     {
-        error_re.push_back(QRegExp(re.c_str()));
+        error_re.push_back(QRegExp(re.c_str(), Qt::CaseInsensitive));
     }
 
     list<string> warn_re;
@@ -461,9 +461,24 @@ void instDialog::showPage(const int page)
             // Flag compile_complete is set in instDialog::mainLoopCompile()
             if (compile_complete)
             {
-                setNextEnabled(page, true);
+                // See #2037: enable "Next" button only if there is at
+                // least one firewall that was successfully compiled.
+                bool can_install = false;
+                list<Firewall*>::iterator i;
+                for(i=compile_fw_list.begin(); i!=compile_fw_list.end(); ++i)
+                {
+                    Firewall *fw = *i;
+                    if (compile_status[fw] ==
+                        fwcompiler::BaseCompiler::FWCOMPILER_SUCCESS)
+                    {
+                        setNextEnabled(page, true);
+                        m_dialog->nextButton->setDefault(true);
+                        can_install = true;
+                        break;
+                    }
+                }
+                if (!can_install) setFinishEnabled(page, true);
                 setBackEnabled(page, true);
-                m_dialog->nextButton->setDefault(true);
             } else
             {
                 mw->fileSave();
@@ -582,7 +597,17 @@ int instDialog::findFilesToInspect(QStringList &files)
 
     foreach(Firewall *f, fwlist)
     {
+        /*
+         * get full path to the generated file. The path is built from
+         * the file name returned by
+         * FirewallInstaller::getGeneratedFileName() and directory
+         * path from the .fwb file. Note that we use the same
+         * algorithm when GUI launches policy compiler, except there
+         * the path is passed to it via "-d" command line option.
+         */
         QString mainFile = FirewallInstaller::getGeneratedFileFullPath(f);
+//        QString mainFile = FirewallInstaller::getGeneratedFileName(f);
+
         if (!QFile::exists(mainFile)) continue;
         instConf cnf;
         cnf.fwobj = f;
@@ -596,10 +621,6 @@ int instDialog::findFilesToInspect(QStringList &files)
     return files.size();
 }
  
-void instDialog::testRunRequested()
-{
-}
-
 struct CaseInsensitiveComparison :
     public std::binary_function<libfwbuilder::FWObject*, libfwbuilder::FWObject*, bool>
 {
