@@ -67,6 +67,7 @@
 #include "fwbuilder/XMLTools.h"
 
 #include "fwcompiler/Preprocessor.h"
+#include "fwcompiler/GroupRegistry.h"
 #include "fwcompiler/exceptions.h"
 
 #include <QStringList>
@@ -209,6 +210,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
     Cluster *cluster = NULL;
     Firewall *fw = NULL;
 
+
     getFirewallAndClusterObjects(cluster_id, firewall_id, &cluster, &fw);
 
     try
@@ -397,6 +399,8 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
         qDebug() << "rulesets_to_remote_file_names=" << rulesets_to_remote_file_names;
 #endif
 
+        GroupRegistry group_registry;
+
         int routing_rules_count = 0;
 
         vector<int> ipv4_6_runs;
@@ -449,6 +453,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
             {
                 Preprocessor_pf* prep = new Preprocessor_pf(
                     objdb , fw, ipv6_policy);
+                prep->setSingleRuleCompileMode(single_rule_id);
                 if (inTestMode()) prep->setTestMode();
                 if (inEmbeddedMode()) prep->setEmbeddedMode();
                 prep->compile();
@@ -471,7 +476,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 if (table_factories.count(ruleset_name) == 0)
                 {
                     table_factories[ruleset_name] =
-                        new fwcompiler::TableFactory(this, persistent_objects);
+                        new fwcompiler::TableFactory(this, fw, persistent_objects, &group_registry);
                 }
 
                 NATCompiler_pf n( objdb, fw, ipv6_policy, oscnf.get(),
@@ -481,6 +486,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 n.setSourceRuleSet( nat );
                 n.setRuleSetName(nat->getName());
                 n.setPersistentObjects(persistent_objects);
+                n.setGroupRegistry(&group_registry);
 
                 n.setSingleRuleCompileMode(single_rule_id);
                 n.setDebugLevel( dl );
@@ -546,7 +552,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 if (table_factories.count(ruleset_name) == 0)
                 {
                     table_factories[ruleset_name] =
-                        new fwcompiler::TableFactory(this, persistent_objects);
+                        new fwcompiler::TableFactory(this, fw, persistent_objects, &group_registry);
                 }
 
                 PolicyCompiler_pf c( objdb, fw, ipv6_policy, oscnf.get(),
@@ -557,6 +563,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
                 c.setSourceRuleSet( policy );
                 c.setRuleSetName(policy->getName());
                 c.setPersistentObjects(persistent_objects);
+                c.setGroupRegistry(&group_registry);
 
                 c.setSingleRuleCompileMode(single_rule_id);
                 c.setDebugLevel( dl );
