@@ -119,6 +119,39 @@ string NATCompiler_pf::debugPrintRule(libfwbuilder::Rule *r)
         " (type=" + rule->getRuleTypeAsString() + ")";
 }
 
+void NATCompiler_pf::_expand_addr(Rule *rule,
+                                  FWObject *s,
+                                  bool expand_cluster_interfaces_fully)
+{
+    if (RuleElementTSrc::isA(s))
+    {
+        // do not replace interfaces with their ip addresses in TSrc
+        // to be able to generate "nat ... -> (em0)" command later
+        list<FWObject*> interfaces_in_re;
+        for (FWObject::iterator i1=s->begin(); i1!=s->end(); ++i1) 
+        {
+            FWObject *o = FWReference::getObject(*i1);
+            assert(o);
+            if (Interface::isA(o))
+                interfaces_in_re.push_back(o);
+        }
+        if (interfaces_in_re.size() > 1)
+        {
+            for (list<FWObject*>::iterator i=interfaces_in_re.begin();
+                 i!=interfaces_in_re.end(); ++i) s->removeRef(*i);
+
+            NATCompiler::_expand_addr(
+                rule, s, expand_cluster_interfaces_fully);
+
+            for (list<FWObject*>::iterator i=interfaces_in_re.begin();
+                 i!=interfaces_in_re.end(); ++i) s->addRef(*i);
+
+        }
+    } else
+        NATCompiler::_expand_addr(
+            rule, s, expand_cluster_interfaces_fully);
+}
+
 bool NATCompiler_pf::NATRuleType::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
@@ -1215,6 +1248,8 @@ void NATCompiler_pf::compile()
     add(new replaceClusterInterfaceInItfOutb(
             "replace cluster interfaces with member interfaces in "
             "the Interface rule element"));
+    add(new singleObjectNegationItfOutb(
+            "process single object negation in inbound Itf"));
     add(new ItfOutbNegation("process negation in Itf"));
 
     add( new recursiveGroupsInOSrc("check for recursive groups in OSRC") );
@@ -1233,7 +1268,16 @@ void NATCompiler_pf::compile()
     add( new emptyGroupsInTDst(    "check for empty groups in TDST"    ) );
     add( new emptyGroupsInTSrv(    "check for empty groups in TSRV"    ) );
 
+    if (fw->getOptionsObject()->getBool("preserve_group_names"))
+    {
+        add(new RegisterGroupsAndTablesInOSrc(
+                "register object groups and tables in OSrc"));
+        add(new RegisterGroupsAndTablesInODst(
+                "register object groups and tables in ODst"));
+    }
+
     add( new ExpandGroups( "expand groups" ) );
+    add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
     add( new eliminateDuplicatesInOSRC(  "eliminate duplicates in OSRC") );
     add( new eliminateDuplicatesInODST(  "eliminate duplicates in ODST") );
     add( new eliminateDuplicatesInOSRV(  "eliminate duplicates in OSRV") );
@@ -1257,6 +1301,8 @@ void NATCompiler_pf::compile()
     add( new processMultiAddressObjectsInTDst(
              "process MultiAddress objects in TDst") );
 
+    add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+
     add( new splitOnOSrv( "split rule on original service" ) );
     add( new fillTranslatedSrv( "fill translated service" ) );
 
