@@ -87,6 +87,10 @@ namespace fwcompiler
 
         virtual std::string debugPrintRule(libfwbuilder::Rule *rule);
 
+	virtual void _expand_addr(libfwbuilder::Rule *rule,
+                                  libfwbuilder::FWObject *s,
+                                  bool expand_cluster_interfaces_fully);
+
         /**
          * analyse given address and decide which interface this NAT
          * rule should be tied to. If interface is found, use
@@ -316,7 +320,6 @@ namespace fwcompiler
                                  libfwbuilder::RuleElementTDst::TYPENAME) {}
         };
 
-
         /**
          * this processor is only called if we are using tables. It
          * creates two tables for each rule element Processor
@@ -366,7 +369,7 @@ namespace fwcompiler
 
             virtual void _printProtocol(libfwbuilder::Service *srv);
             virtual void _printPort(libfwbuilder::Service *srv, bool lhs);
-            virtual void _printSrcPort(libfwbuilder::Service *srv);
+            virtual void _printSrcPort(libfwbuilder::Service *srv, bool lhs);
 
             virtual void _printAddrList(libfwbuilder::FWObject  *o,bool negflag);
             virtual void _printREAddr(libfwbuilder::RuleElement *o);
