@@ -40,6 +40,9 @@
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/DNSName.h"
+#include "fwbuilder/UserService.h"
+#include "fwbuilder/AddressTable.h"
+#include "fwbuilder/AttachedNetworks.h"
 
 #include <assert.h>
 #include <QStringList>
@@ -68,7 +71,7 @@ void NATCompiler_pf::PrintRule::_printInterface(NATRule *rule)
     RuleElementItf *intf_re = rule->getItfOutb();
     QStringList rule_interfaces;
 
-    if (!intf_re->isAny())
+    if ( ! intf_re->isAny())
     {
         for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
         {
@@ -81,6 +84,7 @@ void NATCompiler_pf::PrintRule::_printInterface(NATRule *rule)
             rule_interfaces.push_back("}");
         }
         compiler->output << "on "
+                         << string((intf_re->getBool("single_object_negation")) ? "! " : " ")
                          << rule_interfaces.join(" ").toStdString()
                          << " ";
     }
@@ -153,7 +157,7 @@ bool NATCompiler_pf::PrintRule::processNext()
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr(osrcrel);
-            _printSrcPort(osrv);
+            _printSrcPort(osrv, true);
             compiler->output  << "to ";
             _printREAddr(odstrel);
             _printPort(osrv, true);
@@ -195,14 +199,14 @@ bool NATCompiler_pf::PrintRule::processNext()
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
-            _printSrcPort(osrv);
+            _printSrcPort(osrv, true);
             compiler->output  << "to ";
             _printREAddr( odstrel );
             _printPort( osrv, true );
 
             compiler->output  << "nat-to ";
             _printREAddr( tsrcrel );
-            _printSrcPort(tsrv);
+            _printSrcPort(tsrv, false);
             _printNATRuleOptions(rule);
 
             compiler->output  << endl;
@@ -216,14 +220,14 @@ bool NATCompiler_pf::PrintRule::processNext()
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
-            _printSrcPort(osrv);
+            _printSrcPort(osrv, true);
             compiler->output  << "to ";
             _printREAddr( odstrel );
             _printPort( osrv, true );
 
             compiler->output  << "-> ";
             _printREAddr( tsrcrel );
-            _printSrcPort(tsrv);
+            _printSrcPort(tsrv, false);
             _printNATRuleOptions(rule);
 
             compiler->output  << endl;
@@ -243,7 +247,7 @@ bool NATCompiler_pf::PrintRule::processNext()
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
-            _printSrcPort(osrv);        // this is where it is different from NATRule::Redirect
+            _printSrcPort(osrv, true);        // this is where it is different from NATRule::Redirect
             compiler->output  << "to ";
             _printREAddr( odstrel );
             _printPort(osrv, true);
@@ -261,7 +265,7 @@ bool NATCompiler_pf::PrintRule::processNext()
             _printProtocol(osrv);
             compiler->output  << "from ";
             _printREAddr( osrcrel );
-            _printSrcPort(osrv);        // this is where it is different from NATRule::Redirect
+            _printSrcPort(osrv, true);        // this is where it is different from NATRule::Redirect
             compiler->output  << "to ";
             _printREAddr( odstrel );
             _printPort(osrv, true);
@@ -386,12 +390,14 @@ void NATCompiler_pf::PrintRule::_printProtocol(Service *srv)
         if (minus_p != string::npos) return;
     }
 
-    if ( !TagService::isA(srv))
+    if (!srv->isAny() && !TagService::isA(srv) && !UserService::isA(srv) && 
+        srv->getProtocolName()!="ip")
     {
-        string s = srv->getProtocolName();
-        if (s=="ip" || s=="any") s="{tcp udp icmp}";
-        compiler->output << "proto " <<  s << " ";
+	compiler->output << "proto ";
+	compiler->output << srv->getProtocolName();
+	compiler->output << " ";
     }
+
 }
 
 /*
@@ -433,7 +439,7 @@ void NATCompiler_pf::PrintRule::_printPort(Service *srv, bool lhs)
 /*
  * Print port range spec using source ports of the given service object
  */
-void NATCompiler_pf::PrintRule::_printSrcPort(libfwbuilder::Service *srv)
+void NATCompiler_pf::PrintRule::_printSrcPort(Service *srv, bool lhs)
 {
     if (TCPUDPService::cast(srv))
     {
@@ -444,7 +450,10 @@ void NATCompiler_pf::PrintRule::_printSrcPort(libfwbuilder::Service *srv)
             compiler->output << "port " << srs;
             if (sre != 0 && sre != srs)
             {
-                compiler->output << ":" << sre;
+                if (lhs)
+                    compiler->output << ":" << sre;
+                else
+                    compiler->output << ":*";
             }
             compiler->output  << " ";
         }
@@ -462,22 +471,16 @@ void NATCompiler_pf::PrintRule::_printREAddr(RuleElement *rel)
     FWObject *o=rel->front();
     if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
-    Address *addr= Address::cast(o);
+//    Address *addr= Address::cast(o);
 
     _printNegation(rel);
 
-    if (rel->size()==1 && ! o->getBool("pf_table") )
+    if (rel->size()==1)
     {
-	_printAddr( addr );
+	_printAddr(o);
     } else
     {
-        if (o->getBool("pf_table"))
-        {
-            compiler->output << "<" << o->getName() << "> ";
-        } else
-        {
-            _printAddrList(rel,rel->getNeg());
-        }
+        _printAddrList(rel, rel->getNeg());
     }
 }
 
@@ -490,9 +493,7 @@ void NATCompiler_pf::PrintRule::_printAddrList(FWObject  *grp,bool )
         if (i!=grp->begin())  compiler->output << ", ";
         FWObject *o= *i;
         if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-        Address *s=Address::cast( o );
-        assert(s);
-        _printAddr(s);
+        _printAddr(o);
     }
     compiler->output << "} ";
 }
@@ -507,29 +508,39 @@ void NATCompiler_pf::PrintRule::_printAddr(FWObject *o)
             compiler->output <<  atrt->getSourceName() << " ";
             return;
         }
-        // at this time we only support two types of MultiAddress
-        // objects: AddressTable and DNSName. Both should be converted
-        // to MultiAddressRunTime at this point. If we get some other
-        // kind of MultiAddressRunTime object, we do not know what to do
-        // with it so we stop.
+        if (atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+        {
+            compiler->output << "<" << o->getName() << "> ";
+            return;
+        }
+        if (atrt->getSubstitutionTypeName()==AttachedNetworks::TYPENAME)
+        {
+            compiler->output << atrt->getSourceName() << ":network";
+            return ;
+        }
+
         assert(atrt==NULL);
     }
 
     if (Interface::cast(o)!=NULL)
     {
-        Interface *iface=Interface::cast(o);
-        if (iface->isDyn())
-        {
-            compiler->output << "(" << iface->getName() << ") ";
-            return;
-        }
+        compiler->output << "(" << o->getName() << ") ";
+        return;
     }
 
-    Address *a = Address::cast(o);
-    const InetAddr *addr = a->getAddressPtr();
+    if (o->getBool("pf_table"))
+    {
+        compiler->output << "<" << o->getName() << "> ";
+        return;
+    }
+
+    Address *addr_obj = Address::cast(o);
+    assert(addr_obj!=NULL);
+
+    const InetAddr *addr = addr_obj->getAddressPtr();
     if (addr)
     {
-        InetAddr mask = *(a->getNetmaskPtr());
+        InetAddr mask = *(addr_obj->getNetmaskPtr());
 
         if (Interface::cast(o)!=NULL || Address::cast(o)->dimension()==1)
         {
