@@ -159,10 +159,29 @@ string PolicyCompiler_ipfw::PrintRule::_printTCPFlags(TCPService *srv)
 
 void PolicyCompiler_ipfw::PrintRule::_printAction(PolicyRule *rule)
 {
-    FWOptions *ruleopt =rule->getOptionsObject();
-    Service *srv=compiler->getFirstSrv(rule);    assert(srv);
+    FWOptions *ruleopt = rule->getOptionsObject();
+    Service *srv = compiler->getFirstSrv(rule);
+    assert(srv);
 
-    switch (rule->getAction()) {
+    if (rule->getClassification())
+    {
+        int portNum = ruleopt->getInt("ipfw_pipe_queue_num");
+        switch (ruleopt->getInt("ipfw_classify_method"))
+        {
+        case DUMMYNETPIPE:
+            compiler->output << "pipe "   << portNum << " ";
+            return;
+        case DUMMYNETQUEUE:
+            compiler->output << "queue "  << portNum << " ";
+            return;
+        default:
+            compiler->output << "divert " << portNum << " ";
+            return;
+        }
+    }
+
+    switch (rule->getAction())
+    {
     case PolicyRule::Skip:
         compiler->output << "skipto " << rule->getInt("skip_to") << " ";
         break;
@@ -206,24 +225,6 @@ void PolicyCompiler_ipfw::PrintRule::_printAction(PolicyRule *rule)
         }
 	break;
 
-    case PolicyRule::Classify:
-    {
-        int portNum = ruleopt->getInt("ipfw_pipe_queue_num");
-        switch (ruleopt->getInt("ipfw_classify_method"))
-        {
-        case DUMMYNETPIPE:
-            compiler->output << "pipe "   << portNum << " ";
-            break;
-        case DUMMYNETQUEUE:
-            compiler->output << "queue "  << portNum << " ";
-            break;
-        default:
-            compiler->output << "divert " << portNum << " ";
-            break;
-        }
-    }
-    break;
-
     case PolicyRule::Pipe:
         compiler->output << "divert " << ruleopt->getInt("ipfw_pipe_port_num") << " ";
         break;
@@ -240,6 +241,7 @@ void PolicyCompiler_ipfw::PrintRule::_printAction(PolicyRule *rule)
 
 //   compiler->output << rule->getActionAsString() << " ";
     }
+
 }
 
 /*
