@@ -29,6 +29,7 @@
 #include "NATCompiler_pf.h"
 
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/DNSName.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/Firewall.h"
@@ -426,8 +427,7 @@ bool PolicyCompiler_pf::SplitDirection::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
-    if (rule->getDirection()==PolicyRule::Both &&
-        rule->getAction()==PolicyRule::Route)
+    if (rule->getDirection()==PolicyRule::Both && rule->getRouting())
     {
 	PolicyRule *r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
@@ -509,42 +509,28 @@ bool PolicyCompiler_pf::setQuickFlag::processNext()
 
     FWOptions *ropt = rule->getOptionsObject();
 
-    // as of 4.2.0 build 3477 we provide checkboxes to make Tag and
-    // Classify actions (PF) terminating or non-terminating on
-    // per-rule basis. Old behavior: Tag was non-terminating and
-    // Classify was terminating. Set options accordingly if they are
-    // not set.
-
     switch (rule->getAction())
     {
     case PolicyRule::Scrub:
     case PolicyRule::Accounting:
     case PolicyRule::Branch:
+    case PolicyRule::Continue:
         break;
 
-    case PolicyRule::Tag:
-    {
-        string pf_tag_terminating = ropt->getStr("pf_tag_terminating");
-        if (pf_tag_terminating.empty())
-            ropt->setBool("pf_tag_terminating", false);
-        if (ropt->getBool("pf_tag_terminating")) rule->setBool("quick", true);
-        break;
-    }
-
-    case PolicyRule::Classify:
-    {
-        string pf_classify_terminating = ropt->getStr("pf_classify_terminating");
-        if (pf_classify_terminating.empty())
-            ropt->setBool("pf_classify_terminating", true);
-        if (ropt->getBool("pf_classify_terminating")) rule->setBool("quick", true);
-        break;
-    }
-
     default:
         rule->setBool("quick", true);
         break;
     }
 
+    // as of 4.2.0 build 3477 we provide checkboxes to make Tag and
+    // Classify actions (PF) terminating or non-terminating on
+    // per-rule basis. Old behavior: Tag was non-terminating and
+    // Classify was terminating. Set options accordingly if they are
+    // not set.
+    // 
+    // TODO #2367: now instead of checkboxes, user should use actions Accept
+    // or Continue
+
     return true;
 }
 
@@ -670,6 +656,8 @@ bool PolicyCompiler_pf::addLoopbackForRedirect::processNext()
         for (FWObject::iterator j=dst->begin(); j!=dst->end(); j++) 
         {
             FWObject *o2 = FWReference::getObject(*j);
+            if (o2->getName() == "self" && DNSName::isA(o2)) continue;
+
             Address *a = Address::cast( o2 );
             assert(a);
 
@@ -913,6 +901,14 @@ void PolicyCompiler_pf::compile()
 //        add(new doDstNegation("process negation in Dst"));
     add(new doSrvNegation("process negation in Srv"));
 
+    if (fw->getOptionsObject()->getBool("preserve_group_names"))
+    {
+        add(new RegisterGroupsAndTablesInSrc(
+                "register object groups and tables in Src"));
+        add(new RegisterGroupsAndTablesInDst(
+                "register object groups and tables in Dst"));
+    }
+
 // ExpandGroups opens groups, as well as groups in groups etc.
     add(new ExpandGroups("expand groups"));
     add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
@@ -952,7 +948,20 @@ void PolicyCompiler_pf::compile()
     //    "process interface policy rules and store interface ids"));
 
     add(new splitIfFirewallInSrc("split rule if firewall is in Src"));
+    add(new ReplaceFirewallObjectWithSelfInSrc(
+            "Replace firewall object with 'self' in Src"));
+
     add(new splitIfFirewallInDst("split rule if firewall is in Dst"));
+    add(new ReplaceFirewallObjectWithSelfInDst(
+            "Replace firewall object with 'self' in Dst"));
+
+    // call these again since "self" is a MultiAddress object
+    add( new swapMultiAddressObjectsInSrc(
+             " swap MultiAddress -> MultiAddressRunTime in Src"));
+    add( new swapMultiAddressObjectsInDst(
+             " swap MultiAddress -> MultiAddressRunTime in Dst"));
+
+
     add(new fillDirection("determine directions"));
 
 // commented out for bug #2828602
@@ -964,6 +973,7 @@ void PolicyCompiler_pf::compile()
             "add loopback to rules that permit redirected services"));
     add(new ExpandMultipleAddresses(
             "expand objects with multiple addresses"));
+
     add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
     add(new checkForDynamicInterfacesOfOtherObjects(
             "check for dynamic interfaces of other hosts and firewalls"));
