@@ -4,9 +4,7 @@
 
                  Copyright (C) 2002 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
-
-  $Id$
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -45,6 +43,7 @@
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/AddressTable.h"
 #include "fwbuilder/XMLTools.h"
+#include "fwbuilder/AttachedNetworks.h"
 
 #include <iostream>
 #include <iomanip>
@@ -76,8 +75,18 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
 
     switch (rule->getAction())
     {
-    case PolicyRule::Tag:
-    {
+ // case PolicyRule::Classify:  #2367
+ // case PolicyRule::Route:     #2367
+    case PolicyRule::Accept:  
+    case PolicyRule::Accounting:
+        compiler->output << "pass ";
+        break;
+
+    case PolicyRule::Deny:
+        compiler->output << "block ";
+        break;
+
+    case PolicyRule::Continue:
         if (XMLTools::version_compare(version, "4.6")>=0)
         {
             compiler->output << "match ";
@@ -86,45 +95,39 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
             compiler->output << "pass ";
         }
         break;
-    }
-    case PolicyRule::Accept:  
-    case PolicyRule::Classify:
-    case PolicyRule::Accounting:
-    case PolicyRule::Route:
-        compiler->output << "pass ";
-        break;
-
-    case PolicyRule::Deny:
-        compiler->output << "block ";
-        break;
 
     case PolicyRule::Reject: 
-	if (TCPService::isA(srv)) compiler->output << "block return-rst ";
-	else
+    {
+        string aor = ruleopt->getStr("action_on_reject");
+        string code;
+        if ( aor.find("ICMP")!=string::npos )
         {
-	    string aor=ruleopt->getStr("action_on_reject");
-	    string code;
-	    if ( aor.find("ICMP")!=string::npos )
+            code = "return-icmp ";
+            if (aor.find("unreachable")!=string::npos )
             {
-		code="return-icmp ";
-		if (aor.find("unreachable")!=string::npos )
-                {
- 		    if (aor.find("net")!=string::npos)      code=code+"( 0 ) ";
-		    if (aor.find("host")!=string::npos)     code=code+"( 1 ) ";
-		    if (aor.find("protocol")!=string::npos) code=code+"( 2 ) ";
-		    if (aor.find("port")!=string::npos)     code=code+"( 3 ) ";
-		}
-		if (aor.find("prohibited")!=string::npos )
-                {
- 		    if (aor.find("net")!=string::npos)      code=code+"( 9 ) ";
-		    if (aor.find("host")!=string::npos)     code=code+"( 10 ) ";
-		}
-	    } else
-		code="return-icmp   ";
+                if (aor.find("net")!=string::npos)      code = code + "( 0 ) ";
+                if (aor.find("host")!=string::npos)     code = code + "( 1 ) ";
+                if (aor.find("protocol")!=string::npos) code = code + "( 2 ) ";
+                if (aor.find("port")!=string::npos)     code = code + "( 3 ) ";
+            }
+            if (aor.find("prohibited")!=string::npos )
+            {
+                if (aor.find("net")!=string::npos)      code = code + "( 9 ) ";
+                if (aor.find("host")!=string::npos)     code = code + "( 10 ) ";
+                if (aor.find("filter")!=string::npos)   code = code + "( 13 ) ";
+                if (aor.find("admin")!=string::npos)    code = code + "( 13 ) ";
+            }
+        } else
+        {
+            if ( aor.find("TCP")!=string::npos ) code = "return-rst   ";
+            else
+                code = "return-icmp   ";
+        }
 
-	    compiler->output << "block " << code;
-	}
+        compiler->output << "block " << code;
 	break;
+    }
+
     case PolicyRule::Scrub:
     {
         string version = compiler->fw->getStr("version");
@@ -137,9 +140,11 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
         }
         break;
     }
+
     case PolicyRule::Custom:
         compiler->output << ruleopt->getStr("custom_str") << " ";
         break;
+
     case PolicyRule::Branch:
     {
         RuleSet *ruleset = rule->getBranch();
@@ -156,31 +161,44 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
         }
         break;
     }
+
     default:
         compiler->abort(
-                rule, 
-                string("Unknown action ") + rule->getActionAsString());
+                rule, string("Unknown action ") + rule->getActionAsString());
     }
+
+    // #2367
+    //
+    // if (rule->getTagging())
+    // {
+    //     if (XMLTools::version_compare(version, "4.6")>=0)
+    //     {
+    //         compiler->output << "match ";
+    //     }else
+    //     {
+    //         compiler->output << "pass ";
+    //     }
+    //     break;
+    // }
 }
 
 void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
 {
     FWOptions *ruleopt =rule->getOptionsObject();
 
-    if (rule->getAction() == PolicyRule::Route)
+    if (rule->getRouting())
     {
 	string prefix = "pf";
-	if (compiler->myPlatformName()=="ipf") prefix="ipf";
+	if (compiler->myPlatformName()=="ipf") prefix = "ipf";
 
 	string ro = ruleopt->getStr(prefix+"_route_option");
         if (ruleopt->getBool("pf_fastroute") && ro != "none")
 	{
             compiler->abort(
-                
-                    rule, 
-                    "Cannot use fastroute and route methods in "
-                    "the same rule because they are mutually "
-                    "exclusive.");
+                rule, 
+                "Cannot use fastroute and route methods in "
+                "the same rule because they are mutually "
+                "exclusive.");
 	} else if (ruleopt->getBool("pf_fastroute") && ro == "none")
         {
             compiler->output << "fastroute ";
@@ -193,10 +211,9 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
             {
                 if (roif.empty())
                     compiler->abort(
-                        
-                            rule, 
-                            "Interface specification is required "
-                            "for action Route.");
+                        rule, 
+                        "Interface specification is required "
+                        "for action Route.");
 
                 if (ro == "route_through")
                     compiler->output << "route-to ";
@@ -206,10 +223,9 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                     compiler->output << "dup-to ";
                 else
                     compiler->abort(
-                        
-                            rule, 
-                            "Unknown option for rule action Route: '" + 
-                            ro + "'");
+                        rule, 
+                        "Unknown option for rule action Route: '" + 
+                        ro + "'");
             		
                 compiler->output << "{ ";
 
@@ -240,9 +256,7 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                             } catch (FWException &ex)
                             {
                                 compiler->abort(
-                                    
-                                        rule, 
-                                        "Illegal IP address for next hop");
+                                    rule, "Illegal IP address for next hop");
                             }
                             try
                             {
@@ -268,9 +282,7 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                             } catch (FWException &ex)
                             {
                                 compiler->abort(
-                                    
-                                        rule, 
-                                        "Illegal netmask for next hop");
+                                    rule, "Illegal netmask for next hop");
                             }
                         } else
                         {
@@ -281,9 +293,7 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                             } catch (FWException &ex)
                             {
                                 compiler->abort(
-                                    
-                                        rule, 
-                                        "Illegal IP address for next hop");
+                                    rule, "Illegal IP address for next hop");
                             }
                             route_member++;
                         }
@@ -292,26 +302,22 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                 if (route_member < 1)
                 {
                     compiler->abort(
-                        
-                            rule, 
-                            "No router specified rule action Route: '"+ 
-                            ro + "'");
+                        rule, 
+                        "No router specified rule action Route: '" +  ro + "'");
                 }
                 if (route_member >= 2 && (roload.empty() || roload == "none"))
                 {
                     compiler->abort(
-                        
-                            rule, 
-                            "More than one router specified without load balancing for rule action Route: '" + 
-                            ro + "'");
+                        rule, 
+                        "More than one router specified without load "
+                        "balancing for rule action Route: '" + ro + "'");
                 }
                 if (route_member == 1 && ((!roload.empty()) && roload != "none"))
                 {
                     compiler->abort(
-                        
-                            rule, 
-                            "Only one router specified with load balancing for rule action Route: '" + 
-                            ro + "'");
+                        rule, 
+                        "Only one router specified with load balancing "
+                        "for rule action Route: '" + ro + "'");
                 }
 
                 compiler->output << "} ";
@@ -336,8 +342,11 @@ void PolicyCompiler_pf::PrintRule::_printQueue(PolicyRule *rule)
 {
     FWOptions *ruleopt =rule->getOptionsObject();
 
-    if (rule->getAction() == PolicyRule::Classify)
-        compiler->output << "queue " << ruleopt->getStr("pf_classify_str") << " ";
+    if (rule->getClassification())
+    {
+        compiler->output << "queue ";
+        compiler->output << ruleopt->getStr("pf_classify_str") << " ";
+    }
 }
 
 void PolicyCompiler_pf::PrintRule::_printUser(PolicyRule *rule)
@@ -379,9 +388,8 @@ void PolicyCompiler_pf::PrintRule::_printUser(PolicyRule *rule)
 
 void PolicyCompiler_pf::PrintRule::_printTag(PolicyRule *rule)
 {
-    if (rule->getAction() == PolicyRule::Tag)
+    if (rule->getTagging())
         compiler->output << "tag " << rule->getTagValue() << " ";
-//        compiler->output << "tag " << ruleopt->getStr("tagvalue") << " ";
 }
 
 void PolicyCompiler_pf::PrintRule::_printDirection(PolicyRule *rule)
@@ -755,7 +763,7 @@ string PolicyCompiler_pf::PrintRule::_printTCPFlags(libfwbuilder::TCPService *sr
     return str;
 }
 
-void PolicyCompiler_pf::PrintRule::_printAddr(Address  *o,bool )
+void PolicyCompiler_pf::PrintRule::_printAddr(FWObject *o, bool )
 {
     MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
     if (atrt!=NULL)
@@ -770,10 +778,25 @@ void PolicyCompiler_pf::PrintRule::_printAddr(Address  *o,bool )
             compiler->output << "<" << o->getName() << "> ";
             return;
         }
+        if (atrt->getSubstitutionTypeName()==AttachedNetworks::TYPENAME)
+        {
+            compiler->output << atrt->getSourceName() << ":network";
+            return ;
+        }
+
         assert(atrt==NULL);
     }
 
-    const InetAddr *addr = o->getAddressPtr();
+    if (o->getBool("pf_table"))
+    {
+        compiler->output << "<" << o->getName() << "> ";
+        return;
+    }
+
+    Address *addr_obj = Address::cast(o);
+    assert(addr_obj!=NULL);
+
+    const InetAddr *addr = addr_obj->getAddressPtr();
     InetAddr mask;
 
     if (Interface::cast(o)!=NULL)
@@ -788,10 +811,10 @@ void PolicyCompiler_pf::PrintRule::_printAddr(Address  *o,bool )
 	mask = InetAddr(InetAddr::getAllOnes());
     } else
     {
-        mask = *(o->getNetmaskPtr());
+        mask = *(addr_obj->getNetmaskPtr());
     }
 
-    if (o->dimension()==1) 
+    if (addr_obj->dimension()==1) 
     {
 	mask = InetAddr(InetAddr::getAllOnes());
     }
@@ -819,9 +842,7 @@ void PolicyCompiler_pf::PrintRule::_printAddrList(FWObject  *grp,bool negflag)
         if (i!=grp->begin())  compiler->output << ", ";
         FWObject *o= *i;
         if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-        Address *s=Address::cast( o );
-        assert(s);
-        _printAddr(s , negflag);
+        _printAddr(o , negflag);
     }
     compiler->output << "} ";
 }
@@ -832,8 +853,6 @@ void PolicyCompiler_pf::PrintRule::_printSrcAddr(RuleElement  *rel)
     FWReference *oref = FWReference::cast(o);
     if (o && oref!=NULL) o=oref->getPointer();
 
-    Address *src= Address::cast(o);
-
     _printNegation(rel);
 
     if (o==NULL)
@@ -849,18 +868,12 @@ void PolicyCompiler_pf::PrintRule::_printSrcAddr(RuleElement  *rel)
         compiler->abort(rel->getParent(), errstr.str());
     }
 
-    if (rel->size()==1 && ! o->getBool("pf_table") )
+    if (rel->size()==1)
     {
-	_printAddr( src , rel->getNeg() );
+	_printAddr(o, rel->getNeg() );
     } else
     {
-        if (o->getBool("pf_table"))
-        {
-            compiler->output << "<" << o->getName() << "> ";
-        } else
-        {
-            _printAddrList(rel,rel->getNeg());
-        }
+        _printAddrList(rel,rel->getNeg());
     }
 
 }
@@ -871,8 +884,6 @@ void PolicyCompiler_pf::PrintRule::_printDstAddr(RuleElement  *rel)
     FWReference *oref = FWReference::cast(o);
     if (o && oref!=NULL) o=oref->getPointer();
 
-    Address *dst= Address::cast(o);
-
     _printNegation(rel);
 
     if (o==NULL)
@@ -888,18 +899,12 @@ void PolicyCompiler_pf::PrintRule::_printDstAddr(RuleElement  *rel)
         compiler->abort(rel->getParent(), errstr.str());
     }
 
-    if (rel->size()==1 && ! o->getBool("pf_table") )
+    if (rel->size()==1)
     {
-	_printAddr( dst , rel->getNeg());
+	_printAddr(o, rel->getNeg());
     } else
     {
-        if (o->getBool("pf_table"))
-        {
-            compiler->output << "<" << o->getName() << "> ";
-        } else
-        {
-            _printAddrList(rel,rel->getNeg());
-        }
+        _printAddrList(rel, rel->getNeg());
     }
 }
 
@@ -940,7 +945,8 @@ bool PolicyCompiler_pf::PrintRule::processNext()
 
     _printInterface(rule);
 
-    _printRouteOptions(rule);
+    if (XMLTools::version_compare(version, "4.7")<0)
+        _printRouteOptions(rule);
 
     _printAF(rule);
 
@@ -1171,6 +1177,9 @@ bool PolicyCompiler_pf::PrintRule::processNext()
     _printQueue(rule);
     _printLabel(rule);
 
+    if (XMLTools::version_compare(version, "4.7")>=0)
+        _printRouteOptions(rule);
+
     compiler->output << endl;
 
     return true;
